@@ -12,6 +12,56 @@
 
 using boost::graph_traits;
 
+/** Contig length property. */
+struct Length
+{
+	unsigned length;
+
+	bool operator==(const Length& o) const
+	{
+		return length == o.length;
+	}
+
+	Length& operator+=(const Length& o)
+	{
+		length += o.length;
+		return *this;
+	}
+
+	template <typename T>
+	Length& operator+=(const T& o)
+	{
+		assert((int)length + (int)o.distance > 0);
+		length += o.distance;
+		return *this;
+	}
+
+	friend std::ostream& operator<<(std::ostream& out,
+			const Length& o)
+	{
+		return out << "l=" << o.length;
+	}
+
+	friend std::istream& operator>>(std::istream& in, Length& o)
+	{
+		if (in >> std::ws && in.peek() == 'l')
+			return in >> expect("l =") >> o.length;
+		else
+			return in >> o.length;
+	}
+};
+
+static inline
+void put(vertex_length_t, Length& vp, unsigned length)
+{
+	vp.length = length;
+}
+
+static inline
+void put(vertex_coverage_t, Length&, unsigned)
+{
+}
+
 /** The length and coverage of a contig. */
 struct ContigProperties {
 	unsigned length;
@@ -63,6 +113,9 @@ struct ContigProperties {
 			if (in >> std::ws && in.peek() == 'C')
 				in >> expect("C =") >> o.coverage;
 			return in;
+		} else if (in.peek() == 'C') {
+			return in >> expect("C=") >> o.coverage
+				>> expect(", l=") >> o.length;
 		} else
 			return in >> o.length >> o.coverage;
 	}
@@ -104,15 +157,19 @@ struct Distance {
 
 	friend std::istream& operator>>(std::istream& in, Distance& o)
 	{
-		return in >> expect(" d =") >> o.distance;
+		in >> expect(" d = ");
+		if (in.peek() == '"')
+			return in >> expect("\"") >> o.distance >> expect("\"");
+		else
+			return in >> o.distance;
 	}
 };
 
 /** Add the specified distance (overlap) to the specified contig
  * length.
  */
-static inline ContigProperties& operator+=(
-		ContigProperties& a, const Distance& b)
+template <typename T>
+ContigProperties& operator+=(ContigProperties& a, const T& b)
 {
 	assert((int)a.length + (int)b.distance > 0);
 	a.length += b.distance;
@@ -143,7 +200,8 @@ get(edge_bundle_t, const Graph& g, ContigNode u, ContigNode v)
 		std::pair<edge_descriptor, bool> e = edge(u, v, g);
 		if (!e.second)
 			std::cerr << "error: no edge "
-				<< u << " -> " << v << '\n';
+				<< get(vertex_name, g, u) << " -> "
+				<< get(vertex_name, g, v) << '\n';
 		assert(e.second);
 		return g[e.first];
 	}
@@ -170,4 +228,13 @@ struct EdgeWeightMap {
 	const Graph& m_g;
 };
 
+/** Return the edge weight. */
+template <typename Graph>
+typename EdgeWeightMap<Graph>::value_type
+get(edge_weight_t, const Graph& g,
+		typename graph_traits<Graph>::edge_descriptor e)
+{
+	return EdgeWeightMap<Graph>(g)[e];
+}
+
 #endif
