@@ -29,16 +29,17 @@ static const char* zcatExec(const string& path)
 		startsWith(path, "https://") ? "wget -O-" :
 		startsWith(path, "ftp://") ? "wget -O-" :
 		endsWith(path, ".ar") ? "ar -p" :
-		endsWith(path, ".tar") ? "tar -xOf " :
-		endsWith(path, ".tar.Z") ? "tar -zxOf " :
-		endsWith(path, ".tar.gz") ? "tar -zxOf " :
-		endsWith(path, ".tar.bz2") ? "tar -jxOf " :
+		endsWith(path, ".tar") ? "tar -xOf" :
+		endsWith(path, ".tar.Z") ? "tar -zxOf" :
+		endsWith(path, ".tar.gz") ? "tar -zxOf" :
+		endsWith(path, ".tar.bz2") ? "tar -jxOf" :
 		endsWith(path, ".tar.xz") ?
-			"tar --use-compress-program=xzdec -xOf " :
+			"tar --use-compress-program=xzdec -xOf" :
 		endsWith(path, ".Z") ? "gunzip -c" :
 		endsWith(path, ".gz") ? "gunzip -c" :
 		endsWith(path, ".bz2") ? "bunzip2 -c" :
 		endsWith(path, ".xz") ? "xzdec -c" :
+		endsWith(path, ".zip") ? "unzip -p" :
 		endsWith(path, ".bam") ? "samtools view -h" :
 		endsWith(path, ".jf") ? "jellyfish dump" :
 		endsWith(path, ".jfq") ? "jellyfish qdump" :
@@ -91,7 +92,7 @@ static int uncompress(const char *path)
 			execlp(arg0, arg0, arg1, arg2, path, NULL);
 		// Calling perror after vfork is not allowed, but we're about
 		// to exit and an error message would be really helpful.
-		perror(zcat);
+		perror(arg0);
 		_exit(EXIT_FAILURE);
 	} else {
 		close(fd[1]);
