@@ -5,6 +5,7 @@
 #include "PMF.h"
 #include "SAM.h"
 #include "Uncompress.h"
+#include "Graph/Options.h" // for opt::k
 #include <algorithm>
 #include <cassert>
 #include <climits>
@@ -30,7 +31,7 @@ static const char VERSION_MESSAGE[] =
 PROGRAM " (" PACKAGE_NAME ") " VERSION "\n"
 "Written by Jared Simpson and Shaun Jackman.\n"
 "\n"
-"Copyright 2011 Canada's Michael Smith Genome Science Centre\n";
+"Copyright 2012 Canada's Michael Smith Genome Science Centre\n";
 
 static const char USAGE_MESSAGE[] =
 "Usage: " PROGRAM " [OPTION]... HIST [PAIR]\n"
@@ -38,12 +39,21 @@ static const char USAGE_MESSAGE[] =
 "  HIST  distribution of fragments size\n"
 "  PAIR  alignments between contigs\n"
 "\n"
-"  -k, --kmer=KMER_SIZE  k-mer size\n"
+"      --mind=N          minimum distance between contigs [-(k-1)]\n"
+"      --maxd=N          maximum distance between contigs\n"
+"      --fr              force the orientation to forward-reverse\n"
+"      --rf              force the orientation to reverse-forward\n"
+"  -k, --kmer=N          set --mind to -(k-1) bp\n"
+"  -l, --min-align=N     the minimal alignment size [1]\n"
 "  -n, --npairs=NPAIRS   minimum number of pairs\n"
 "  -s, --seed-length=L   minimum length of the seed contigs\n"
 "  -q, --min-mapq=N      ignore alignments with mapping quality\n"
-"                        less than this threshold [1]\n"
+"                        less than this threshold [10]\n"
 "  -o, --out=FILE        write result to FILE\n"
+"      --mle             use the MLE [default]\n"
+"                        (maximum likelihood estimator)\n"
+"      --mean            use the difference of the population mean\n"
+"                        and the sample mean\n"
 "      --dist            output graph in dist format [default]\n"
 "      --dot             output graph in dot format\n"
 "  -j, --threads=N       use N parallel threads [1]\n"
@@ -53,31 +63,55 @@ static const char USAGE_MESSAGE[] =
 "\n"
 "Report bugs to <" PACKAGE_BUGREPORT ">.\n";
 
+/** Which estimator to use. See opt::method. */
+enum { MLE, MEAN };
+
 namespace opt {
-	unsigned k; // used by MLE
+	unsigned k; // used by Estimate.h
 
 	/** Output graph format. */
 	int format = DIST;
 
+	/** The minimal alignment size. */
+	static int minAlign = 1;
+
+	/** Minimum distance between contigs. */
+	static int minDist = numeric_limits<int>::min();
+
+	/** Maximum distance between contigs. */
+	static int maxDist = numeric_limits<int>::max();
+
 	static unsigned seedLen;
 	static unsigned npairs;
-	static unsigned minMapQ = 1;
+	static unsigned minMapQ = 10;
 
 	/** Reverse-forward mate pair orientation. */
-	static bool rf = false;
+	static int rf = -1;
+
+	/** Which estimator to use. */
+	static int method = MLE;
 
 	static int verbose;
 	static string out;
 	static int threads = 1;
 }
 
-static const char shortopts[] = "j:k:n:o:q:s:v";
+static const char shortopts[] = "j:k:l:n:o:q:s:v";
 
-enum { OPT_HELP = 1, OPT_VERSION };
+enum { OPT_HELP = 1, OPT_VERSION,
+	OPT_MIND, OPT_MAXD, OPT_FR, OPT_RF
+};
 
 static const struct option longopts[] = {
 	{ "dist",        no_argument,       &opt::format, DIST, },
 	{ "dot",         no_argument,       &opt::format, DOT, },
+	{ "fr",          no_argument,       &opt::rf, false },
+	{ "rf",          no_argument,       &opt::rf, true },
+	{ "min-align",   required_argument, NULL, 'l' },
+	{ "mind",        required_argument, NULL, OPT_MIND },
+	{ "maxd",        required_argument, NULL, OPT_MAXD },
+	{ "mle",         no_argument,       &opt::method, MLE },
+	{ "mean",        no_argument,       &opt::method, MEAN },
 	{ "kmer",        required_argument, NULL, 'k' },
 	{ "npairs",      required_argument, NULL, 'n' },
 	{ "out",         required_argument, NULL, 'o' },
@@ -90,7 +124,32 @@ static const struct option longopts[] = {
 	{ NULL, 0, NULL, 0 }
 };
 
-typedef vector<SAMRecord> AlignPairVec;
+/** A collection of aligned read pairs. */
+typedef vector<SAMRecord> Pairs;
+
+/** Estimate the distance between two contigs using the difference of
+ * the population mean and the sample mean.
+ * @param numPairs [out] the number of pairs that agree with the
+ * expected distribution
+ * @return the estimated distance
+ */
+static int estimateDistanceUsingMean(
+		const std::vector<int>& samples, const PMF& pmf,
+		unsigned& numPairs)
+{
+	Histogram h(samples.begin(), samples.end());
+	int d = pmf.mean() - h.mean();
+
+	// Count the number of samples that agree with the distribution.
+	unsigned n = 0;
+	for (Histogram::const_iterator it = h.begin();
+			it != h.end(); ++it)
+		if (pmf[it->first + d] > pmf.minProbability())
+			n += it->second;
+
+	numPairs = n;
+	return d;
+}
 
 /** Estimate the distance between two contigs.
  * @param numPairs [out] the number of pairs that agree with the
@@ -98,7 +157,7 @@ typedef vector<SAMRecord> AlignPairVec;
  * @return the estimated distance
  */
 static int estimateDistance(unsigned len0, unsigned len1,
-		const AlignPairVec& pairs, const PMF& pmf,
+		const Pairs& pairs, const PMF& pmf,
 		unsigned& numPairs)
 {
 	// The provisional fragment sizes are calculated as if the contigs
@@ -106,7 +165,7 @@ static int estimateDistance(unsigned len0, unsigned len1,
 	typedef vector<pair<int, int> > Fragments;
 	Fragments fragments;
 	fragments.reserve(pairs.size());
-	for (AlignPairVec::const_iterator it = pairs.begin();
+	for (Pairs::const_iterator it = pairs.begin();
 			it != pairs.end(); ++it) {
 		int a0 = it->targetAtQueryStart();
 		int a1 = it->mateTargetAtQueryStart();
@@ -130,17 +189,39 @@ static int estimateDistance(unsigned len0, unsigned len1,
 	vector<int> fragmentSizes;
 	fragmentSizes.reserve(fragments.size());
 	for (Fragments::const_iterator it = fragments.begin();
-			it != fragments.end(); ++it)
-		fragmentSizes.push_back(it->second - it->first);
+			it != fragments.end(); ++it) {
+		int x = it->second - it->first;
+		if (!opt::rf && x <= 2 * int(opt::minAlign - 1)) {
+			cerr << PROGRAM ": error: The observed fragment of size "
+				<< x << " bp is shorter than 2*l "
+				"(l=" << opt::minAlign << "). "
+				"Decrease l to " << x / 2 << ".\n";
+			exit(EXIT_FAILURE);
+		}
+		fragmentSizes.push_back(x);
+	}
 
-	return maximumLikelihoodEstimate(-opt::k+1, pmf.maxValue(),
-			fragmentSizes, pmf, len0, len1, opt::rf, numPairs);
+	switch (opt::method) {
+	  case MLE:
+		// Use the maximum likelihood estimator.
+		return maximumLikelihoodEstimate(opt::minAlign,
+				opt::minDist, opt::maxDist,
+				fragmentSizes, pmf, len0, len1, opt::rf, numPairs);
+	  case MEAN:
+		// Use the difference of the population mean
+		// and the sample mean.
+		return estimateDistanceUsingMean(
+				fragmentSizes, pmf, numPairs);
+	  default:
+		assert(false);
+		abort();
+	}
 }
 
 static void writeEstimate(ostream& out,
 		const ContigNode& id0, const ContigNode& id1,
 		unsigned len0, unsigned len1,
-		const AlignPairVec& pairs, const PMF& pmf)
+		const Pairs& pairs, const PMF& pmf)
 {
 	if (pairs.size() < opt::npairs)
 		return;
@@ -150,17 +231,17 @@ static void writeEstimate(ostream& out,
 			pairs, pmf, est.numPairs);
 	est.stdDev = pmf.getSampleStdDev(est.numPairs);
 
+	std::pair<ContigNode, ContigNode> e(id0, id1 ^ id0.sense());
 	if (est.numPairs >= opt::npairs) {
 		if (opt::format == DOT) {
-			ContigNode v = id1 ^ id0.sense();
 #pragma omp critical(out)
-			out << '"' << id0 << "\" -> \"" << v << "\""
-				" [" << est << "]\n";
+			out << get(g_contigNames, e) << " [" << est << "]\n";
 		} else
-			out << ' ' << id1 << ',' << est;
+			out << ' ' << get(g_contigNames, id1) << ',' << est;
 	} else if (opt::verbose > 1) {
 #pragma omp critical(cerr)
-		cerr << "warning: " << id0 << ',' << id1 << ' '
+		cerr << "warning: " << get(g_contigNames, e)
+			<< " [d=" << est.distance << "] "
 			<< est.numPairs << " of " << pairs.size()
 			<< " pairs fit the expected distribution\n";
 	}
@@ -172,7 +253,7 @@ static void writeEstimates(ostream& out,
 		const vector<unsigned>& lengthVec, const PMF& pmf)
 {
 	assert(!pairs.empty());
-	ContigID id0(pairs.front().rname);
+	ContigID id0(get(g_contigNames, pairs.front().rname));
 	assert(id0 < lengthVec.size());
 	unsigned len0 = lengthVec[id0];
 	if (len0 < opt::seedLen)
@@ -182,19 +263,20 @@ static void writeEstimates(ostream& out,
 	if (opt::format == DIST)
 		ss << pairs.front().rname;
 
-	typedef map<ContigNode, AlignPairVec> Pairs;
-	Pairs dataMap[2];
-	for (AlignPairVec::const_iterator it = pairs.begin();
+	typedef map<ContigNode, Pairs> PairsMap;
+	PairsMap dataMap[2];
+	for (Pairs::const_iterator it = pairs.begin();
 			it != pairs.end(); ++it)
-		dataMap[it->isReverse()][ContigNode(it->mrnm,
-				it->isReverse() == it->isMateReverse())]
+		dataMap[it->isReverse()][find_vertex(
+				it->mrnm, it->isReverse() == it->isMateReverse(),
+				g_contigNames)]
 			.push_back(*it);
 
 	for (int sense0 = false; sense0 <= true; sense0++) {
 		if (opt::format == DIST && sense0)
 			ss << " ;";
-		const Pairs& x = dataMap[sense0 ^ opt::rf];
-		for (Pairs::const_iterator it = x.begin();
+		const PairsMap& x = dataMap[sense0 ^ opt::rf];
+		for (PairsMap::const_iterator it = x.begin();
 				it != x.end(); ++it)
 			writeEstimate(opt::format == DOT ? out : ss,
 					ContigNode(id0, sense0), it->first,
@@ -229,7 +311,7 @@ static void readContigLengths(istream& in, vector<unsigned>& lengths)
 {
 	assert(in);
 	assert(lengths.empty());
-	assert(ContigID::empty());
+	assert(g_contigNames.empty());
 	for (string line; in.peek() == '@' && getline(in, line);) {
 		istringstream ss(line);
 		string type;
@@ -242,8 +324,7 @@ static void readContigLengths(istream& in, vector<unsigned>& lengths)
 		ss >> expect(" SN:") >> s >> expect(" LN:") >> len;
 		assert(ss);
 
-		ContigID id = ContigID::insert(s);
-		assert(id == lengths.size());
+		put(g_contigNames, lengths.size(), s);
 		lengths.push_back(len);
 	}
 	if (lengths.empty()) {
@@ -280,7 +361,8 @@ static void readPairs(It& it, const It& last, vector<SAMRecord>& out)
 
 	// Check that the input is sorted.
 	if (it != last && !out.empty()
-			&& ContigID(it->rname) < ContigID(out.front().rname)) {
+			&& get(g_contigNames, it->rname)
+				< get(g_contigNames, out.front().rname)) {
 		cerr << "error: input must be sorted: saw `"
 			<< out.front().rname << "' before `"
 			<< it->rname << "'\n";
@@ -296,6 +378,15 @@ int main(int argc, char** argv)
 		istringstream arg(optarg != NULL ? optarg : "");
 		switch (c) {
 			case '?': die = true; break;
+			case OPT_MIND:
+				arg >> opt::minDist;
+				break;
+			case OPT_MAXD:
+				arg >> opt::maxDist;
+				break;
+			case 'l':
+				arg >> opt::minAlign;
+				break;
 			case 'j': arg >> opt::threads; break;
 			case 'k': arg >> opt::k; break;
 			case 'n': arg >> opt::npairs; break;
@@ -310,6 +401,11 @@ int main(int argc, char** argv)
 				cout << VERSION_MESSAGE;
 				exit(EXIT_SUCCESS);
 		}
+		if (optarg != NULL && !arg.eof()) {
+			cerr << PROGRAM ": invalid option: `-"
+				<< (char)c << optarg << "'\n";
+			exit(EXIT_FAILURE);
+		}
 	}
 
 	if (opt::k <= 0) {
@@ -345,6 +441,8 @@ int main(int argc, char** argv)
 		cerr << "warning: the seed-length should be at least twice k:"
 			" k=" << opt::k << ", s=" << opt::seedLen << '\n';
 
+	assert(opt::minAlign > 0);
+
 #if _OPENMP
 	if (opt::threads > 0)
 		omp_set_num_threads(opt::threads);
@@ -372,30 +470,41 @@ int main(int argc, char** argv)
 			"s=" << opt::seedLen << " "
 			"n=" << opt::npairs << "]\n";
 
-	// Read the contig lengths.
-	vector<unsigned> contigLens;
-	readContigLengths(in, contigLens);
-	ContigID::lock();
+	// The fragment size histogram may not be written out until after
+	// the alignments complete. Wait for the alignments to complete.
+	in.peek();
 
 	// Read the fragment size distribution.
 	Histogram distanceHist = loadHist(distanceCountFile);
 	unsigned numRF = distanceHist.count(INT_MIN, 0);
 	unsigned numFR = distanceHist.count(1, INT_MAX);
 	unsigned numTotal = distanceHist.size();
-	if (opt::verbose > 0)
+	bool libRF = numFR < numRF;
+	if (opt::verbose > 0) {
 		cerr << "Mate orientation FR: " << numFR << setprecision(3)
 			<< " (" << (float)100*numFR/numTotal << "%)"
 			<< " RF: " << numRF << setprecision(3)
-			<< " (" << (float)100*numRF/numTotal << "%)"
-			<< endl;
-	if (numFR < numRF) {
-		cerr << "The mate pairs of this library are oriented "
-			"reverse-forward (RF)." << endl;
-		opt::rf = true;
-		distanceHist = distanceHist.negate();
+			<< " (" << (float)100*numRF/numTotal << "%)\n"
+			<< "The library " << distanceCountFile << " is oriented "
+			<< (libRF
+					? "reverse-forward (RF)" : "forward-reverse (FR)")
+			<< ".\n";
 	}
 
+	// Determine the orientation of the library.
+	if (opt::rf == -1)
+		opt::rf = libRF;
+	if (opt::rf)
+		distanceHist = distanceHist.negate();
+	if (opt::rf != libRF)
+		cerr << "warning: The orientation is forced to "
+			<< (opt::rf
+					? "reverse-forward (RF)" : "forward-reverse (FR)")
+			<< " which differs from the detected orientation.\n";
+
 	distanceHist.eraseNegative();
+	distanceHist.removeNoise();
+	distanceHist.removeOutliers();
 	Histogram h = distanceHist.trimFraction(0.0001);
 	if (opt::verbose > 0)
 		cerr << "Stats mean: " << setprecision(4) << h.mean() << " "
@@ -406,6 +515,20 @@ int main(int argc, char** argv)
 			<< h.barplot() << endl;
 	PMF pmf(h);
 
+	if (opt::minDist == numeric_limits<int>::min())
+		opt::minDist = -opt::k + 1;
+	if (opt::maxDist == numeric_limits<int>::max())
+		opt::maxDist = pmf.maxValue();
+	if (opt::verbose > 0)
+		cerr << "Minimum and maximum distance are set to "
+			<< opt::minDist << " and " << opt::maxDist << " bp.\n";
+	assert(opt::minDist < opt::maxDist);
+
+	// Read the contig lengths.
+	vector<unsigned> contigLens;
+	readContigLengths(in, contigLens);
+	g_contigNames.lock();
+
 	// Estimate the distances between contigs.
 	istream_iterator<SAMRecord> it(in), last;
 	assert(in);
