@@ -40,7 +40,7 @@ static const char VERSION_MESSAGE[] =
 PROGRAM " (" PACKAGE_NAME ") " VERSION "\n"
 "Written by Tony Raymond.\n"
 "\n"
-"Copyright 2011 Canada's Michael Smith Genome Science Centre\n";
+"Copyright 2012 Canada's Michael Smith Genome Science Centre\n";
 
 static const char USAGE_MESSAGE[] =
 "Usage: " PROGRAM " [OPTION]... ADJ\n"
@@ -60,6 +60,11 @@ static const char USAGE_MESSAGE[] =
 "      --no-assemble     disable assembling of paths [default]\n"
 "  -g, --graph=FILE      write the contig adjacency graph to FILE\n"
 "  -i, --ignore=FILE     ignore contigs seen in FILE\n"
+"      --adj             output the graph in adj format [default]\n"
+"      --asqg            output the graph in asqg format\n"
+"      --dot             output the graph in dot format\n"
+"      --dot-meancov     same as above but give the mean coverage\n"
+"      --sam             output the graph in SAM format\n"
 "  -v, --verbose         display verbose output\n"
 "      --help            display this help and exit\n"
 "      --version         output version information and exit\n"
@@ -93,7 +98,8 @@ namespace opt {
 	/** The minimum overlap allowed between two contigs. */
 	static int minOverlap = 10;
 
-	int format; // used by ContigProperties
+	/** Output graph format. */
+	int format = ADJ; // used by ContigProperties
 }
 
 static const char shortopts[] = "g:i:k:l:m:t:T:v";
@@ -101,6 +107,11 @@ static const char shortopts[] = "g:i:k:l:m:t:T:v";
 enum { OPT_HELP = 1, OPT_VERSION };
 
 static const struct option longopts[] = {
+	{ "adj",           no_argument,       &opt::format, ADJ },
+	{ "asqg",          no_argument,       &opt::format, ASQG },
+	{ "dot",           no_argument,       &opt::format, DOT },
+	{ "dot-meancov",   no_argument,       &opt::format, DOT_MEANCOV },
+	{ "sam",           no_argument,       &opt::format, SAM },
 	{ "graph",         required_argument, NULL, 'g' },
 	{ "ignore",        required_argument, NULL, 'i' },
 	{ "kmer",          required_argument, NULL, 'k' },
@@ -141,6 +152,7 @@ static bool removable(const Graph* pg, vertex_descriptor v)
 	typedef graph_traits<Graph> GTraits;
 	typedef GTraits::out_edge_iterator OEit;
 	typedef GTraits::in_edge_iterator IEit;
+	typedef GTraits::vertex_descriptor V;
 
 	const Graph& g = *pg;
 
@@ -167,7 +179,9 @@ static bool removable(const Graph* pg, vertex_descriptor v)
 	OEit oei0, oei1;
 	tie(oei0, oei1) = out_edges(v, g);
 	for (OEit vw = oei0; vw != oei1; ++vw) {
-		if (v == target(*vw, g) || ~v == target(*vw, g)) {
+		V w = target(*vw, g);
+		V vc = get(vertex_complement, g, v);
+		if (v == w || vc == w) {
 			g_count.self_adj++;
 			return false;
 		}
@@ -198,7 +212,7 @@ static bool removable(const Graph* pg, vertex_descriptor v)
 struct EdgeInfo {
 	vertex_descriptor u;
 	vertex_descriptor w;
-	Distance ep;
+	edge_bundle_type<Graph>::type ep;
 
 	EdgeInfo(vertex_descriptor u, vertex_descriptor w, int ep)
 		: u(u), w(w), ep(ep) {}
@@ -243,8 +257,8 @@ static bool findNewEdges(const Graph& g, vertex_descriptor v,
 			}
 		}
 	}
-	for (vector<V>::const_iterator it = marked.begin(); it != marked.end();
-			it++)
+	for (vector<V>::const_iterator it = marked.begin();
+			it != marked.end(); it++)
 		markedContigs[get(vertex_index, g, *it)] = true;
 	return true;
 }
@@ -296,7 +310,7 @@ static void removeContigs(Graph& g, vector<vertex_descriptor>& sc)
 
 		clear_vertex(v, g);
 		remove_vertex(v, g);
-		g_removed.push_back(v);
+		g_removed.push_back(get(vertex_contig_index, g, v));
 		g_count.removed++;
 	}
 	sc.swap(out);
@@ -305,12 +319,14 @@ static void removeContigs(Graph& g, vector<vertex_descriptor>& sc)
 /** Return the value of the bit at the specified index. */
 struct Marked : unary_function<vertex_descriptor, bool> {
 	typedef vector<bool> Data;
-	Marked(const Data& data) : m_data(data) { }
+	Marked(const Graph& g, const Data& data)
+		: m_g(g), m_data(data) { }
 	bool operator()(vertex_descriptor u) const
 	{
-		return m_data[ContigID(u)];
+		return m_data[get(vertex_contig_index, m_g, u)];
 	}
   private:
+	const Graph& m_g;
 	const Data& m_data;
 };
 
@@ -323,7 +339,7 @@ static void findShortContigs(const Graph& g, const vector<bool>& seen,
 	Vit first, second;
 	tie(first, second) = vertices(g);
 	copy_if(first, second, back_inserter(sc),
-			!bind(Marked(seen), _1) && bind(removable, &g, _1));
+			!bind(Marked(g, seen), _1) && bind(removable, &g, _1));
 }
 
 /** Functor used for sorting contigs based on degree, then size,
@@ -359,21 +375,22 @@ struct ShorterThanX : unary_function<vertex_descriptor, bool> {
 	bool operator()(vertex_descriptor y) const
 	{
 		return g[y].length < x && !get(vertex_removed, g, y)
-			&& !seen[ContigID(y)];
+			&& !seen[get(vertex_contig_index, g, y)];
 	}
 };
 
 static void removeShims(Graph& g, const vector<bool>& seen)
 {
 	if (opt::verbose > 0)
-		cerr << "Removing shims from graph...\n";
+		cerr << "Removing shim contigs from the graph...\n";
 	vector<vertex_descriptor> shortContigs;
 	findShortContigs(g, seen, shortContigs);
 	for (unsigned i = 0; !shortContigs.empty(); ++i) {
 		if (opt::verbose > 0)
-			cerr << "Starting pass " << i << ": There are "
-				<< shortContigs.size() << " contigs to check.\n";
-		sort(shortContigs.begin(), shortContigs.end(), sortContigs(g));
+			cerr << "Pass " << i + 1 << ": Checking "
+				<< shortContigs.size() << " contigs.\n";
+		sort(shortContigs.begin(), shortContigs.end(),
+				sortContigs(g));
 		removeContigs(g, shortContigs);
 	}
 	if (opt::verbose > 0) {
@@ -398,7 +415,8 @@ static void removeShortContigs(Graph& g, const vector<bool>& seen)
 	copy_if(first, second, back_inserter(sc),
 			ShorterThanX(g, seen, opt::minLen));
 	remove_vertex_if(g, sc.begin(), sc.end(), True<V>());
-	copy(sc.begin(), sc.end(), back_inserter(g_removed));
+	transform(sc.begin(), sc.end(), back_inserter(g_removed),
+			mem_fun_ref(&ContigNode::contigIndex));
 	if (opt::verbose > 0)
 		cerr << "Removed " << sc.size()/2 << " short contigs.\n";
 }
@@ -419,21 +437,44 @@ int main(int argc, char** argv)
 					shortopts, longopts, NULL)) != -1;) {
 		istringstream arg(optarg != NULL ? optarg : "");
 		switch (c) {
-			case '?': die = true; break;
-			case 'l': arg >> opt::minLen; assert(arg.eof()); break;
-			case 'm': arg >> opt::minOverlap; assert(arg.eof()); break;
-			case 'g': arg >> opt::graphPath; assert(arg.eof()); break;
-			case 'i': arg >> opt::ignorePath; assert(arg.eof()); break;
-			case 'k': arg >> opt::k; assert(arg.eof()); break;
-			case 'T': arg >> opt::minIslandLen; assert(arg.eof()); break;
-			case 't': arg >> opt::minTipLen; assert(arg.eof()); break;
-			case 'v': opt::verbose++; break;
-			case OPT_HELP:
-				cout << USAGE_MESSAGE;
-				exit(EXIT_SUCCESS);
-			case OPT_VERSION:
-				cout << VERSION_MESSAGE;
-				exit(EXIT_SUCCESS);
+		  case '?':
+			die = true;
+			break;
+		  case 'l':
+			arg >> opt::minLen;
+			break;
+		  case 'm':
+			arg >> opt::minOverlap;
+			break;
+		  case 'g':
+			arg >> opt::graphPath;
+			break;
+		  case 'i':
+			arg >> opt::ignorePath;
+			break;
+		  case 'k':
+			arg >> opt::k;
+			break;
+		  case 'T':
+			arg >> opt::minIslandLen;
+			break;
+		  case 't':
+			arg >> opt::minTipLen;
+			break;
+		  case 'v':
+			opt::verbose++;
+			break;
+		  case OPT_HELP:
+			cout << USAGE_MESSAGE;
+			exit(EXIT_SUCCESS);
+		  case OPT_VERSION:
+			cout << VERSION_MESSAGE;
+			exit(EXIT_SUCCESS);
+		}
+		if (optarg != NULL && !arg.eof()) {
+			cerr << PROGRAM ": invalid option: `-"
+				<< (char)c << optarg << "'\n";
+			exit(EXIT_FAILURE);
 		}
 	}
 
@@ -526,17 +567,23 @@ int main(int argc, char** argv)
 	sort(g_removed.begin(), g_removed.end());
 	g_removed.erase(unique(g_removed.begin(), g_removed.end()),
 			g_removed.end());
-	copy(g_removed.begin(), g_removed.end(),
-			ostream_iterator<ContigID>(cout, "\n"));
+	for (vector<ContigID>::const_iterator it = g_removed.begin();
+			it != g_removed.end(); ++it)
+		cout << get(g_contigNames, *it) << '\n';
 
 	// Assemble unambiguous paths.
 	if (opt::assemble) {
+		size_t numContigs = num_vertices(g) / 2;
 		typedef vector<ContigPath> ContigPaths;
 		ContigPaths paths;
 		assemble(g, back_inserter(paths));
 		for (ContigPaths::const_iterator it = paths.begin();
-				it != paths.end(); ++it)
-			cout << ContigID::create() << '\t' << *it << '\n';
+				it != paths.end(); ++it) {
+			ContigNode u(numContigs + it - paths.begin(), false);
+			string name = createContigName();
+			put(vertex_name, g, u, name);
+			cout << name << '\t' << *it << '\n';
+		}
 	}
 
 	// Output the updated adjacency graph.
