@@ -10,7 +10,9 @@
 #include <cstdlib>
 #include <istream>
 #include <ostream>
+#include <utility>
 
+using namespace std::rel_ops;
 using boost::graph_traits;
 
 template <typename EdgeProp>
@@ -42,14 +44,15 @@ std::ostream& write_adj(std::ostream& out, const Graph& g)
  		if (get(vertex_removed, g, *u))
 			continue;
 		if (!sense)
-			out << ContigID(*u) << get(vertex_bundle, g, *u);
+			out << get(vertex_contig_name, g, *u)
+				<< get(vertex_bundle, g, *u);
 		out << "\t;";
 		std::pair<out_edge_iterator, out_edge_iterator>
 			adj = out_edges(*u, g);
 		for (out_edge_iterator e = adj.first; e != adj.second; ++e) {
 			vertex_descriptor v = target(*e, g);
 			assert(!get(vertex_removed, g, v));
-			out << ' ' << (v ^ sense);
+			out << ' ' << get(vertex_name, g, v ^ sense);
 			write_edge_prop(out, get(edge_bundle, g, e));
 		}
 		if (sense)
@@ -67,10 +70,10 @@ std::istream& readDistEdges(std::istream& in, ContigGraph<Graph>& g,
 	typedef typename graph_traits<Graph>::vertex_descriptor V;
 	typedef typename graph_traits<Graph>::edge_descriptor E;
 	typedef typename Graph::edge_property_type EP;
-	for (std::string id; getline(in >> std::ws, id, ',');) {
-		assert(!id.empty());
-		V v(id);
-		v = v ^ u.sense();
+	for (std::string vname; getline(in >> std::ws, vname, ',');) {
+		assert(!vname.empty());
+		V v = find_vertex(vname, g);
+		v = v ^ get(vertex_sense, g, u);
 		EP ep;
 		in >> ep;
 		assert(in);
@@ -108,7 +111,8 @@ std::istream& read_adj(std::istream& in, ContigGraph<Graph>& g,
 	std::string line;
 	getline(in, line);
 	assert(in);
-	unsigned numSemicolons = std::count(line.begin(), line.end(), ';');
+	unsigned numSemicolons
+		= std::count(line.begin(), line.end(), ';');
 	if (numSemicolons > 2) {
 		std::cerr << "error: expected 0, 1 or 2 semicolons and saw "
 			<< numSemicolons << '\n';
@@ -139,7 +143,7 @@ std::istream& read_adj(std::istream& in, ContigGraph<Graph>& g,
 		assert(in.eof());
 	}
 	assert(num_vertices(g) > 0);
-	ContigID::lock();
+	g_contigNames.lock();
 
 	if (faiFormat)
 		return in;
@@ -148,10 +152,10 @@ std::istream& read_adj(std::istream& in, ContigGraph<Graph>& g,
 	in.clear();
 	in.seekg(std::ios_base::beg);
 	assert(in);
-	for (ContigID id; in >> id;) {
+	for (std::string name; in >> name;) {
 		if (adjFormat)
 			in >> Ignore(';');
-		vertex_descriptor u(id, false);
+		vertex_descriptor u = find_vertex(name, false, g);
 		for (int sense = false; sense <= true; ++sense) {
 			std::string s;
 			getline(in, s, !sense ? ';' : '\n');
@@ -160,7 +164,8 @@ std::istream& read_adj(std::istream& in, ContigGraph<Graph>& g,
 			if (!adjFormat) {
 				readDistEdges(ss, g, u ^ sense, betterEP);
 			} else
-			for (vertex_descriptor v; ss >> v >> std::ws;) {
+			for (std::string vname; ss >> vname >> std::ws;) {
+				vertex_descriptor v = find_vertex(vname, g);
 				assert(!edge(u ^ sense, v ^ sense, g).second);
 				if (ss.peek() == '[') {
 					ss.get();
