@@ -2,7 +2,9 @@
 #define CONTIGGRAPHALGORITHMS_H 1
 
 #include "Algorithms.h"
+#include "ContigGraph.h"
 #include "ContigNode.h"
+#include "Estimate.h" // for BetterDistanceEst
 #include "Functional.h"
 #include "Iterator.h"
 #include <boost/graph/graph_traits.hpp>
@@ -23,7 +25,8 @@ struct IsPalindrome : std::unary_function<
 	bool operator()(
 			typename graph_traits<Graph>::edge_descriptor e) const
 	{
-		return source(e, m_g) == ~target(e, m_g);
+		return source(e, m_g)
+			== get(vertex_complement, m_g, target(e, m_g));
 	}
   private:
 	const Graph& m_g;
@@ -43,7 +46,7 @@ template<typename Graph>
 bool contiguous_in(const Graph& g,
 		typename graph_traits<Graph>::vertex_descriptor u)
 {
-	return contiguous_out(g, ~u);
+	return contiguous_out(g, get(vertex_complement, g, u));
 }
 
 /** Add the outgoing edges of vertex u to vertex uout. */
@@ -64,7 +67,8 @@ void copy_out_edges(Graph &g,
 	edge_property_type palindrome_ep;
 	for (out_edge_iterator e = edges.first; e != edges.second; ++e) {
 		vertex_descriptor v = target(*e, g);
-		if (~v == u) {
+		vertex_descriptor vc = get(vertex_complement, g, v);
+		if (vc == u) {
 			// When ~v == u, adding the edge (~v,~u), which is (u,~u),
 			// would invalidate our iterator. Add the edge after this
 			// loop completes.
@@ -74,8 +78,10 @@ void copy_out_edges(Graph &g,
 			g.add_edge(uout, v, g[*e]);
 	}
 	if (palindrome) {
-		g.add_edge(uout, ~u, palindrome_ep);
-		g.add_edge(uout, ~uout, palindrome_ep);
+		vertex_descriptor uc = get(vertex_complement, g, u);
+		vertex_descriptor uoutc = get(vertex_complement, g, uout);
+		g.add_edge(uout, uc, palindrome_ep);
+		g.add_edge(uout, uoutc, palindrome_ep);
 	}
 }
 
@@ -85,7 +91,9 @@ void copy_in_edges(Graph& g,
 		typename Graph::vertex_descriptor u,
 		typename Graph::vertex_descriptor v)
 {
-	copy_out_edges(g, ~u, ~v);
+	copy_out_edges(g,
+			get(vertex_complement, g, u),
+			get(vertex_complement, g, v));
 }
 
 /** Assemble a path of unambigous out edges starting at vertex u.
@@ -174,7 +182,8 @@ typename vertex_property<Graph>::type addProp(const Graph& g,
  * Remove the vertices [first, last).
  */
 template<typename Graph, typename It>
-void merge(Graph& g, It first, It last)
+typename graph_traits<Graph>::vertex_descriptor
+merge(Graph& g, It first, It last)
 {
 	typedef typename graph_traits<Graph>::vertex_descriptor
 		vertex_descriptor;
@@ -182,6 +191,7 @@ void merge(Graph& g, It first, It last)
 	vertex_descriptor u = add_vertex(addProp(g, first, last), g);
 	copy_in_edges(g, *first, u);
 	copy_out_edges(g, *(last - 1), u);
+	return u;
 }
 
 /** Assemble unambiguous paths. Write the paths to out.
@@ -252,7 +262,8 @@ OutputIt pruneTips_if(Graph& g, OutputIt result, Pred p)
 
 	/** Remove the tips. */
 	remove_vertex_if(g, tips.begin(), tips.end(), True<V>());
-	copy(tips.begin(), tips.end(), result);
+	std::transform(tips.begin(), tips.end(), result,
+			std::mem_fun_ref(&ContigNode::contigIndex));
 
 	return result;
 }
@@ -287,7 +298,7 @@ OutputIt removeIslands_if(Graph& g, OutputIt result, Pred p)
 		if (get(vertex_removed, g, u))
 			continue;
 		if (p(u) && in_degree(u, g) == 0 && out_degree(u, g) == 0) {
-			*result++ = u;
+			*result++ = get(vertex_contig_index, g, u);
 			clear_vertex(u, g);
 			remove_vertex(u, g);
 		}
@@ -295,4 +306,35 @@ OutputIt removeIslands_if(Graph& g, OutputIt result, Pred p)
 	return result;
 }
 
+/** Add missing complementary edges. */
+template <typename DG>
+size_t addComplementaryEdges(ContigGraph<DG>& g)
+{
+	typedef ContigGraph<DG> Graph;
+	typedef graph_traits<Graph> GTraits;
+	typedef typename GTraits::edge_descriptor E;
+	typedef typename GTraits::edge_iterator Eit;
+	typedef typename GTraits::vertex_descriptor V;
+
+	std::pair<Eit, Eit> erange = edges(g);
+	size_t numAdded = 0;
+	for (Eit eit = erange.first; eit != erange.second; ++eit) {
+		E e = *eit;
+		V u = source(e, g), v = target(e, g);
+		V uc = get(vertex_complement, g, u);
+		V vc = get(vertex_complement, g, v);
+		E f;
+		bool found;
+		tie(f, found) = edge(vc, uc, g);
+		if (!found) {
+			add_edge(vc, uc, g[e], static_cast<DG&>(g));
+			numAdded++;
+		} else if (g[e] != g[f]) {
+			// The edge properties do not agree. Select the better.
+			g[e] = g[f] = BetterDistanceEst()(g[e], g[f]);
+		}
+	}
+	return numAdded;
+}
+
 #endif
