@@ -5,6 +5,7 @@
 #include "IOUtil.h"
 #include "MemoryUtil.h"
 #include "StringUtil.h"
+#include "Uncompress.h"
 #include <algorithm>
 #include <cctype> // for toupper
 #include <cstdlib>
@@ -12,7 +13,6 @@
 #include <getopt.h>
 #include <iostream>
 #include <iterator>
-#include <sstream>
 #include <string>
 
 using namespace std;
@@ -23,7 +23,7 @@ static const char VERSION_MESSAGE[] =
 PROGRAM " (" PACKAGE_NAME ") " VERSION "\n"
 "Written by Shaun Jackman.\n"
 "\n"
-"Copyright 2011 Canada's Michael Smith Genome Science Centre\n";
+"Copyright 2012 Canada's Michael Smith Genome Science Centre\n";
 
 static const char USAGE_MESSAGE[] =
 "Usage: " PROGRAM " [OPTION]... FILE\n"
@@ -32,6 +32,9 @@ static const char USAGE_MESSAGE[] =
 "      --both              build both FAI and FM indexes [default]\n"
 "      --fai               build a FAI index\n"
 "      --fm                build a FM index\n"
+"      --fa2bwt            build the BWT directly without the SA\n"
+"      --bwt2fm            build the FM index from the BWT\n"
+"  -a, --alphabet=STRING   use the alphabet STRING [-ACGT]\n"
 "  -s, --sample=N          sample the suffix array [16]\n"
 "  -d, --decompress        decompress the index FILE\n"
 "  -c, --stdout            write output to standard output\n"
@@ -49,6 +52,15 @@ namespace opt {
 	enum { NONE, FAI, FM, BOTH };
 	static int indexes = BOTH;
 
+	/** Build the BWT directly without the SA. */
+	static int fa2bwt;
+
+	/** Build the FM index from the BWT. */
+	static int bwt2fm;
+
+	/** The alphabet. */
+	static string alphabet = "-ACGT";
+
 	/** Decompress the index. */
 	static bool decompress;
 
@@ -59,7 +71,7 @@ namespace opt {
 	static int verbose;
 }
 
-static const char shortopts[] = "cds:v";
+static const char shortopts[] = "a:cds:v";
 
 enum { OPT_HELP = 1, OPT_VERSION };
 
@@ -67,6 +79,9 @@ static const struct option longopts[] = {
 	{ "both", no_argument, &opt::indexes, opt::BOTH },
 	{ "fai", no_argument, &opt::indexes, opt::FAI },
 	{ "fm", no_argument, &opt::indexes, opt::FM },
+	{ "fa2bwt", no_argument, &opt::fa2bwt, true },
+	{ "bwt2fm", no_argument, &opt::bwt2fm, true },
+	{ "alphabet", optional_argument, NULL, 'a' },
 	{ "decompress", no_argument, NULL, 'd' },
 	{ "sample", required_argument, NULL, 's' },
 	{ "stdout", no_argument, NULL, 'c' },
@@ -93,27 +108,64 @@ static void indexFasta(const string& faPath, const string& faiPath)
 	assert_good(out, faiPath);
 }
 
+/** Build the FM index from the BWT. */
+static void buildFMIndexFromBWT(FMIndex& fm, const string& path)
+{
+	if (opt::verbose > 0)
+		cerr << "Reading `" << path << "'...\n";
+	std::vector<FMIndex::value_type> bwt;
+	readFile(path.c_str(), bwt);
+	assert(bwt.size() > 1);
+
+	if (opt::alphabet.empty()) {
+		fm.setAlphabet(bwt.begin(), bwt.end());
+		std::cerr << "The alphabet has "
+			<< fm.alphabetSize() << " symbols.\n";
+	} else
+		fm.setAlphabet(opt::alphabet);
+
+	fm.encode(bwt.begin(), bwt.end());
+	fm.sampleSA(opt::sampleSA);
+	fm.assignBWT(bwt.begin(), bwt.end());
+}
+
 /** Build an FM index of the specified file. */
-static void buildFMIndex(FMIndex& fm, const char* path)
+static void buildFMIndex(FMIndex& fm, const string& path)
 {
 	if (opt::verbose > 0)
 		std::cerr << "Reading `" << path << "'...\n";
 	std::vector<FMIndex::value_type> s;
-	readFile(path, s);
+	readFile(path.c_str(), s);
 
 	size_t MAX_SIZE = numeric_limits<FMIndex::sais_size_type>::max();
 	if (s.size() > MAX_SIZE) {
 		std::cerr << PROGRAM << ": `" << path << "', "
 			<< toSI(s.size())
-			<< "B, must be smaller than " 
+			<< "B, must be smaller than "
 			<< toSI(MAX_SIZE) << "B\n";
 		exit(EXIT_FAILURE);
 	}
 
+	// Set the alphabet.
 	transform(s.begin(), s.end(), s.begin(), ::toupper);
-	fm.setAlphabet("-ACGT");
-	fm.assign(s.begin(), s.end());
-	fm.sampleSA(opt::sampleSA);
+	if (opt::alphabet.empty()) {
+		fm.setAlphabet(s.begin(), s.end());
+		std::cerr << "The alphabet has "
+			<< fm.alphabetSize() << " symbols.\n";
+	} else
+		fm.setAlphabet(opt::alphabet);
+
+	if (opt::fa2bwt) {
+		// Build the BWT first.
+		s.push_back(0);
+		fm.buildBWT(s.begin(), s.end() - 1);
+		fm.sampleSA(opt::sampleSA);
+		fm.assignBWT(s.begin(), s.end());
+	} else {
+		// Construct the suffix array first.
+		fm.assign(s.begin(), s.end());
+		fm.sampleSA(opt::sampleSA);
+	}
 }
 
 int main(int argc, char **argv)
@@ -126,9 +178,13 @@ int main(int argc, char **argv)
 		istringstream arg(optarg != NULL ? optarg : "");
 		switch (c) {
 			case '?': die = true; break;
+			case 'a':
+				opt::alphabet = arg.str();
+				arg.clear(ios::eofbit);
+				break;
 			case 'c': opt::toStdout = true; break;
 			case 'd': opt::decompress = true; break;
-			case 's': arg >> opt::sampleSA; assert(arg.eof()); break;
+			case 's': arg >> opt::sampleSA; break;
 			case 'v': opt::verbose++; break;
 			case OPT_HELP:
 				cout << USAGE_MESSAGE;
@@ -137,6 +193,11 @@ int main(int argc, char **argv)
 				cout << VERSION_MESSAGE;
 				exit(EXIT_SUCCESS);
 		}
+		if (optarg != NULL && !arg.eof()) {
+			cerr << PROGRAM ": invalid option: `-"
+				<< (char)c << optarg << "'\n";
+			exit(EXIT_FAILURE);
+		}
 	}
 
 	if (argc - optind < 1) {
@@ -180,10 +241,7 @@ int main(int argc, char **argv)
 		out.flush();
 		assert_good(out, faPath);
 
-		ostringstream ss;
-		ss << faPath << ".fai";
-		string faiPath(ss.str());
-		in.open(faiPath.c_str());
+		in.open((faPath + ".fai").c_str());
 		FastaIndex faIndex;
 		if (in) {
 			in >> faIndex;
@@ -192,31 +250,30 @@ int main(int argc, char **argv)
 		return 0;
 	}
 
-	const char* faPath(argv[optind]);
-	ostringstream ss;
-	ss << faPath << ".fm";
-	string fmPath = opt::toStdout ? "-" : ss.str();
-	ss.str("");
-	ss << faPath << ".fai";
-	string faiPath(ss.str());
-
-	if (opt::indexes & opt::FAI)
-		indexFasta(faPath, faiPath);
-
-	if ((opt::indexes & opt::FM) == 0)
-		return 0;
 
+	string path = argv[optind];
 	FMIndex fm;
-	buildFMIndex(fm, faPath);
+	if (opt::bwt2fm) {
+		buildFMIndexFromBWT(fm, path);
+	} else {
+		if (opt::indexes & opt::FAI)
+			indexFasta(path, path + ".fai");
 
-	if (opt::verbose > 0) {
-		size_t bp = fm.size();
-		ssize_t bytes = getMemoryUsage();
-		cerr << "Read " << toSI(bp) << "B. "
-			"Used " << toSI(bytes) << "B of memory and "
-				<< setprecision(3) << (float)bytes / bp << " B/bp.\n";
+		if ((opt::indexes & opt::FM) == 0)
+			return 0;
+
+		buildFMIndex(fm, path);
 	}
 
+	if (opt::verbose > 0) {
+		size_t n = fm.size();
+		ssize_t bytes = getMemoryUsage();
+		cerr << "Read " << toSI(n) << "B. "
+			"Used " << toSI(bytes) << "B of memory and "
+				<< setprecision(3) << (float)bytes / n << " B/bp.\n";
+	}
+
+	string fmPath = opt::toStdout ? "-" : path + ".fm";
 	cerr << "Writing `" << fmPath << "'...\n";
 	ofstream fout;
 	if (!opt::toStdout)
