@@ -1,3 +1,11 @@
+#if __APPLE_CC__
+/* Work around a bug in i686-apple-darwin11-llvm-gcc 4.2.1
+ * Undefined symbols for architecture x86_64:
+ *  "___builtin_expect", referenced from:
+ */
+#define NDEBUG 1
+#endif
+
 #include "BitUtil.h"
 #include "ContigProperties.h"
 #include "DataLayer/Options.h"
@@ -35,7 +43,7 @@ static const char VERSION_MESSAGE[] =
 PROGRAM " (" PACKAGE_NAME ") " VERSION "\n"
 "Written by Shaun Jackman.\n"
 "\n"
-"Copyright 2011 Canada's Michael Smith Genome Science Centre\n";
+"Copyright 2012 Canada's Michael Smith Genome Science Centre\n";
 
 static const char USAGE_MESSAGE[] =
 "Usage: " PROGRAM " [OPTION]... FILE\n"
@@ -107,13 +115,13 @@ static void addSuffixOverlaps(Graph &g,
 {
 	typedef edge_property<Graph>::type EP;
 	typedef graph_traits<Graph>::edge_descriptor E;
+	typedef graph_traits<Graph>::vertex_descriptor V;
 
 	Distance ep(-fmi.qspan());
 	assert(ep.distance < 0);
 	for (unsigned i = fmi.l; i < fmi.u; ++i) {
 		size_t tstart = fmIndex[i] + 1;
-		pair<FAIRecord, size_t> idPos = faIndex[tstart];
-		ContigNode v(idPos.first.id, false);
+		V v = find_vertex(faIndex[tstart].get<0>().id, false, g);
 #pragma omp critical(g)
 		{
 			pair<E, bool> e = edge(u, v, g);
@@ -121,7 +129,7 @@ static void addSuffixOverlaps(Graph &g,
 				const EP& ep0 = g[e.first];
 				if (opt::verbose > 1)
 					cerr << "duplicate edge: "
-						<< u << " -> " << v << ' '
+						<< get(edge_name, g, e.first) << ' '
 						<< ep0 << ' ' << ep << '\n';
 				assert(ep0.distance < ep.distance);
 			} else if(u.sense()) {
@@ -142,6 +150,7 @@ static void addPrefixOverlaps(Graph &g,
 {
 	typedef edge_property<Graph>::type EP;
 	typedef graph_traits<Graph>::edge_descriptor E;
+	typedef graph_traits<Graph>::vertex_descriptor V;
 
 	assert(v.sense());
 	assert(fmi.qstart == 0);
@@ -149,8 +158,7 @@ static void addPrefixOverlaps(Graph &g,
 	assert(ep.distance < 0);
 	for (unsigned i = fmi.l; i < fmi.u; ++i) {
 		size_t tstart = fmIndex[i];
-		pair<FAIRecord, size_t> idPos = faIndex[tstart];
-		ContigNode u(idPos.first.id, false);
+		V u = find_vertex(faIndex[tstart].get<0>().id, false, g);
 #pragma omp critical(g)
 		{
 			pair<E, bool> e = edge(u, v, g);
@@ -158,7 +166,7 @@ static void addPrefixOverlaps(Graph &g,
 				const EP& ep0 = g[e.first];
 				if (opt::verbose > 1)
 					cerr << "duplicate edge: "
-						<< u << " -> " << v << ' '
+						<< get(edge_name, g, e.first) << ' '
 						<< ep0 << ' ' << ep << '\n';
 				assert(ep0.distance < ep.distance);
 			} else {
@@ -209,14 +217,16 @@ static void findOverlaps(Graph& g,
 		const FastaIndex& faIndex, const FMIndex& fmIndex,
 		const FastqRecord& rec)
 {
-	ContigNode u(rec.id, false);
+	typedef graph_traits<Graph>::vertex_descriptor V;
+	V u = find_vertex(rec.id, false, g);
+	V uc = get(vertex_complement, g, u);
 	// Add edges u+ -> v+ and v- -> u-
 	findOverlapsSuffix(g, faIndex, fmIndex, u, rec.seq);
 	string rcseq = reverseComplement(rec.seq);
 	// Add edges u- -> v+
-	findOverlapsSuffix(g, faIndex, fmIndex, ~u, rcseq);
+	findOverlapsSuffix(g, faIndex, fmIndex, uc, rcseq);
 	// Add edges v+ -> u-
-	findOverlapsPrefix(g, faIndex, fmIndex, ~u, rcseq);
+	findOverlapsPrefix(g, faIndex, fmIndex, uc, rcseq);
 }
 
 /** Map the sequences of the specified file. */
@@ -268,7 +278,7 @@ static void addVertices(const string& path, Graph& g)
 	ifstream in(path.c_str());
 	assert_good(in, path);
 	in >> g;
-	ContigID::lock();
+	g_contigNames.lock();
 	assert(in.eof());
 }
 
@@ -324,17 +334,23 @@ int main(int argc, char** argv)
 		istringstream arg(optarg != NULL ? optarg : "");
 		switch (c) {
 		  case '?':
-			die = true; break;
+			die = true;
+			break;
 		  case 'j':
-			arg >> opt::threads; assert(arg.eof()); break;
+			arg >> opt::threads;
+			break;
 		  case 'm':
-			arg >> opt::minOverlap; assert(arg.eof()); break;
+			arg >> opt::minOverlap;
+			break;
 		  case 'k':
-			arg >> opt::maxOverlap; assert(arg.eof()); break;
+			arg >> opt::maxOverlap;
+			break;
 		  case 's':
-			arg >> opt::sampleSA; assert(arg.eof()); break;
+			arg >> opt::sampleSA;
+			break;
 		  case 'v':
-			opt::verbose++; break;
+			opt::verbose++;
+			break;
 		  case OPT_HELP:
 			cout << USAGE_MESSAGE;
 			exit(EXIT_SUCCESS);
@@ -342,6 +358,11 @@ int main(int argc, char** argv)
 			cout << VERSION_MESSAGE;
 			exit(EXIT_SUCCESS);
 		}
+		if (optarg != NULL && !arg.eof()) {
+			cerr << PROGRAM ": invalid option: `-"
+				<< (char)c << optarg << "'\n";
+			exit(EXIT_FAILURE);
+		}
 	}
 
 	if (argc - optind < 1) {
