@@ -6,9 +6,11 @@
 #include "DataLayer/Options.h"
 #include "Dictionary.h"
 #include "FastaReader.h"
+#include "Histogram.h"
 #include "IOUtil.h"
 #include "MemoryUtil.h"
 #include "smith_waterman.h"
+#include "Sequence.h"
 #include "StringUtil.h"
 #include "Uncompress.h"
 #include "Graph/ContigGraph.h"
@@ -36,7 +38,7 @@ static const char VERSION_MESSAGE[] =
 PROGRAM " (" PACKAGE_NAME ") " VERSION "\n"
 "Written by Shaun Jackman.\n"
 "\n"
-"Copyright 2011 Canada's Michael Smith Genome Science Centre\n";
+"Copyright 2012 Canada's Michael Smith Genome Science Centre\n";
 
 static const char USAGE_MESSAGE[] =
 "Usage: " PROGRAM " [OPTION]... FASTA [OVERLAP] PATH\n"
@@ -223,8 +225,8 @@ static void mergeContigs(const Graph& g, const Contigs& contigs,
 		seq += o.overlap_str;
 		seq += Sequence(s, o.overlap_h_pos + 1);
 	} else {
-		cerr << "warning: the head of `" << v << "' "
-			"does not match the tail of the previous contig\n"
+		cerr << "warning: the head of " << get(vertex_name, g, v)
+			<< " does not match the tail of the previous contig\n"
 			<< ao << '\n' << bo << '\n' << path << endl;
 		seq += 'n';
 		seq += s;
@@ -232,15 +234,16 @@ static void mergeContigs(const Graph& g, const Contigs& contigs,
 }
 
 /** Return a FASTA comment for the specified path. */
-static void pathToComment(ostream& out, const ContigPath& path)
+static void pathToComment(ostream& out,
+		const Graph& g, const ContigPath& path)
 {
 	assert(path.size() > 1);
-	out << path.front();
+	out << get(vertex_name, g, path.front());
 	if (path.size() == 3)
-		out << ',' << path[1];
+		out << ',' << get(vertex_name, g, path[1]);
 	else if (path.size() > 3)
 		out << ",...";
-	out << ',' << path.back();
+	out << ',' << get(vertex_name, g, path.back());
 }
 
 /** Merge the specified path. */
@@ -262,7 +265,7 @@ static Contig mergePath(const Graph& g, const Contigs& contigs,
 	}
 	ostringstream ss;
 	ss << seq.size() << ' ' << coverage << ' ';
-	pathToComment(ss, path);
+	pathToComment(ss, g, path);
 	return Contig(ss.str(), seq);
 }
 
@@ -325,9 +328,14 @@ static void markRemovedContigs(vector<bool>& marked,
 		const vector<string>& pathIDs, const ContigPaths& paths)
 {
 	for (ContigPaths::const_iterator it = paths.begin();
-			it != paths.end(); ++it)
-		if (it->empty())
-			marked[ContigID(pathIDs[it - paths.begin()])] = true;
+			it != paths.end(); ++it) {
+		if (it->empty()) {
+			size_t i = get(g_contigNames,
+					pathIDs[it - paths.begin()]);
+			assert(i < marked.size());
+			marked[i] = true;
+		}
+	}
 }
 
 /** Output the updated overlap graph. */
@@ -335,16 +343,20 @@ static void outputGraph(Graph& g,
 		const vector<string>& pathIDs, const ContigPaths& paths,
 		const string& commandLine)
 {
+	typedef graph_traits<Graph>::vertex_descriptor V;
+
 	// Add the path vertices.
+	g_contigNames.unlock();
 	for (ContigPaths::const_iterator it = paths.begin();
 			it != paths.end(); ++it) {
 		const ContigPath& path = *it;
 		const string& id = pathIDs[it - paths.begin()];
 		if (!path.empty()) {
-			ContigID::insert(id);
-			merge(g, path.begin(), path.end());
+			V u = merge(g, path.begin(), path.end());
+			put(vertex_name, g, u, id);
 		}
 	}
+	g_contigNames.lock();
 
 	// Remove the vertices that are used in paths.
 	for (ContigPaths::const_iterator it = paths.begin();
@@ -352,7 +364,7 @@ static void outputGraph(Graph& g,
 		const ContigPath& path = *it;
 		const string& id = pathIDs[it - paths.begin()];
 		if (path.empty()) {
-			remove_vertex(ContigNode(id, false), g);
+			remove_vertex(find_vertex(id, false, g), g);
 		} else {
 			remove_vertex_if(g, path.begin(), path.end(),
 					not1(std::mem_fun_ref(&ContigNode::ambiguous)));
@@ -391,7 +403,7 @@ int main(int argc, char** argv)
 		istringstream arg(optarg != NULL ? optarg : "");
 		switch (c) {
 			case '?': die = true; break;
-			case 'g': arg >> opt::graphPath; assert(arg.eof()); break;
+			case 'g': arg >> opt::graphPath; break;
 			case 'k': arg >> opt::k; break;
 			case 'o': arg >> opt::out; break;
 			case 'v': opt::verbose++; break;
@@ -402,6 +414,11 @@ int main(int argc, char** argv)
 				cout << VERSION_MESSAGE;
 				exit(EXIT_SUCCESS);
 		}
+		if (optarg != NULL && !arg.eof()) {
+			cerr << PROGRAM ": invalid option: `-"
+				<< (char)c << optarg << "'\n";
+			exit(EXIT_FAILURE);
+		}
 	}
 
 	if (opt::k <= 0) {
@@ -445,7 +462,8 @@ int main(int argc, char** argv)
 		assert(fin.eof());
 		if (opt::verbose > 0)
 			cerr << "Read " << num_vertices(g) << " vertices. "
-				"Using " << toSI(getMemoryUsage()) << "B of memory.\n";
+				"Using " << toSI(getMemoryUsage())
+				<< "B of memory.\n";
 	}
 	mergedPathFile = string(argv[optind++]);
 
@@ -458,12 +476,12 @@ int main(int argc, char** argv)
 		FastaReader in(contigFile, FastaReader::NO_FOLD_CASE);
 		for (FastaRecord rec; in >> rec;) {
 			if (!adjPath.empty()
-					&& ContigID::count(rec.id) == 0)
+					&& g_contigNames.count(rec.id) == 0)
 				continue;
-			ContigID id = adjPath.empty()
-				? ContigID::insert(rec.id)
-				: ContigID(rec.id);
-			assert(id == contigs.size());
+			if (adjPath.empty())
+				put(g_contigNames, contigs.size(), rec.id);
+			else
+				assert(get(g_contigNames, rec.id) == contigs.size());
 			contigs.push_back(rec);
 
 			++count;
@@ -479,7 +497,7 @@ int main(int argc, char** argv)
 		assert(in.eof());
 		assert(!contigs.empty());
 		opt::colourSpace = isdigit(contigs[0].seq[0]);
-		ContigID::lock();
+		g_contigNames.lock();
 	}
 
 	vector<string> pathIDs;
@@ -491,6 +509,7 @@ int main(int argc, char** argv)
 	markRemovedContigs(seen, pathIDs, paths);
 
 	// Output those contigs that were not seen in a path.
+	Histogram lengthHistogram;
 	ofstream fout;
 	ostream& out = opt::out == "-" ? cout
 		: (fout.open(opt::out.c_str()), fout);
@@ -501,10 +520,14 @@ int main(int argc, char** argv)
 			ContigID id(it - contigs.begin());
 			if (!seen[id]) {
 				const Contig& contig = *it;
-				out << '>' << id;
+				out << '>' << get(g_contigNames, id);
 				if (!contig.comment.empty())
 					out << ' ' << contig.comment;
 				out << '\n' << contig.seq << '\n';
+				if (opt::verbose > 0)
+					lengthHistogram.insert(
+						count_if(contig.seq.begin(), contig.seq.end(),
+							isACGT));
 			}
 		}
 	}
@@ -524,6 +547,10 @@ int main(int argc, char** argv)
 			<< contig.seq << '\n';
 		assert_good(out, opt::out);
 		npaths++;
+		if (opt::verbose > 0)
+			lengthHistogram.insert(
+					count_if(contig.seq.begin(), contig.seq.end(),
+						isACGT));
 	}
 
 	if (npaths == 0)
@@ -553,5 +580,10 @@ int main(int argc, char** argv)
 		cerr << "Consider increasing the coverage threshold "
 			"parameter, c, to " << minCovUsed << ".\n";
 
+	if (opt::verbose > 0) {
+		const unsigned STATS_MIN_LENGTH = 200; // bp
+		printContiguityStats(cerr, lengthHistogram, STATS_MIN_LENGTH)
+			<< '\t' << opt::out << '\n';
+	}
 	return 0;
 }
