@@ -1,11 +1,9 @@
 #include "config.h"
 #include "Common/Options.h"
 #include "ContigID.h"
-#include "ContigLength.h"
 #include "ContigPath.h"
 #include "Functional.h" // for mem_var
 #include "IOUtil.h"
-#include "Iterator.h"
 #include "Uncompress.h"
 #include "Graph/Assemble.h"
 #include "Graph/ContigGraph.h"
@@ -30,6 +28,7 @@
 #include <set>
 #include <sstream>
 #include <string>
+#include <vector>
 #if _OPENMP
 # include <omp.h>
 #endif
@@ -43,7 +42,7 @@ static const char VERSION_MESSAGE[] =
 PROGRAM " (" PACKAGE_NAME ") " VERSION "\n"
 "Written by Jared Simpson and Shaun Jackman.\n"
 "\n"
-"Copyright 2011 Canada's Michael Smith Genome Science Centre\n";
+"Copyright 2012 Canada's Michael Smith Genome Science Centre\n";
 
 static const char USAGE_MESSAGE[] =
 "Usage: " PROGRAM " [OPTION]... LEN PATH\n"
@@ -65,7 +64,7 @@ static const char USAGE_MESSAGE[] =
 "Report bugs to <" PACKAGE_BUGREPORT ">.\n";
 
 namespace opt {
-	unsigned k; // used by readContigLengths
+	unsigned k; // used by GraphIO
 	static string out;
 	static int threads = 1;
 
@@ -99,9 +98,6 @@ static const struct option longopts[] = {
 
 typedef map<ContigID, ContigPath> ContigPathMap;
 
-/** Lengths of contigs in k-mer. */
-static vector<unsigned> g_contigLengths;
-
 /** Orientation of an edge. */
 enum dir_type {
 	DIR_X, // u--v none
@@ -110,9 +106,14 @@ enum dir_type {
 	DIR_B, // u<>v both
 };
 
-static ContigPath align(const ContigPath& p1, const ContigPath& p2,
+/** Lengths of contigs measured in k-mer. */
+typedef vector<unsigned> Lengths;
+
+static ContigPath align(const Lengths& lengths,
+		const ContigPath& p1, const ContigPath& p2,
 		ContigNode pivot);
-static ContigPath align(const ContigPath& p1, const ContigPath& p2,
+static ContigPath align(const Lengths& lengths,
+		const ContigPath& p1, const ContigPath& p2,
 		ContigNode pivot, dir_type& orientation);
 
 static bool gDebugPrint;
@@ -130,7 +131,7 @@ static set<ContigID> findRepeats(const ContigPathMap& paths)
 		for (ContigPath::const_iterator it = path.begin();
 				it != path.end(); ++it)
 			if (!it->ambiguous())
-				count[ContigID(*it)]++;
+				count[it->contigIndex()]++;
 		for (map<ContigID, unsigned>::const_iterator
 				it = count.begin(); it != count.end(); ++it)
 			if (it->second > 1)
@@ -147,10 +148,11 @@ static set<ContigID> removeRepeats(ContigPathMap& paths)
 	set<ContigID> repeats = findRepeats(paths);
 	if (gDebugPrint) {
 		cout << "Repeats:";
-		if (!repeats.empty())
-			copy(repeats.begin(), repeats.end(),
-					affix_ostream_iterator<ContigID>(cout, " "));
-		else
+		if (!repeats.empty()) {
+			for (set<ContigID>::const_iterator it = repeats.begin();
+					it != repeats.end(); ++it)
+				cout << ' ' << get(g_contigNames, *it);
+		} else
 			cout << " none";
 		cout << '\n';
 	}
@@ -171,7 +173,7 @@ static set<ContigID> removeRepeats(ContigPathMap& paths)
 	for (set<ContigID>::const_iterator it = repeats.begin();
 			it != repeats.end(); ++it)
 		if (paths.erase(*it) > 0)
-			ss << ' ' << ContigID(*it);
+			ss << ' ' << get(g_contigNames, *it);
 
 	if (opt::verbose > 0 && removed > 0)
 		cout << "Removing paths in repeats:" << ss.str() << '\n';
@@ -194,7 +196,8 @@ typedef ContigGraph<DirectedGraph<> > PathGraph;
  * @param pivot the pivot at which to seed the alignment
  * @return whether an overlap was found
  */
-static bool addOverlapEdge(PathGraph& gout, ContigNode pivot,
+static bool addOverlapEdge(const Lengths& lengths,
+		PathGraph& gout, ContigNode pivot,
 		ContigNode seed1, const ContigPath& path1,
 		ContigNode seed2, const ContigPath& path2)
 {
@@ -202,7 +205,8 @@ static bool addOverlapEdge(PathGraph& gout, ContigNode pivot,
 
 	// Determine the orientation of the overlap edge.
 	dir_type orientation = DIR_X;
-	ContigPath consensus = align(path1, path2, pivot, orientation);
+	ContigPath consensus = align(lengths,
+			path1, path2, pivot, orientation);
 	if (consensus.empty())
 		return false;
 	assert(orientation != DIR_X);
@@ -230,7 +234,7 @@ static bool addOverlapEdge(PathGraph& gout, ContigNode pivot,
 /** Return the specified path. */
 static ContigPath getPath(const ContigPathMap& paths, ContigNode u)
 {
-	ContigPathMap::const_iterator it = paths.find(ContigID(u));
+	ContigPathMap::const_iterator it = paths.find(u.contigIndex());
 	assert(it != paths.end());
 	ContigPath path = it->second;
 	if (u.sense())
@@ -239,7 +243,8 @@ static ContigPath getPath(const ContigPathMap& paths, ContigNode u)
 }
 
 /** Find the overlaps between paths and add edges to the graph. */
-static void findPathOverlaps(const ContigPathMap& paths,
+static void findPathOverlaps(const Lengths& lengths,
+		const ContigPathMap& paths,
 		const ContigNode& seed1, const ContigPath& path1,
 		PathGraph& gout)
 {
@@ -251,21 +256,23 @@ static void findPathOverlaps(const ContigPathMap& paths,
 		if (seed2.ambiguous())
 			continue;
 		ContigPathMap::const_iterator path2It
-			= paths.find(ContigID(seed2));
+			= paths.find(seed2.contigIndex());
 		if (path2It == paths.end())
 			continue;
 
 		ContigPath path2 = path2It->second;
 		if (seed2.sense())
 			reverseComplement(path2.begin(), path2.end());
-		addOverlapEdge(gout, seed2, seed1, path1, seed2, path2);
+		addOverlapEdge(lengths,
+				gout, seed2, seed1, path1, seed2, path2);
 	}
 }
 
 /** Attempt to merge the paths specified in mergeQ with path.
  * @return the number of paths merged
  */
-static unsigned mergePaths(ContigPath& path,
+static unsigned mergePaths(const Lengths& lengths,
+		ContigPath& path,
 		deque<ContigNode>& mergeQ, set<ContigNode>& seen,
 		const ContigPathMap& paths)
 {
@@ -274,14 +281,14 @@ static unsigned mergePaths(ContigPath& path,
 	for (ContigNode pivot; !mergeQ.empty(); mergeQ.pop_front()) {
 		pivot = mergeQ.front();
 		ContigPathMap::const_iterator path2It
-			= paths.find(ContigID(pivot));
+			= paths.find(pivot.contigIndex());
 		if (path2It == paths.end())
 			continue;
 
 		ContigPath path2 = path2It->second;
 		if (pivot.sense())
 			reverseComplement(path2.begin(), path2.end());
-		ContigPath consensus = align(path, path2, pivot);
+		ContigPath consensus = align(lengths, path, path2, pivot);
 		if (consensus.empty()) {
 			invalid.push_back(pivot);
 			continue;
@@ -291,7 +298,8 @@ static unsigned mergePaths(ContigPath& path,
 		path.swap(consensus);
 		if (gDebugPrint)
 #pragma omp critical(cout)
-			cout << pivot << '\t' << path2 << '\n'
+			cout << get(g_contigNames, pivot)
+				<< '\t' << path2 << '\n'
 				<< '\t' << path << '\n';
 		merged++;
 	}
@@ -302,13 +310,13 @@ static unsigned mergePaths(ContigPath& path,
 /** Merge the paths of the specified seed path.
  * @return the merged contig path
  */
-static ContigPath mergePath(
+static ContigPath mergePath(const Lengths& lengths,
 		const ContigPathMap& paths, const ContigPath& seedPath)
 {
 	assert(!seedPath.empty());
 	ContigNode seed1 = seedPath.front();
 	ContigPathMap::const_iterator path1It
-		= paths.find(ContigID(seed1));
+		= paths.find(seed1.contigIndex());
 	assert(path1It != paths.end());
 	ContigPath path(path1It->second);
 	if (seedPath.front().sense())
@@ -316,12 +324,13 @@ static ContigPath mergePath(
 	if (opt::verbose > 1)
 #pragma omp critical(cout)
 		cout << "\n* " << seedPath << '\n'
-			<< seedPath.front() << '\t' << path << '\n';
+			<< get(g_contigNames, seedPath.front())
+			<< '\t' << path << '\n';
 	for (ContigPath::const_iterator it = seedPath.begin() + 1;
 			it != seedPath.end(); ++it) {
 		ContigNode seed2 = *it;
 		ContigPathMap::const_iterator path2It
-			= paths.find(ContigID(seed2));
+			= paths.find(seed2.contigIndex());
 		assert(path2It != paths.end());
 		ContigPath path2 = path2It->second;
 		if (seed2.sense())
@@ -330,18 +339,20 @@ static ContigPath mergePath(
 		ContigNode pivot
 			= find(path.begin(), path.end(), seed2) != path.end()
 			? seed2 : seed1;
-		ContigPath consensus = align(path, path2, pivot);
+		ContigPath consensus = align(lengths, path, path2, pivot);
 		if (consensus.empty()) {
 			// This seed could be removed from the seed path.
 			if (opt::verbose > 1)
 #pragma omp critical(cout)
-				cout << seed2 << '\t' << path2 << '\n'
+				cout << get(g_contigNames, seed2)
+					<< '\t' << path2 << '\n'
 					<< "\tinvalid\n";
 		} else {
 			path.swap(consensus);
 			if (opt::verbose > 1)
 #pragma omp critical(cout)
-				cout << seed2 << '\t' << path2 << '\n'
+				cout << get(g_contigNames, seed2)
+					<< '\t' << path2 << '\n'
 					<< '\t' << path << '\n';
 		}
 		seed1 = seed2;
@@ -355,7 +366,7 @@ typedef vector<ContigPath> ContigPaths;
 /** Merge the specified seed paths.
  * @return the merged contig paths
  */
-static ContigPaths mergeSeedPaths(
+static ContigPaths mergeSeedPaths(const Lengths& lengths,
 		const ContigPathMap& paths, const ContigPaths& seedPaths)
 {
 	if (opt::verbose > 0)
@@ -365,14 +376,15 @@ static ContigPaths mergeSeedPaths(
 	out.reserve(seedPaths.size());
 	for (ContigPaths::const_iterator it = seedPaths.begin();
 			it != seedPaths.end(); ++it)
-		out.push_back(mergePath(paths, *it));
+		out.push_back(mergePath(lengths, paths, *it));
 	return out;
 }
 
 /** Extend the specified path as long as is unambiguously possible and
  * add the result to the specified container.
  */
-static void extendPaths(ContigID id, const ContigPathMap& paths,
+static void extendPaths(const Lengths& lengths,
+		ContigID id, const ContigPathMap& paths,
 		ContigPathMap& out)
 {
 	ContigPathMap::const_iterator pathIt = paths.find(id);
@@ -386,14 +398,14 @@ static void extendPaths(ContigID id, const ContigPathMap& paths,
 
 	if (gDebugPrint)
 		#pragma omp critical(cout)
-		cout << "\n* " << ContigNode(id, false) << '\n'
+		cout << "\n* " << get(g_contigNames, id) << "+\n"
 			<< '\t' << path << '\n';
 
 	set<ContigNode> seen;
 	seen.insert(ContigNode(id, false));
 	deque<ContigNode> mergeQ;
 	appendToMergeQ(mergeQ, seen, path);
-	while (mergePaths(path, mergeQ, seen, paths) > 0)
+	while (mergePaths(lengths, path, mergeQ, seen, paths) > 0)
 		;
 
 	if (!mergeQ.empty() && gDebugPrint) {
@@ -402,8 +414,8 @@ static void extendPaths(ContigID id, const ContigPathMap& paths,
 			cout << "invalid\n";
 			for (deque<ContigNode>::const_iterator it
 					= mergeQ.begin(); it != mergeQ.end(); ++it)
-				cout << *it << '\t'
-					<< paths.find(ContigID(*it))->second << '\n';
+				cout << get(g_contigNames, *it) << '\t'
+					<< paths.find(it->contigIndex())->second << '\n';
 		}
 	}
 }
@@ -424,16 +436,16 @@ static bool equalIgnoreAmbiguos(const ContigPath& a,
 }
 
 /** Return whether this path is a cycle. */
-static bool isCycle(const ContigPath& path)
+static bool isCycle(const Lengths& lengths, const ContigPath& path)
 {
-	return !align(path, path, path.front()).empty();
+	return !align(lengths, path, path, path.front()).empty();
 }
 
 /** Identify paths subsumed by the specified path.
  * @param overlaps [out] paths that are found to overlap
  * @return the ID of the subsuming path
  */
-static ContigID identifySubsumedPaths(
+static ContigID identifySubsumedPaths(const Lengths& lengths,
 		ContigPathMap::const_iterator path1It,
 		ContigPathMap& paths,
 		set<ContigID>& out,
@@ -444,45 +456,50 @@ static ContigID identifySubsumedPaths(
 	ContigID id(path1It->first);
 	const ContigPath& path = path1It->second;
 	if (gDebugPrint)
-		vout << ContigNode(id, false) << '\t' << path << '\n';
+		vout << get(g_contigNames, ContigNode(id, false))
+			<< '\t' << path << '\n';
 
 	for (ContigPath::const_iterator it = path.begin();
 			it != path.end(); ++it) {
 		ContigNode pivot = *it;
 		if (pivot.ambiguous() || pivot.id() == id)
 			continue;
-		ContigPathMap::iterator path2It = paths.find(ContigID(pivot));
+		ContigPathMap::iterator path2It
+			= paths.find(pivot.contigIndex());
 		if (path2It == paths.end())
 			continue;
 		ContigPath path2 = path2It->second;
 		if (pivot.sense())
 			reverseComplement(path2.begin(), path2.end());
-		ContigPath consensus = align(path, path2, pivot);
+		ContigPath consensus = align(lengths, path, path2, pivot);
 		if (consensus.empty())
 			continue;
 		if (equalIgnoreAmbiguos(consensus, path)) {
 			if (gDebugPrint)
-				vout << pivot << '\t' << path2 << '\n';
+				vout << get(g_contigNames, pivot)
+					<< '\t' << path2 << '\n';
 			out.insert(path2It->first);
 		} else if (equalIgnoreAmbiguos(consensus, path2)) {
 			// This path is larger. Use it as the seed.
-			return identifySubsumedPaths(path2It, paths, out,
+			return identifySubsumedPaths(lengths, path2It, paths, out,
 					overlaps);
-		} else if (isCycle(consensus)) {
+		} else if (isCycle(lengths, consensus)) {
 			// The consensus path is a cycle.
-			bool isCyclePath1 = isCycle(path);
-			bool isCyclePath2 = isCycle(path2);
+			bool isCyclePath1 = isCycle(lengths, path);
+			bool isCyclePath2 = isCycle(lengths, path2);
 			if (!isCyclePath1 && !isCyclePath2) {
 				// Neither path is a cycle.
 				if (gDebugPrint)
-					vout << pivot << '\t' << path2 << '\n'
+					vout << get(g_contigNames, pivot)
+						<< '\t' << path2 << '\n'
 						<< "ignored\t" << consensus << '\n';
 				overlaps.insert(id);
 				overlaps.insert(path2It->first);
 			} else {
 				// At least one path is a cycle.
 				if (gDebugPrint)
-					vout << pivot << '\t' << path2 << '\n'
+					vout << get(g_contigNames, pivot)
+						<< '\t' << path2 << '\n'
 						<< "cycle\t" << consensus << '\n';
 				if (isCyclePath1 && isCyclePath2)
 					out.insert(path2It->first);
@@ -493,7 +510,8 @@ static ContigID identifySubsumedPaths(
 			}
 		} else {
 			if (gDebugPrint)
-				vout << pivot << '\t' << path2 << '\n'
+				vout << get(g_contigNames, pivot)
+					<< '\t' << path2 << '\n'
 					<< "ignored\t" << consensus << '\n';
 			overlaps.insert(id);
 			overlaps.insert(path2It->first);
@@ -509,13 +527,15 @@ static ContigID identifySubsumedPaths(
  * @return the next iterator after path1it
  */
 static ContigPathMap::const_iterator removeSubsumedPaths(
+		const Lengths& lengths,
 		ContigPathMap::const_iterator path1It, ContigPathMap& paths,
 		ContigID& seed, set<ContigID>& overlaps)
 {
 	if (gDebugPrint)
 		cout << '\n';
 	set<ContigID> eq;
-	seed = identifySubsumedPaths(path1It, paths, eq, overlaps);
+	seed = identifySubsumedPaths(lengths,
+			path1It, paths, eq, overlaps);
 	++path1It;
 	for (set<ContigID>::const_iterator it = eq.begin();
 			it != eq.end(); ++it) {
@@ -529,14 +549,16 @@ static ContigPathMap::const_iterator removeSubsumedPaths(
 /** Remove paths subsumed by another path.
  * @return paths that are found to overlap
  */
-static set<ContigID> removeSubsumedPaths(ContigPathMap& paths)
+static set<ContigID> removeSubsumedPaths(const Lengths& lengths,
+		ContigPathMap& paths)
 {
 	set<ContigID> overlaps, seen;
 	for (ContigPathMap::const_iterator iter = paths.begin();
 			iter != paths.end();) {
 		if (seen.count(iter->first) == 0) {
 			ContigID seed;
-			iter = removeSubsumedPaths(iter, paths, seed, overlaps);
+			iter = removeSubsumedPaths(lengths,
+					iter, paths, seed, overlaps);
 			seen.insert(seed);
 		} else
 			++iter;
@@ -548,7 +570,8 @@ static set<ContigID> removeSubsumedPaths(ContigPathMap& paths)
  * outgoing edges, (u,v1) and (u,v2), add the edge (v1,v2) if v1 < v2,
  * and add the edge (v2,v1) if v2 < v1.
  */
-static void addMissingEdges(PathGraph& g, const ContigPathMap& paths)
+static void addMissingEdges(const Lengths& lengths,
+		PathGraph& g, const ContigPathMap& paths)
 {
 	typedef graph_traits<PathGraph>::adjacency_iterator Vit;
 	typedef graph_traits<PathGraph>::vertex_iterator Uit;
@@ -566,7 +589,8 @@ static void addMissingEdges(PathGraph& g, const ContigPathMap& paths)
 			++vit1;
 			assert(v1 != u);
 			ContigPath path1 = getPath(paths, v1);
-			if (find(path1.begin(), path1.end(), u) == path1.end())
+			if (find(path1.begin(), path1.end(),
+						ContigPath::value_type(u)) == path1.end())
 				continue;
 			for (Vit vit2 = vit1; vit2 != vrange.second; ++vit2) {
 				V v2 = *vit2;
@@ -575,11 +599,11 @@ static void addMissingEdges(PathGraph& g, const ContigPathMap& paths)
 				if (edge(v1, v2, g).second || edge(v2, v1, g).second)
 					continue;
 				ContigPath path2 = getPath(paths, v2);
-				if (find(path2.begin(), path2.end(), u)
-						== path2.end())
+				if (find(path2.begin(), path2.end(),
+							ContigPath::value_type(u)) == path2.end())
 					continue;
-				numAdded
-					+= addOverlapEdge(g, u, v1, path1, v2, path2);
+				numAdded += addOverlapEdge(lengths,
+						g, u, v1, path1, v2, path2);
 			}
 		}
 	}
@@ -628,7 +652,7 @@ static void removeSmallOverlaps(PathGraph& g,
 				continue;
 			ContigPath pathv = getPath(paths, v);
 			if (pathu.back() == pathv.front()
-					&& paths.count(pathu.back()) > 0)
+					&& paths.count(pathu.back().contigIndex()) > 0)
 				edges.push_back(uv);
 		}
 	}
@@ -664,17 +688,18 @@ static void outputSortedPaths(const ContigPathMap& paths)
 	assert_good(out, opt::out);
 	for (vector<ContigPath>::const_iterator it = sortedPaths.begin();
 			it != sortedPaths.end(); ++it)
-		out << ContigID::create() << '\t' << *it << '\n';
+		out << createContigName() << '\t' << *it << '\n';
 	assert_good(out, opt::out);
 }
 
 /** Assemble the path overlap graph. */
-static void assemblePathGraph(
+static void assemblePathGraph(const Lengths& lengths,
 		PathGraph& pathGraph, ContigPathMap& paths)
 {
 	ContigPaths seedPaths;
 	assembleDFS(pathGraph, back_inserter(seedPaths));
-	ContigPaths mergedPaths = mergeSeedPaths(paths, seedPaths);
+	ContigPaths mergedPaths = mergeSeedPaths(lengths,
+			paths, seedPaths);
 	if (opt::verbose > 1)
 		cout << '\n';
 
@@ -688,7 +713,8 @@ static void assemblePathGraph(
 				it2 != it1->end(); ++it2) {
 			ContigNode seed(*it2);
 			if (find(path.begin(), path.end(), seed) != path.end()) {
-				paths[ContigID(seed)] = seed.sense() ? pathrc : path;
+				paths[seed.contigIndex()]
+					= seed.sense() ? pathrc : path;
 			} else {
 				// This seed was not included in the merged path.
 			}
@@ -700,13 +726,14 @@ static void assemblePathGraph(
 	// Remove the subsumed paths.
 	if (opt::verbose > 0)
 		cout << "Removing redundant contigs\n";
-	removeSubsumedPaths(paths);
+	removeSubsumedPaths(lengths, paths);
 
 	outputSortedPaths(paths);
 }
 
 /** Read a set of paths from the specified file. */
-static ContigPathMap readPaths(const string& filePath)
+static ContigPathMap readPaths(const Lengths& lengths,
+		const string& filePath)
 {
 	if (opt::verbose > 0)
 		cerr << "Reading `" << filePath << "'..." << endl;
@@ -715,11 +742,12 @@ static ContigPathMap readPaths(const string& filePath)
 
 	unsigned tooSmall = 0;
 	ContigPathMap paths;
-	ContigID id;
+	std::string name;
 	ContigPath path;
-	while (in >> id >> path) {
+	while (in >> name >> path) {
 		// Ignore seed contigs shorter than the threshold length.
-		unsigned len = g_contigLengths[id] + opt::k - 1;
+		ContigID id(get(g_contigNames, name));
+		unsigned len = lengths[id] + opt::k - 1;
 		if (len < opt::seedLen) {
 			tooSmall++;
 			continue;
@@ -751,16 +779,17 @@ bool atomicInc(T& it, T last, T& out)
 }
 
 /** Build the path overlap graph. */
-static void buildPathGraph(PathGraph& g, const ContigPathMap& paths)
+static void buildPathGraph(const Lengths& lengths,
+		PathGraph& g, const ContigPathMap& paths)
 {
 	// Create the vertices of the path overlap graph.
-	PathGraph(g_contigLengths.size()).swap(g);
+	PathGraph(lengths.size()).swap(g);
 
 	// Remove the non-seed contigs.
 	typedef graph_traits<PathGraph>::vertex_iterator vertex_iterator;
 	pair<vertex_iterator, vertex_iterator> vit = g.vertices();
 	for (vertex_iterator u = vit.first; u != vit.second; ++u)
-		if (paths.count(ContigID(*u)) == 0)
+		if (paths.count(get(vertex_contig_index, g, *u)) == 0)
 			remove_vertex(*u, g);
 
 	// Find the overlapping paths.
@@ -768,12 +797,12 @@ static void buildPathGraph(PathGraph& g, const ContigPathMap& paths)
 	#pragma omp parallel
 	for (ContigPathMap::const_iterator it;
 			atomicInc(sharedIt, paths.end(), it);)
-		findPathOverlaps(paths,
+		findPathOverlaps(lengths, paths,
 				ContigNode(it->first, false), it->second, g);
 	if (gDebugPrint)
 		cout << '\n';
 
-	addMissingEdges(g, paths);
+	addMissingEdges(lengths, g, paths);
 	removeTransitiveEdges(g);
 	removeSmallOverlaps(g, paths);
 	if (opt::verbose > 0)
@@ -781,6 +810,34 @@ static void buildPathGraph(PathGraph& g, const ContigPathMap& paths)
 	outputPathGraph(g);
 }
 
+/** Read contig lengths. */
+static Lengths readContigLengths(istream& in)
+{
+	assert(in);
+	assert(g_contigNames.empty());
+	Lengths lengths;
+	string s;
+	unsigned len;
+	while (in >> s >> len) {
+		in.ignore(numeric_limits<streamsize>::max(), '\n');
+		put(g_contigNames, lengths.size(), s);
+		assert(len >= opt::k);
+		lengths.push_back(len - opt::k + 1);
+	}
+	assert(in.eof());
+	assert(!lengths.empty());
+	g_contigNames.lock();
+	return lengths;
+}
+
+/** Read contig lengths. */
+static Lengths readContigLengths(const string& path)
+{
+	ifstream in(path.c_str());
+	assert_good(in, path);
+	return readContigLengths(in);
+}
+
 int main(int argc, char** argv)
 {
 	bool die = false;
@@ -802,6 +859,11 @@ int main(int argc, char** argv)
 				cout << VERSION_MESSAGE;
 				exit(EXIT_SUCCESS);
 		}
+		if (optarg != NULL && !arg.eof()) {
+			cerr << PROGRAM ": invalid option: `-"
+				<< (char)c << optarg << "'\n";
+			exit(EXIT_FAILURE);
+		}
 	}
 
 	if (opt::k <= 0) {
@@ -835,17 +897,19 @@ int main(int argc, char** argv)
 
 	if (opt::verbose > 0)
 		cerr << "Reading `" << argv[optind] << "'..." << endl;
-	g_contigLengths = readContigLengths(argv[optind++]);
-	ContigPathMap originalPathMap = readPaths(argv[optind++]);
+
+	Lengths lengths = readContigLengths(argv[optind++]);
+	ContigPathMap originalPathMap = readPaths(
+			lengths, argv[optind++]);
 
 	removeRepeats(originalPathMap);
 
 	if (!opt::greedy) {
 		// Assemble the path overlap graph.
 		PathGraph pathGraph;
-		buildPathGraph(pathGraph, originalPathMap);
+		buildPathGraph(lengths, pathGraph, originalPathMap);
 		if (!opt::out.empty())
-			assemblePathGraph(pathGraph, originalPathMap);
+			assemblePathGraph(lengths, pathGraph, originalPathMap);
 		exit(EXIT_SUCCESS);
 	}
 
@@ -855,11 +919,13 @@ int main(int argc, char** argv)
 	#pragma omp parallel
 	for (ContigPathMap::iterator it;
 			atomicInc(sharedIt, originalPathMap.end(), it);)
-		extendPaths(it->first, originalPathMap, resultsPathMap);
+		extendPaths(lengths,
+				it->first, originalPathMap, resultsPathMap);
 #else
 	for (ContigPathMap::const_iterator it = originalPathMap.begin();
 			it != originalPathMap.end(); ++it)
-		extendPaths(it->first, originalPathMap, resultsPathMap);
+		extendPaths(lengths,
+				it->first, originalPathMap, resultsPathMap);
 #endif
 	if (gDebugPrint)
 		cout << '\n';
@@ -868,7 +934,8 @@ int main(int argc, char** argv)
 
 	if (gDebugPrint)
 		cout << "\nRemoving redundant contigs\n";
-	set<ContigID> overlaps = removeSubsumedPaths(resultsPathMap);
+	set<ContigID> overlaps = removeSubsumedPaths(lengths,
+			resultsPathMap);
 
 	if (!overlaps.empty() && !repeats.empty()) {
 		// Remove the newly-discovered repeat contigs from the
@@ -880,8 +947,9 @@ int main(int argc, char** argv)
 		// Reassemble the paths that were found to overlap.
 		if (gDebugPrint) {
 			cout << "\nReassembling overlapping contigs:";
-			copy(overlaps.begin(), overlaps.end(),
-					affix_ostream_iterator<ContigID>(cout, " "));
+			for (set<ContigID>::const_iterator it = overlaps.begin();
+					it != overlaps.end(); ++it)
+				cout << ' ' << get(g_contigNames, *it);
 			cout << '\n';
 		}
 
@@ -894,7 +962,8 @@ int main(int argc, char** argv)
 				continue; // subsumed
 			ContigPath old = oldIt->second;
 			resultsPathMap.erase(oldIt);
-			extendPaths(*it, originalPathMap, resultsPathMap);
+			extendPaths(lengths,
+					*it, originalPathMap, resultsPathMap);
 			if (gDebugPrint) {
 				if (resultsPathMap[*it] == old)
 					cout << "no change\n";
@@ -906,11 +975,12 @@ int main(int argc, char** argv)
 			cout << '\n';
 
 		removeRepeats(resultsPathMap);
-		overlaps = removeSubsumedPaths(resultsPathMap);
+		overlaps = removeSubsumedPaths(lengths, resultsPathMap);
 		if (!overlaps.empty() && gDebugPrint) {
 			cout << "\nOverlapping contigs:";
-			copy(overlaps.begin(), overlaps.end(),
-					affix_ostream_iterator<ContigID>(cout, " "));
+			for (set<ContigID>::const_iterator it = overlaps.begin();
+					it != overlaps.end(); ++it)
+				cout << ' ' << get(g_contigNames, *it);
 			cout << '\n';
 		}
 	}
@@ -921,17 +991,24 @@ int main(int argc, char** argv)
 }
 
 /** Return the length of the specified contig in k-mer. */
-static unsigned length(const ContigNode& contig)
+static unsigned getLength(const Lengths& lengths,
+		const ContigNode& u)
 {
-	return contig.ambiguous() ? contig.length()
-		: g_contigLengths.at(contig.id());
+	return u.ambiguous() ? u.length()
+		: lengths.at(u.id());
 }
 
-/** Add the number of k-mer in two contigs. */
-static unsigned addLength(unsigned addend, const ContigNode& contig)
+/** Functor to add the number of k-mer in two contigs. */
+struct AddLength
 {
-	return addend + length(contig);
-}
+	AddLength(const Lengths& lengths) : m_lengths(lengths) { }
+	unsigned operator()(unsigned addend, const ContigNode& u) const
+	{
+		return addend + getLength(m_lengths, u);
+	}
+  private:
+	const Lengths& m_lengths;
+};
 
 /** Attempt to fill in gaps in one path with the sequence from the
  * other path and store the consensus at result if an alignment is
@@ -939,7 +1016,8 @@ static unsigned addLength(unsigned addend, const ContigNode& contig)
  * @return true if an alignment is found
  */
 template <class iterator, class oiterator>
-static bool alignCoordinates(iterator& first1, iterator last1,
+static bool alignCoordinates(const Lengths& lengths,
+		iterator& first1, iterator last1,
 		iterator& first2, iterator last2, oiterator& result)
 {
 	oiterator out = result;
@@ -971,10 +1049,10 @@ static bool alignCoordinates(iterator& first1, iterator last1,
 				ambiguous1 = 0;
 			}
 		} else if (ambiguous1 > 0) {
-			ambiguous1 -= length(*it2);
+			ambiguous1 -= getLength(lengths, *it2);
 			*out++ = *it2++;
 		} else if (ambiguous2 > 0) {
-			ambiguous2 -= length(*it1);
+			ambiguous2 -= getLength(lengths, *it1);
 			*out++ = *it1++;
 		} else
 			assert(false);
@@ -987,7 +1065,11 @@ static bool alignCoordinates(iterator& first1, iterator last1,
 
 	assert(ambiguous1 == 0 || ambiguous2 == 0);
 	int ambiguous = ambiguous1 + ambiguous2;
-	*out++ = ContigNode(max(1, ambiguous), 'N');
+	assert(out > result);
+	if (out[-1].ambiguous())
+		assert(ambiguous == 0);
+	else
+		*out++ = ContigNode(max(1, ambiguous), 'N');
 	first1 = it1;
 	first2 = it2;
 	result = out;
@@ -999,7 +1081,8 @@ static bool alignCoordinates(iterator& first1, iterator last1,
  * @return true if an alignment is found
  */
 template <class iterator, class oiterator>
-static bool buildConsensus(iterator it1, iterator it1e,
+static bool buildConsensus(const Lengths& lengths,
+		iterator it1, iterator it1e,
 		iterator it2, iterator it2e, oiterator& out)
 {
 	iterator it1b = it1 + 1;
@@ -1021,9 +1104,9 @@ static bool buildConsensus(iterator it1, iterator it1e,
 	unsigned ambiguous1 = it1->length();
 	unsigned ambiguous2 = it2a->length();
 	unsigned unambiguous1 = accumulate(it1b, it1e,
-			0, addLength);
+			0, AddLength(lengths));
 	unsigned unambiguous2 = accumulate(it2, it2a,
-			0, addLength);
+			0, AddLength(lengths));
 	if (ambiguous1 < unambiguous2
 			|| ambiguous2 < unambiguous1) {
 		// Two gaps overlap and either of the gaps is smaller
@@ -1047,7 +1130,7 @@ static bool buildConsensus(iterator it1, iterator it1e,
  * @return true if an alignment is found
  */
 template <class iterator, class oiterator>
-static bool alignAtSeed(
+static bool alignAtSeed(const Lengths& lengths,
 		iterator& it1, iterator it1e, iterator last1,
 		iterator& it2, iterator last2, oiterator& out)
 {
@@ -1064,8 +1147,8 @@ static bool alignAtSeed(
 	for (iterator it2e = it2;
 			(it2e = find(it2e, last2, *it1e)) != last2; ++it2e) {
 		oiterator myOut = out;
-		if (buildConsensus(it1, it1e, it2, it2e, myOut)
-				&& align(it1e, last1, it2e, last2, myOut)) {
+		if (buildConsensus(lengths, it1, it1e, it2, it2e, myOut)
+				&& align(lengths, it1e, last1, it2e, last2, myOut)) {
 			unsigned len = myOut - out;
 			if (len <= bestLen) {
 				bestLen = len;
@@ -1074,7 +1157,8 @@ static bool alignAtSeed(
 		}
 	}
 	if (bestLen != UINT_MAX) {
-		bool good = buildConsensus(it1, it1e, it2, bestIt2e, out);
+		bool good = buildConsensus(lengths,
+				it1, it1e, it2, bestIt2e, out);
 		assert(good);
 		it1 = it1e;
 		it2 = bestIt2e;
@@ -1088,7 +1172,8 @@ static bool alignAtSeed(
  * @return true if an alignment is found
  */
 template <class iterator, class oiterator>
-static bool alignAmbiguous(iterator& it1, iterator last1,
+static bool alignAmbiguous(const Lengths& lengths,
+		iterator& it1, iterator last1,
 		iterator& it2, iterator last2, oiterator& out)
 {
 	assert(it1 != last1);
@@ -1100,13 +1185,13 @@ static bool alignAmbiguous(iterator& it1, iterator last1,
 	for (iterator it1e = it1; it1e != last1; ++it1e) {
 		if (it1e->ambiguous())
 			continue;
-		if (alignAtSeed(it1, it1e, last1, it2, last2, out))
+		if (alignAtSeed(lengths, it1, it1e, last1, it2, last2, out))
 			return true;
 	}
 
 	// No valid seeded alignment. Check whether path2 fits entirely
 	// within the gap of path1.
-	return alignCoordinates(it1, last1, it2, last2, out);
+	return alignCoordinates(lengths, it1, last1, it2, last2, out);
 }
 
 /** Align the next pair of contigs.
@@ -1114,7 +1199,8 @@ static bool alignAmbiguous(iterator& it1, iterator last1,
  * @return true if an alignment is found
  */
 template <class iterator, class oiterator>
-static bool alignOne(iterator& it1, iterator last1,
+static bool alignOne(const Lengths& lengths,
+		iterator& it1, iterator last1,
 		iterator& it2, iterator last2, oiterator& out)
 {
 	// Check for a trivial alignment.
@@ -1138,12 +1224,13 @@ static bool alignOne(iterator& it1, iterator last1,
 	return
 		it1->ambiguous() && it2->ambiguous()
 			? (it1->length() > it2->length()
-				? alignAmbiguous(it1, last1, it2, last2, out)
-				: alignAmbiguous(it2, last2, it1, last1, out))
+				? alignAmbiguous(lengths, it1, last1, it2, last2, out)
+				: alignAmbiguous(lengths, it2, last2, it1, last1, out)
+			)
 		: it1->ambiguous()
-			? alignAmbiguous(it1, last1, it2, last2, out)
+			? alignAmbiguous(lengths, it1, last1, it2, last2, out)
 		: it2->ambiguous()
-			? alignAmbiguous(it2, last2, it1, last1, out)
+			? alignAmbiguous(lengths, it2, last2, it1, last1, out)
 			: (*out++ = *it1, *it1++ == *it2++);
 }
 
@@ -1153,13 +1240,14 @@ static bool alignOne(iterator& it1, iterator last1,
  * or zero otherwise
  */
 template <class iterator, class oiterator>
-static dir_type align(iterator it1, iterator last1,
+static dir_type align(const Lengths& lengths,
+		iterator it1, iterator last1,
 		iterator it2, iterator last2, oiterator& out)
 {
 	assert(it1 != last1);
 	assert(it2 != last2);
 	while (it1 != last1 && it2 != last2)
-		if (!alignOne(it1, last1, it2, last2, out))
+		if (!alignOne(lengths, it1, last1, it2, last2, out))
 			return DIR_X;
 	assert(it1 == last1 || it2 == last2);
 	out = copy(it1, last1, out);
@@ -1175,7 +1263,8 @@ static dir_type align(iterator it1, iterator last1,
  * @param[out] orientation the orientation of the alignment
  * @return the consensus sequence
  */
-static ContigPath align(const ContigPath& p1, const ContigPath& p2,
+static ContigPath align(const Lengths& lengths,
+		const ContigPath& p1, const ContigPath& p2,
 		ContigPath::const_iterator pivot1,
 		ContigPath::const_iterator pivot2,
 		dir_type& orientation)
@@ -1186,13 +1275,15 @@ static ContigPath align(const ContigPath& p1, const ContigPath& p2,
 		rit2 = ContigPath::const_reverse_iterator(pivot2+1);
 	ContigPath alignmentr(p1.rend() - rit1 + p2.rend() - rit2);
 	ContigPath::iterator rout = alignmentr.begin();
-	dir_type alignedr = align(rit1, p1.rend(), rit2, p2.rend(), rout);
+	dir_type alignedr = align(lengths,
+			rit1, p1.rend(), rit2, p2.rend(), rout);
 	alignmentr.erase(rout, alignmentr.end());
 
 	ContigPath::const_iterator it1 = pivot1, it2 = pivot2;
 	ContigPath alignmentf(p1.end() - it1 + p2.end() - it2);
 	ContigPath::iterator fout = alignmentf.begin();
-	dir_type alignedf = align(it1, p1.end(), it2, p2.end(), fout);
+	dir_type alignedf = align(lengths,
+			it1, p1.end(), it2, p2.end(), fout);
 	alignmentf.erase(fout, alignmentf.end());
 
 	ContigPath consensus;
@@ -1242,7 +1333,7 @@ static pair<ContigNode, bool> findPivot(
 			it != path2.end(); ++it) {
 		if (it->ambiguous())
 			continue;
-		if (count(it, path2.end(), *it) == 1
+		if (count(path2.begin(), path2.end(), *it) == 1
 				&& count(path1.begin(), path1.end(), *it) == 1)
 			return make_pair(*it, true);
 	}
@@ -1253,7 +1344,7 @@ static pair<ContigNode, bool> findPivot(
  * @param[out] orientation the orientation of the alignment
  * @return the consensus sequence
  */
-static ContigPath align(
+static ContigPath align(const Lengths& lengths,
 		const ContigPath& path1, const ContigPath& path2,
 		ContigNode pivot, dir_type& orientation)
 {
@@ -1311,7 +1402,8 @@ static ContigPath align(
 			// trivial alignment, which we'll ignore.
 			continue;
 		}
-		consensus = align(path1, path2, it1, it2, orientation);
+		consensus = align(lengths,
+				path1, path2, it1, it2, orientation);
 		if (!consensus.empty())
 			return consensus;
 	}
@@ -1321,10 +1413,10 @@ static ContigPath align(
 /** Find an equivalent region of the two specified paths.
  * @return the consensus sequence
  */
-static ContigPath align(
+static ContigPath align(const Lengths& lengths,
 		const ContigPath& path1, const ContigPath& path2,
 		ContigNode pivot)
 {
 	dir_type orientation;
-	return align(path1, path2, pivot, orientation);
+	return align(lengths, path1, path2, pivot, orientation);
 }
