@@ -1,15 +1,3 @@
-/**
- *
- * Resolve ambiguity ("N"s) in paths after MergePaths
- * by using pairwise/multiple sequence alignment
- * (pairwise: Needleman-Wunsch)
- * (multi: based on Dialign program by A.R.Subramanian)
- *
- * Author: Rong She (rshe@bcgsc.ca)
- *
- * Last Modified: July 2010
- */
-
 #include "dialign.h"
 #include "config.h"
 #include "Common/Options.h"
@@ -28,6 +16,7 @@
 #include "Graph/GraphIO.h"
 #include <algorithm>
 #include <cctype>
+#include <climits>
 #include <cstdlib>
 #include <fstream>
 #include <getopt.h>
@@ -44,9 +33,9 @@ using namespace std;
 
 static const char VERSION_MESSAGE[] =
 PROGRAM " (" PACKAGE_NAME ") " VERSION "\n"
-"Written by Rong She and Shaun Jackman.\n"
+"Written by Shaun Jackman and Rong She.\n"
 "\n"
-"Copyright 2011 Canada's Michael Smith Genome Science Centre\n";
+"Copyright 2012 Canada's Michael Smith Genome Science Centre\n";
 
 static const char USAGE_MESSAGE[] =
 "Usage: " PROGRAM " [OPTION]... FASTA ADJ PATH\n"
@@ -95,7 +84,7 @@ namespace opt {
 	/** The the number of bases to continue the constrained search of
 	 * the graph beyond the size of the ambiguous gap in the path.
 	 */
-	static unsigned distanceError = 6;
+	unsigned distanceError = 6;
 }
 
 static const char shortopts[] = "d:k:o:s:g:a:p:vD:M:P:";
@@ -160,15 +149,6 @@ typedef vector<const_string> Contigs;
 static Contigs g_contigs;
 AmbPath2Contig g_ambpath_contig;
 
-#if 0
-static set< pair<ContigNode, ContigNode> > g_edges_irregular;
-#endif
-
-#if 0
-static double g_coverage_mean;
-static double g_coverage_variance;
-#endif
-
 /** Return the sequence of the specified contig node. The sequence
  * may be ambiguous or reverse complemented.
  */
@@ -185,14 +165,28 @@ static const Sequence getSequence(ContigNode id)
 	}
 }
 
+/** Return the distance from vertex u to v. */
+static int getDistance(const Graph& g,
+		graph_traits<Graph>::vertex_descriptor u,
+		graph_traits<Graph>::vertex_descriptor v)
+{
+	typedef graph_traits<Graph>::edge_descriptor E;
+	pair<E, bool> e = edge(u, v, g);
+	assert(e.second);
+	return g[e.first].distance;
+}
+
 /** Read contig paths from the specified file.
- * @param ids [out] the string ID of the paths
- * @param isAmb [out] whether the path contains a gap
+ * @param[in] inPath the filename of the contig paths
+ * @param[out] ids the string ID of the paths
+ * @param[out] isAmb whether the path contains a gap
  */
-static ContigPaths readPath(const string& inPath,
+static ContigPaths readPaths(const string& inPath,
 	vector<string>& ids, vector<bool>& isAmb)
 {
-	assert(ids.empty()); //this seed is contigID of the path
+	typedef graph_traits<Graph>::vertex_descriptor V;
+
+	assert(ids.empty());
 	assert(isAmb.empty());
 	assert(g_ambpath_contig.empty());
 	ifstream fin(inPath.c_str());
@@ -205,80 +199,30 @@ static ContigPaths readPath(const string& inPath,
 	ContigPaths paths;
 	string id;
 	Path path;
-	Path::iterator prev, next;
 	while (in >> id >> path) {
 		paths.push_back(path);
 		ids.push_back(id);
+		isAmb.push_back(false);
 
-		if (path.size() <= 2) {
-			isAmb.push_back(false);
+		if (path.size() <= 2)
 			continue;
-		}
-
-		/* contig with Ns must have prev and next */
-		bool cur_is_amb = false;
-		prev = path.begin();
-		next = path.begin();
-		Path::iterator it = path.begin();
-		it++; next++; next++;
-		for (; next != path.end(); it++, prev++, next++) {
-			if (it->ambiguous()) {
-				cur_is_amb = true;
-				/* add the entry to g_ambpath_contig
-				(value 0: unprocessed/no proper paths) */
+		for (Path::iterator it = path.begin() + 2;
+				it != path.end(); ++it) {
+			ContigPath::value_type t = it[-2], u = it[-1], v = it[0];
+			if (u.ambiguous()) {
+				assert(!t.ambiguous());
+				assert(!v.ambiguous());
 				g_ambpath_contig.insert(AmbPath2Contig::value_type(
-					AmbPathConstraint(*prev, *next, -it->id()),
+					AmbPathConstraint(t, v, u.length()),
 					ContigPath()));
+				isAmb.back() = true;
 			}
 		}
-		isAmb.push_back(cur_is_amb);
 	}
 	assert(in.eof());
 	return paths;
 }
 
-#if 0
-/** Read ambiguous path info from SimpleGraph verbose output.
- * format:
- * 275+ 1181+ 1287- 614+
- * 275+ 1198+ 1287- 614+
- * 275+ 73N 1287- 614+
- *
- * Store path info in "g_amb_paths"
- */
-static void readAmbPaths(const string& ambPath)
-{
-	ifstream fin(ambPath.c_str());
-	if (opt::verbose > 0)
-		cerr << "Reading `" << ambPath << "'..." << endl;
-	if (ambPath != "-")
-		assert_good(fin, ambPath);
-	istream& in = ambPath == "-" ? cin : fin;
-
-	vector<Path> paths;
-	string id;
-	Path path;
-	bool isAmbPath;
-	while (in >> path) {
-		isAmbPath = false;
-		for (Path::iterator it = path.begin();
-			it != path.end(); it++)
-			if (it->ambiguous()) {
-				isAmbPath = true;
-				assert(!paths.empty());
-				g_amb_paths.insert(
-					map<Path, vector<Path> >::value_type(
-					path, paths));
-				paths.clear();
-				break;
-			}
-		if (!isAmbPath)
-			paths.push_back(path);
-	}
-	assert(in.eof());
-}
-#endif
-
 /** Mark every contig in path as seen. */
 static void markSeen(vector<bool>& seen, const ContigPath& path,
 		bool flag)
@@ -298,110 +242,18 @@ static void markSeen(vector<bool>& seen, const vector<Path>& paths,
 		markSeen(seen, *it, flag);
 }
 
-#if 0
-/* read in adj file, store which contigs have irregular overlaps
- * (>=k-1 or non-exact match)
- */
-static void ReadAdj(const string& irregularAdj)
+/** A new vertex and two edges that will be added to the graph. */
+struct NewVertex
 {
-	ifstream in_adj(irregularAdj.c_str());
-	assert_good(in_adj, irregularAdj);
-	while (!in_adj.eof()) {
-		char line[65536];
-		in_adj.getline(line, 65536);
-		string s = line;
-		vector<string> strs;
-		size_t start_pos = 0, delimit_pos;
-		while (start_pos < s.length()
-			&& (delimit_pos = s.find(';', start_pos))
-			!= string::npos) {
-			//get first, second parts
-			if (delimit_pos > start_pos + 1)
-				strs.push_back(s.substr(start_pos,
-					delimit_pos-start_pos));
-			else
-				strs.push_back("");
-			start_pos = delimit_pos+1;
-		}
-		//ignore lines that do not have two ';'s
-		if (strs.size() < 2)
-			continue;
-		//get the last part
-		if (start_pos + 1 < s.length())
-			strs.push_back(s.substr(start_pos));
-		else
-			strs.push_back("");
-		assert(strs.size() == 3);
-		size_t space_pos;
-		assert ((space_pos = strs[0].find(' '))
-			!= string::npos);
-		string t_id = strs[0].substr(0,space_pos);
-		if (!strs[1].empty()) {
-			start_pos = 0;
-			while (start_pos < strs[1].length()
-				&& (delimit_pos = strs[1].find_first_of(" \t", start_pos))
-				!= string::npos) {
-				if (delimit_pos > start_pos) {
-					string h_id =
-						strs[1].substr(start_pos, delimit_pos-start_pos-1);
-					bool h_sense =
-						(strs[1][delimit_pos-1] == '+'? false : true);
-
-					g_edges_irregular.insert(pair<ContigNode, ContigNode>(
-						ContigNode(t_id, false),
-						ContigNode(h_id, h_sense) ));
-
-					g_edges_irregular.insert(pair<ContigNode, ContigNode>(
-						ContigNode(h_id, !h_sense),
-						ContigNode(t_id, true) ));
-				}
-				start_pos = delimit_pos + 1;
-			}
-		}
-		if (!strs[2].empty()) {
-			start_pos = 0;
-			while (start_pos < strs[2].length()
-				&& (delimit_pos = strs[2].find_first_of(' ', start_pos))
-				!= string::npos) {
-				if (delimit_pos > start_pos) {
-					string h_id =
-						strs[2].substr(start_pos, delimit_pos-start_pos-1);
-					bool h_sense =
-						(strs[2][delimit_pos-1] == '+'? false : true);
-
-					g_edges_irregular.insert(pair<ContigNode, ContigNode>(
-						ContigNode(h_id, h_sense),
-						ContigNode(t_id, false) ));
-
-					g_edges_irregular.insert(pair<ContigNode, ContigNode>(
-						ContigNode(t_id, true),
-						ContigNode(h_id, !h_sense) ));
-
-				}
-				start_pos = delimit_pos + 1;
-			}
-		}
-	}
-	in_adj.close();
-	if (opt::verbose > 1) {
-		cerr << "g_edges_irregular:\n";
-		set< pair<ContigNode, ContigNode> >::const_iterator it =
-			g_edges_irregular.begin();
-		for (; it != g_edges_irregular.end(); it++)
-			cerr << it->first << " to " << it->second << '\n';
-	}
-}
-#endif
-
-struct NewVertex {
-	Graph::vertex_descriptor t, u, v;
-	Graph::vertex_property_type vp;
-	NewVertex(
-			Graph::vertex_descriptor t,
-			Graph::vertex_descriptor u,
-			Graph::vertex_descriptor v,
-			const Graph::vertex_property_type& vp)
-		: t(t), u(u), v(v), vp(vp) { }
+	typedef graph_traits<Graph>::vertex_descriptor V;
+	typedef vertex_property<Graph>::type VP;
+	typedef edge_property<Graph>::type EP;
+	V t, u, v;
+	VP vpu;
+	EP eptu, epuv;
+	NewVertex(V t, V u, V v, const VP& vpu,
+			const EP& eptu, const EP& epuv)
+		: t(t), u(u), v(v), vpu(vpu), eptu(eptu), epuv(epuv) { }
 };
 typedef vector<NewVertex> NewVertices;
 
@@ -409,7 +261,7 @@ typedef vector<NewVertex> NewVertices;
 static NewVertices g_newVertices;
 
 /** Output a new contig. */
-static ContigID outputNewContig(
+static ContigNode outputNewContig(const Graph& g,
 	const vector<Path>& solutions,
 	size_t longestPrefix, size_t longestSuffix,
 	const Sequence& seq, const unsigned coverage,
@@ -418,17 +270,15 @@ static ContigID outputNewContig(
 	assert(!solutions.empty());
 	assert(longestPrefix > 0);
 	assert(longestSuffix > 0);
-	ContigID id(ContigID::create());
 
-	// Record the newly-created contig to be added to the graph later.
-	g_newVertices.push_back(NewVertex(
-				*(solutions[0].begin() + longestPrefix - 1),
-				ContigNode(id, false),
-				*(solutions[0].rbegin() + longestSuffix - 1),
-				ContigProperties(seq.length(), coverage)));
+	size_t numContigs = num_vertices(g) / 2;
+	ContigNode u(numContigs + g_newVertices.size(), false);
+	string name = createContigName();
+	put(vertex_name, g, u, name);
+	out << '>' << name
+		<< ' ' << seq.length() << ' ' << coverage << ' ';
 
-	out << '>' << id.str() << ' ' << seq.length()
-		<< ' ' << coverage << ' ';
+	int dtu = INT_MAX, duv = INT_MAX;
 	for (vector<Path>::const_iterator it = solutions.begin();
 			it != solutions.end(); it++) {
 		if (it != solutions.begin())
@@ -439,14 +289,27 @@ static ContigID outputNewContig(
 			last = path.end() - longestSuffix;
 		assert(first <= last);
 		if (first < last) {
-			copy(first, last - 1,
-					ostream_iterator<ContigNode>(out, ","));
-			out << *(last - 1);
+			ContigPath::const_iterator it = first;
+			out << get(vertex_name, g, *it);
+			for (++it; it != last; ++it)
+				out << ',' << get(vertex_name, g, *it);
+			dtu = min(dtu, getDistance(g, first[-1], first[0]));
+			duv = min(duv, getDistance(g, last[-1], last[0]));
 		} else
 			out << '*';
 	}
 	out << '\n' << seq << '\n';
-	return id;
+	assert(dtu < INT_MAX);
+	assert(duv < INT_MAX);
+
+	// Record the newly-created contig to be added to the graph later.
+	g_newVertices.push_back(NewVertex(
+				*(solutions[0].begin() + longestPrefix - 1),
+				u,
+				*(solutions[0].rbegin() + longestSuffix - 1),
+				ContigProperties(seq.length(), coverage),
+				dtu, duv));
+	return u;
 }
 
 /** Return a consensus sequence of a and b.
@@ -474,47 +337,12 @@ static string createConsensus(const Sequence& a, const Sequence& b)
 	return s;
 }
 
-#if 0
-/** Merge the specified two contigs,
- * With modified Overlap, we no longer assume overlap is k-1, instead,
- * do a smith-waterman alignment between contigs to determine their
- * overlap.
-*/
-static overlap_align mergeContigs_SW(Sequence& seq, const Sequence& s,
-	const ContigNode& node, const Path& path)
-{
-	vector<overlap_align> overlaps;
-	overlaps.reserve(1); /* only need the longest overlap */
-	do {
-		unsigned len = min(seq.length(), s.length());
-		assert(len > opt::k - 1);
-		alignOverlap(seq.substr(seq.length()-len),
-			s.substr(0, len),
-			seq.length()-len, overlaps, false,
-			opt::verbose > 1);
-	} while (overlaps.empty() && chomp(seq, 'n'));
-
-	if (overlaps.empty()) {
-		cerr << "warning: the head of `" << node << "' "
-			"does not match the tail of the previous contig\n"
-			<< seq << '\n' << s << '\n' << path << endl;
-		seq += 'n';
-		seq += s;
-		return overlap_align();
-	} else {
-		seq.erase(overlaps[0].overlap_t_pos);
-		seq += overlaps[0].overlap_str;
-		seq += Sequence(s, overlaps[0].overlap_h_pos+1);
-		return overlaps[0];
-	}
-}
-#endif
-
 /** Merge the specified two contigs, default overlap is k-1,
  * generate a consensus sequence of the overlapping region. The result
  * is stored in the first argument.
  */
-static void mergeContigs(unsigned overlap, Sequence& seq,
+static void mergeContigs(const Graph& g,
+		unsigned overlap, Sequence& seq,
 		const Sequence& s, const ContigNode& node, const Path& path)
 {
 	assert(s.length() > overlap);
@@ -527,8 +355,9 @@ static void mergeContigs(unsigned overlap, Sequence& seq,
 		o = createConsensus(ao, bo);
 	} while (o.empty() && chomp(seq, 'n'));
 	if (o.empty()) {
-		cerr << "warning: the head of `" << node << "' "
-			"does not match the tail of the previous contig\n"
+		cerr << "warning: the head of "
+			<< get(vertex_name, g, node)
+			<< " does not match the tail of the previous contig\n"
 			<< ao << '\n' << bo << '\n' << path << endl;
 		seq += 'n';
 		seq += s;
@@ -548,41 +377,17 @@ static Sequence mergePath(const Graph&g, const Path& path)
 		if (seq.empty()) {
 			seq = getSequence(*it);
 		} else {
-#if 0
-			if (g_edges_irregular.find(
-					pair<ContigNode, ContigNode>(
-					*prev_it, *it))
-					== g_edges_irregular.end())
-				mergeContigs(seq, getSequence(*it),
-					*it, path);
-			else
-				mergeContigs_SW(seq, getSequence(*it),
-					*it, path);
-#else
 			int d = get(edge_bundle, g, *(it-1), *it).distance;
 			assert(d < 0);
 			unsigned overlap = -d;
-			mergeContigs(overlap, seq, getSequence(*it), *it, path);
-#endif
+			mergeContigs(g, overlap, seq,
+					getSequence(*it), *it, path);
 		}
 		prev_it = it;
 	}
 	return seq;
 }
 
-#if 0
-/** Validate path coverage, at 95% confidence interval. */
-static bool validCoverage(unsigned pathLen, unsigned pathCover)
-{
-	double cover_mean
-		= (double)pathCover / (pathLen + opt::k - 1);
-	double cover_deviation
-		= sqrt(g_coverage_variance / (pathLen + opt::k - 1));
-	return cover_mean <= g_coverage_mean + 1.96*cover_deviation
-		&& cover_mean >= g_coverage_mean - 1.96*cover_deviation;
-}
-#endif
-
 /** Calculate the ContigProperties of a path. */
 static ContigProperties calculatePathProperties(const Graph& g,
 		const ContigPath& path)
@@ -590,9 +395,9 @@ static ContigProperties calculatePathProperties(const Graph& g,
 	return addProp(g, path.begin(), path.end());
 }
 
-/* Resolve ambiguous region using pairwise alignment (needleman-wunsch)
- * ('solutions' contain exactly two paths, from a source contig
- * to a dest contig)
+/* Resolve ambiguous region using pairwise alignment
+ * (Needleman-Wunsch) ('solutions' contain exactly two paths, from a
+ * source contig to a dest contig)
  */
 static ContigPath alignPair(const Graph& g,
 		const ContigPaths& solutions, ofstream& out)
@@ -616,20 +421,20 @@ static ContigPath alignPair(const Graph& g,
 
 		unsigned match = opt::k - 1;
 		float identity = (float)match / consensus.size();
-		if (opt::verbose > 1)
+		if (opt::verbose > 2)
 			cerr << consensus << '\n';
-		if (opt::verbose > 0)
+		if (opt::verbose > 1)
 			cerr << identity
 				<< (identity < opt::identity ? " (too low)\n" : "\n");
 		if (identity < opt::identity)
 			return ContigPath();
 
 		unsigned coverage = calculatePathProperties(g, sol).coverage;
-		ContigID id
-			= outputNewContig(solutions, 1, 1, consensus, coverage, out);
+		ContigNode u = outputNewContig(g,
+				solutions, 1, 1, consensus, coverage, out);
 		ContigPath path;
 		path.push_back(solutions.front().front());
-		path.push_back(ContigNode(id, false));
+		path.push_back(u);
 		path.push_back(solutions.front().back());
 		return path;
 	}
@@ -637,18 +442,30 @@ static ContigPath alignPair(const Graph& g,
 	Sequence fstPathContig(mergePath(g, fstSol));
 	Sequence sndPathContig(mergePath(g, sndSol));
 	if (fstPathContig == sndPathContig) {
-		// A perfect match must be caused by palindrome.
-		assert(fstSol.size() == sndSol.size());
-		typedef ContigPath::const_iterator It;
-		pair<It, It> it = mismatch(
-				fstSol.begin(), fstSol.end(), sndSol.begin());
-		assert(it.first != fstSol.end());
-		assert(it.second != sndSol.end());
-		assert(*it.first == ~*it.second);
-		assert(std::equal(it.first+1, It(fstSol.end()), it.second+1));
-		if (opt::verbose > 0)
-			cerr << "Palindrome: " << ContigID(*it.first) << '\n';
-		return solutions[0];
+		// These two paths have identical sequence.
+		if (fstSol.size() == sndSol.size()) {
+			// A perfect match must be caused by palindrome.
+			typedef ContigPath::const_iterator It;
+			pair<It, It> it = mismatch(
+					fstSol.begin(), fstSol.end(), sndSol.begin());
+			assert(it.first != fstSol.end());
+			assert(it.second != sndSol.end());
+			assert(*it.first
+					== get(vertex_complement, g, *it.second));
+			assert(equal(it.first+1, It(fstSol.end()), it.second+1));
+			if (opt::verbose > 1)
+				cerr << "Palindrome: "
+					<< get(vertex_contig_name, g, *it.first) << '\n';
+			return solutions[0];
+		} else {
+			// The paths are different lengths.
+			cerr << PROGRAM ": warning: "
+				"Two paths have identical sequence, which may be "
+				"caused by a transitive edge in the overlap graph.\n"
+				<< '\t' << fstSol << '\n'
+				<< '\t' << sndSol << '\n';
+			return solutions[fstSol.size() > sndSol.size() ? 0 : 1];
+		}
 	}
 
 	unsigned minLength = min(
@@ -657,7 +474,7 @@ static ContigPath alignPair(const Graph& g,
 			fstPathContig.length(), sndPathContig.length());
 	float lengthRatio = (float)minLength / maxLength;
 	if (lengthRatio < opt::identity) {
-		if (opt::verbose > 0)
+		if (opt::verbose > 1)
 			cerr << minLength << '\t' << maxLength
 				<< '\t' << lengthRatio << "\t(different length)\n";
 		return ContigPath();
@@ -667,9 +484,9 @@ static ContigPath alignPair(const Graph& g,
 	unsigned match = alignGlobal(fstPathContig, sndPathContig,
 		   	align);
 	float identity = (float)match / align.size();
-	if (opt::verbose > 1)
+	if (opt::verbose > 2)
 		cerr << align;
-	if (opt::verbose > 0)
+	if (opt::verbose > 1)
 		cerr << identity
 			<< (identity < opt::identity ? " (too low)\n" : "\n");
 	if (identity < opt::identity)
@@ -677,11 +494,11 @@ static ContigPath alignPair(const Graph& g,
 
 	unsigned coverage = calculatePathProperties(g, fstSol).coverage
 		+ calculatePathProperties(g, sndSol).coverage;
-	ContigID id = outputNewContig(solutions, 1, 1,
+	ContigNode u = outputNewContig(g, solutions, 1, 1,
 			align.consensus(), coverage, out);
 	ContigPath path;
 	path.push_back(solutions.front().front());
-	path.push_back(ContigNode(id, false));
+	path.push_back(u);
 	path.push_back(solutions.front().back());
 	return path;
 }
@@ -703,7 +520,8 @@ static ContigPath alignMulti(const Graph& g,
 	Path vppath;
 	size_t longestPrefix;
 	bool commonPrefix = true;
-	for (longestPrefix = 0; longestPrefix < min_len; longestPrefix++) {
+	for (longestPrefix = 0;
+			longestPrefix < min_len; longestPrefix++) {
 		const ContigNode& common_path_node = firstSol[longestPrefix];
 		for (vector<Path>::const_iterator solIter = solutions.begin();
 				solIter != solutions.end(); ++solIter) {
@@ -743,7 +561,7 @@ static ContigPath alignMulti(const Graph& g,
 	}
 	reverse(vspath.begin(), vspath.end());
 
-	if (opt::verbose > 0 && vppath.size() + vspath.size() > 2)
+	if (opt::verbose > 1 && vppath.size() + vspath.size() > 2)
 		cerr << vppath << " * " << vspath << '\n';
 
 	// Get sequence of ambiguous region in paths
@@ -772,7 +590,7 @@ static ContigPath alignMulti(const Graph& g,
 	unsigned maxLength = *max_element(lengths.begin(), lengths.end());
 	float lengthRatio = (float)minLength / maxLength;
 	if (lengthRatio < opt::identity) {
-		if (opt::verbose > 0)
+		if (opt::verbose > 1)
 			cerr << minLength << '\t' << maxLength
 				<< '\t' << lengthRatio << "\t(different length)\n";
 		return ContigPath();
@@ -781,10 +599,10 @@ static ContigPath alignMulti(const Graph& g,
 	string alignment;
 	unsigned matches;
 	Sequence consensus = dialign(amb_seqs, alignment, matches);
-	if (opt::verbose > 1)
+	if (opt::verbose > 2)
 	   	cerr << alignment << consensus << '\n';
 	float identity = (float)matches / consensus.size();
-	if (opt::verbose > 0)
+	if (opt::verbose > 1)
 		cerr << identity
 			<< (identity < opt::identity ? " (too low)\n" : "\n");
 	if (identity < opt::identity)
@@ -792,11 +610,15 @@ static ContigPath alignMulti(const Graph& g,
 
 	if (identity == 1) {
 		// A perfect match must be caused by two palindromes.
-		ContigID palindrome0 = solutions[0][longestPrefix];
-		ContigID palindrome1 = solutions[0].rbegin()[longestSuffix];
-		if (opt::verbose > 0)
-			cerr << "Palindrome: " << palindrome0 << '\n'
-				<< "Palindrome: " << palindrome1 << '\n';
+		ContigID palindrome0
+			= solutions[0][longestPrefix].contigIndex();
+		ContigID palindrome1
+			= solutions[0].rbegin()[longestSuffix].contigIndex();
+		if (opt::verbose > 1)
+			cerr << "Palindrome: "
+				<< get(g_contigNames, palindrome0) << '\n'
+				<< "Palindrome: "
+				<< get(g_contigNames, palindrome1) << '\n';
 #ifndef NDEBUG
 		string s0 = getSequence(ContigNode(palindrome0, false));
 		string s1 = getSequence(ContigNode(palindrome1, false));
@@ -805,8 +627,8 @@ static ContigPath alignMulti(const Graph& g,
 		for (vector<Path>::const_iterator it = solutions.begin();
 				it != solutions.end(); ++it) {
 			const ContigPath& path = *it;
-			assert(ContigID(path[longestPrefix]) == palindrome0);
-			assert(ContigID(path.rbegin()[longestSuffix])
+			assert(path[longestPrefix].contigIndex() == palindrome0);
+			assert(path.rbegin()[longestSuffix].contigIndex()
 					== palindrome1);
 			assert(path.size() == solutions[0].size());
 		}
@@ -814,10 +636,10 @@ static ContigPath alignMulti(const Graph& g,
 		return solutions[0];
 	}
 
-	ContigID id = outputNewContig(solutions,
+	ContigNode u = outputNewContig(g, solutions,
 		longestPrefix, longestSuffix, consensus, coverage, out);
 	ContigPath path(vppath);
-	path.push_back(ContigNode(id, false));
+	path.push_back(u);
 	path.insert(path.end(), vspath.begin(), vspath.end());
 	return path;
 }
@@ -828,44 +650,23 @@ static ContigPath alignMulti(const Graph& g,
 static ContigPath align(const Graph& g, const vector<Path>& sequences,
 		ofstream& out)
 {
-	assert(!sequences.empty());
-	switch (sequences.size()) {
-	  case 1:
-		return sequences.front();
-	  case 2:
-		return alignPair(g, sequences, out);
-	  default:
-		return alignMulti(g, sequences, out);
-	}
+	assert(sequences.size() > 1);
+	return sequences.size() == 2
+		? alignPair(g, sequences, out)
+		: alignMulti(g, sequences, out);
 }
 
-#if 0
-static void calculateCoverageStatistics(const Graph& g)
-{
-	double coverage = 0, variance = 0;
-	unsigned num = g.num_vertices() / 2;
-	for (unsigned i = 0; i < num; ++i) {
-		ContigNode u(i, false);
-		double c = (double)g[u].coverage / (g[u].length - opt::k + 1);
-		coverage += c;
-		variance += c * c;
-	}
-	g_coverage_mean = coverage / num;
-	g_coverage_variance = (variance - 2 * g_coverage_mean * coverage)
-		/ num + g_coverage_mean * g_coverage_mean;
-}
-#endif
-
 /** Return the consensus sequence of the specified gap. */
 static ContigPath fillGap(const Graph& g,
 		const AmbPathConstraint& apConstraint,
 		vector<bool>& seen,
 		ofstream& outFasta)
 {
-	if (opt::verbose > 0)
-		cerr << "\n* " << apConstraint.source << ' '
+	if (opt::verbose > 1)
+		cerr << "\n* "
+			<< get(vertex_name, g, apConstraint.source) << ' '
 			<< apConstraint.dist << "N "
-			<< apConstraint.dest << '\n';
+			<< get(vertex_name, g, apConstraint.dest) << '\n';
 
 	Constraints constraints;
 	constraints.push_back(Constraint(apConstraint.dest,
@@ -885,30 +686,33 @@ static ContigPath fillGap(const Graph& g,
 	bool tooManySolutions = solutions.size() > opt::numBranches;
 	if (tooComplex) {
 		stats.tooComplex++;
-		if (opt::verbose > 0)
+		if (opt::verbose > 1)
 			cerr << solutions.size() << " paths (too complex)\n";
 	} else if (tooManySolutions) {
 		stats.numTooManySolutions++;
-		if (opt::verbose > 0)
+		if (opt::verbose > 1)
 			cerr << solutions.size() << " paths (too many)\n";
 	} else if (solutions.empty()) {
 		stats.numNoSolutions++;
-		if (opt::verbose > 0)
-			cerr << "no paths\n";
-	} else {
 		if (opt::verbose > 1)
+			cerr << "no paths\n";
+	} else if (solutions.size() == 1) {
+		if (opt::verbose > 1)
+			cerr << "1 path\n" << solutions.front() << '\n';
+		stats.numMerged++;
+	} else {
+		assert(solutions.size() > 1);
+		if (opt::verbose > 2)
 			copy(solutions.begin(), solutions.end(),
 					ostream_iterator<ContigPath>(cerr, "\n"));
-		else if (opt::verbose > 0)
+		else if (opt::verbose > 1)
 			cerr << solutions.size() << " paths\n";
 		consensus = align(g, solutions, outFasta);
 		if (!consensus.empty()) {
 			stats.numMerged++;
-			if (solutions.size() > 1) {
-				// Mark contigs that are used in a consensus.
-				markSeen(seen, solutions, true);
-			}
-			if (opt::verbose > 0)
+			// Mark contigs that are used in a consensus.
+			markSeen(seen, solutions, true);
+			if (opt::verbose > 1)
 				cerr << consensus << '\n';
 		} else
 			stats.notMerged++;
@@ -951,6 +755,11 @@ int main(int argc, char** argv)
 			cout << VERSION_MESSAGE;
 			exit(EXIT_SUCCESS);
 		}
+		if (optarg != NULL && !arg.eof()) {
+			cerr << PROGRAM ": invalid option: `-"
+				<< (char)c << optarg << "'\n";
+			exit(EXIT_FAILURE);
+		}
 	}
 
 	if (opt::k <= 0) {
@@ -991,6 +800,7 @@ int main(int argc, char** argv)
 	Graph g;
 	fin >> g;
 	assert(fin.eof());
+	g_contigNames.lock();
 
 	// Read contigs
 	Contigs& contigs = g_contigs;
@@ -999,31 +809,24 @@ int main(int argc, char** argv)
 			cerr << "Reading `" << contigFile << "'..." << endl;
 		FastaReader in(contigFile, FastaReader::NO_FOLD_CASE);
 		for (FastaRecord rec; in >> rec;) {
-			ContigID id(rec.id);
-			assert(contigs.size() == id);
+			assert(contigs.size() == get(g_contigNames, rec.id));
 			contigs.push_back(rec.seq);
 		}
 		assert(in.eof());
 		assert(!contigs.empty());
 		opt::colourSpace = isdigit(contigs[0][0]);
 	}
-	ContigID::lock();
-
-#if 0
-	// Get contig k-mer-coverage statistics.
-	calculateCoverageStatistics(g);
-#endif
 
 	vector<string> pathIDs;
 	vector<bool> isAmbPath;
-	ContigPaths paths = readPath(allPaths, pathIDs, isAmbPath);
+	ContigPaths paths = readPaths(allPaths, pathIDs, isAmbPath);
 	stats.numAmbPaths = g_ambpath_contig.size();
 	if (opt::verbose > 0)
 		cerr << "Read " << paths.size() << " paths\n";
 
 	// Start numbering new contigs from the last
 	if (!pathIDs.empty())
-		ContigID::setNextContigID(pathIDs.back());
+		setNextContigName(pathIDs.back());
 
 	// Prepare output fasta file
 	ofstream fa(opt::consensusPath.c_str());
@@ -1040,12 +843,14 @@ int main(int argc, char** argv)
 	vector<bool> seen(contigs.size());
 
 	// resolve ambiguous paths recorded in g_ambpath_contig
+	g_contigNames.unlock();
 	for (AmbPath2Contig::iterator ambIt = g_ambpath_contig.begin();
 			ambIt != g_ambpath_contig.end(); ambIt++)
 		ambIt->second = fillGap(g, ambIt->first, seen, fa);
+	g_contigNames.lock();
 	assert_good(fa, opt::consensusPath);
 	fa.close();
-	if (opt::verbose > 0)
+	if (opt::verbose > 1)
 		cerr << '\n';
 
 	// Unmark contigs that are used in a path.
@@ -1060,7 +865,7 @@ int main(int argc, char** argv)
 	// Output those contigs that were not seen in ambiguous path.
 	for (unsigned id = 0; id < contigs.size(); ++id)
 		if (seen[id])
-			out << ContigID(id) << '\n';
+			out << get(g_contigNames, id) << '\n';
 
 	for (ContigPaths::const_iterator path = paths.begin();
 			path != paths.end(); ++path) {
@@ -1117,10 +922,10 @@ int main(int argc, char** argv)
 		// Add the newly-created consensus contigs to the graph.
 		for (NewVertices::const_iterator it = g_newVertices.begin();
 				it != g_newVertices.end(); ++it) {
-			Graph::vertex_descriptor u = add_vertex(it->vp, g);
+			Graph::vertex_descriptor u = add_vertex(it->vpu, g);
 			assert(u == it->u);
-			add_edge(it->t, it->u, g);
-			add_edge(it->u, it->v, g);
+			add_edge(it->t, it->u, it->eptu, g);
+			add_edge(it->u, it->v, it->epuv, g);
 		}
 		write_graph(fout, g, PROGRAM, commandLine);
 		assert_good(fout, opt::graphPath);
