@@ -14,6 +14,10 @@
 #include "Graph/ContigGraphAlgorithms.h"
 #include "Graph/DirectedGraph.h"
 #include "Graph/GraphIO.h"
+#include "Graph/GraphUtil.h"
+#include <boost/lambda/bind.hpp>
+#include <boost/lambda/lambda.hpp>
+#include <boost/ref.hpp>
 #include <algorithm>
 #include <cassert>
 #include <cctype>
@@ -27,6 +31,11 @@
 #include <vector>
 
 using namespace std;
+using namespace boost::lambda;
+#if !__GXX_EXPERIMENTAL_CXX0X__
+using boost::cref;
+using boost::ref;
+#endif
 
 #define PROGRAM "Overlap"
 
@@ -34,20 +43,20 @@ static const char VERSION_MESSAGE[] =
 PROGRAM " (" PACKAGE_NAME ") " VERSION "\n"
 "Written by Shaun Jackman.\n"
 "\n"
-"Copyright 2011 Canada's Michael Smith Genome Science Centre\n";
+"Copyright 2012 Canada's Michael Smith Genome Science Centre\n";
 
 static const char USAGE_MESSAGE[] =
 "Usage: " PROGRAM " [OPTION]... CONTIGS ADJ DIST\n"
-"Find overlaps between blunt contigs that have negative distance estimates.\n"
-"Output the small contigs that fill in the gaps.\n"
+"Find overlaps between blunt contigs that have negative distance\n"
+"estimates. Add edges to the overlap graph.\n"
 "\n"
 "  -k, --kmer=KMER_SIZE  k-mer size\n"
 "  -m, --min=OVERLAP     require a minimum of OVERLAP bases\n"
 "                        default is 5 bases\n"
 "      --scaffold        join contigs with Ns [default]\n"
 "      --no-scaffold     do not scaffold\n"
-"      --mask-repeat     join contigs at a simple repeat and mask the repeat\n"
-"                        [default]\n"
+"      --mask-repeat     join contigs at a simple repeat and mask\n"
+"                        the repeat sequence [default]\n"
 "      --no-merge-repeat don't join contigs at a repeat\n"
 "  -g, --graph=FILE      write the contig adjacency graph to FILE\n"
 "  -o, --out=FILE        write result to FILE\n"
@@ -63,6 +72,9 @@ namespace opt {
 	static int mask = 1;
 	static int scaffold = 1;
 
+	/** The acceptable error of a distance estimate. */
+	unsigned distanceError = 6;
+
 	/** Write the contig adjacency graph to this file. */
 	static string graphPath;
 
@@ -114,7 +126,8 @@ static string sequence(const ContigNode& id)
 	return id.sense() ? reverseComplement(seq) : seq;
 }
 
-static unsigned findOverlap(const ContigNode& t_id,
+static unsigned findOverlap(const Graph& g,
+		const ContigNode& t_id,
 		const ContigNode& h_id,
 		bool& mask)
 {
@@ -131,7 +144,8 @@ static unsigned findOverlap(const ContigNode& t_id,
 	}
 
 	if (opt::verbose > 0) {
-		cout << t_id << '\t' << h_id;
+		cout << get(vertex_name, g, t_id)
+			<< '\t' << get(vertex_name, g, h_id);
 		for (vector<unsigned>::const_iterator i = overlaps.begin();
 				i != overlaps.end(); ++i)
 			cout << '\t' << *i;
@@ -161,24 +175,26 @@ static unsigned findOverlap(const ContigNode& t_id,
 	return overlaps[0];
 }
 
-static FastaRecord newContig(const ContigNode& t, const ContigNode& h,
+static FastaRecord newContig(const Graph& g,
+		const ContigNode& t, const ContigNode& v,
 		int dist, const string& seq)
 {
 	ostringstream comment;
-	comment << seq.length() << " 0 " << t << ' ' << h << ' ' << dist;
-	return FastaRecord((string)ContigID::create().str(),
-			comment.str(), seq);
+	comment << seq.length() << " 0 "
+		<< get(vertex_name, g, t) << ' '
+		<< get(vertex_name, g, v) << ' ' << dist;
+	return FastaRecord(createContigName(), comment.str(), seq);
 }
 
 /** An overlap of two sequences. */
-struct Overlap {
-	Estimate est;
+struct Overlap : public DistanceEst {
 	unsigned overlap;
 	bool mask;
 
-	Overlap() : est(), overlap(UINT_MAX), mask(false) { }
-	Overlap(const Estimate& est, unsigned overlap, bool mask)
-		: est(est), overlap(overlap), mask(mask) { }
+	Overlap() : overlap(UINT_MAX), mask(false) { }
+	Overlap(int) { assert(false); }
+	Overlap(const DistanceEst& est, unsigned overlap, bool mask)
+		: DistanceEst(est), overlap(overlap), mask(mask) { }
 
 	bool operator==(const Overlap& o) const
 	{
@@ -194,56 +210,120 @@ struct Overlap {
 	friend ostream& operator<<(ostream& out, const Overlap& o)
 	{
 		return out << "d="
-			<< (o.overlap > 0 ? -(int)o.overlap : o.est.distance);
+			<< (o.overlap > 0 ? -(int)o.overlap : o.distance);
 	}
 };
 
 /** Create a contig representing the gap between contigs u and v. */
-static FastaRecord createGapContig(
+static FastaRecord createGapContig(const Graph& g,
 		const ContigNode& u, const ContigNode& v,
 		const Overlap& o)
 {
 	assert(opt::scaffold);
 	assert(o.overlap == 0);
 	stats.scaffold++;
-	int distance = o.est.distance;
+	int distance = o.distance;
 	if (opt::verbose > 0)
-		cout << u << '\t' << v << "\t(" << distance << ")\n";
+		cout << get(vertex_name, g, u)
+			<< '\t' << get(vertex_name, g, v)
+			<< "\t(" << distance << ")\n";
 	assert(distance < 100000);
 	string gap = distance <= 0 ? string("n")
 		: string(distance, 'N');
 	const string& useq = sequence(u);
 	const string& vseq = sequence(v);
 	unsigned overlap = opt::k - 1; // by convention
-	return newContig(u, v, distance,
+	return newContig(g, u, v, distance,
 			useq.substr(useq.length() - overlap) + gap
 			+ vseq.substr(0, overlap));
 }
 
 /** The scaffold graph. Edges join two blunt contigs that are joined
  * by a distance estimate. */
-typedef ContigGraph<DirectedGraph<
-	no_property, Overlap> > OverlapGraph;
+typedef ContigGraph<DirectedGraph<NoProperty, Overlap> > OverlapGraph;
+
+/**
+ * Check for an overlap between the specified pair of contigs.
+ * Add the size of the overlap to the edge properties. Add the
+ * complementary edge if it does not exist in the graph.
+ * @param goverlap the contig overlap graph
+ * @param g the scaffold graph
+ * @return true if the contigs overlap
+ */
+static bool checkEdgeForOverlap(const Graph& goverlap,
+		OverlapGraph& g,
+		graph_traits<OverlapGraph>::edge_descriptor e)
+{
+	typedef graph_traits<Graph>::vertex_descriptor V;
+	typedef graph_traits<Graph>::edge_descriptor E;
+	typedef edge_bundle_type<OverlapGraph>::type EP;
+
+	V u = source(e, g), v = target(e, g);
+	V uc = get(vertex_complement, g, u);
+	V vc = get(vertex_complement, g, v);
+	assert(u != v);
+	assert(u != vc);
+	EP& ep = g[e];
+	if (ep.overlap != UINT_MAX) {
+		// Found the complementary overlap.
+		return ep.overlap > 0 || opt::scaffold;
+	}
+	if (ep.distance >= 0 && !opt::scaffold) {
+		// Positive distance estimate and not scaffolding.
+		return false;
+	}
+	if (out_degree(u, goverlap) > 0 || in_degree(v, goverlap) > 0) {
+		// Not blunt.
+		return false;
+	}
+
+	bool mask = false;
+	unsigned overlap
+		= ep.distance - (int)allowedError(ep.stdDev) <= 0
+		? findOverlap(goverlap, u, v, mask) : 0;
+	if (mask && !opt::mask) {
+		// Ambiguous overlap.
+		return false;
+	}
+	if (overlap == 0 && !opt::scaffold) {
+		// No overlap and not scaffolding.
+		return false;
+	}
+	ep.overlap = overlap;
+	ep.mask = mask;
+	pair<E, bool> ecomplement = edge(vc, uc, g);
+	if (ecomplement.second) {
+		// Modify the complementary edge.
+		g[ecomplement.first] = ep;
+	} else {
+		// Add the complementary edge.
+		assert(vc != u);
+		add_edge(vc, uc, ep,
+				static_cast<OverlapGraph::base_type&>(g));
+	}
+	return true;
+}
 
 static void findOverlap(const Graph& g,
-		ContigID refID, bool rc, const Estimate& est,
+		ContigID refID, bool rc,
+		const ContigNode& pair,
+		const DistanceEst& est,
 		OverlapGraph& out)
 {
-	if (refID == est.contig.id()
+	if (refID == pair.id()
 			|| (est.distance >= 0 && !opt::scaffold))
 		return;
 	ContigNode ref(refID, false);
-	const ContigNode& pair = est.contig;
 	const ContigNode& t = rc ? pair : ref;
 	const ContigNode& h = rc ? ref : pair;
-	if (g.out_degree(t) > 0 || g.in_degree(h) > 0
+	if (out_degree(t, g) > 0 || in_degree(h, g) > 0
 			|| edge(t, h, out).second)
 		return;
 
 	bool mask = false;
 	unsigned overlap
 		= est.distance - (int)allowedError(est.stdDev) <= 0
-		? findOverlap(t, h, mask) : 0;
+		? findOverlap(g, t, h, mask) : 0;
 	if (mask && !opt::mask)
 		return;
 	if (overlap > 0 || opt::scaffold)
@@ -289,6 +369,11 @@ int main(int argc, char** argv)
 				cout << VERSION_MESSAGE;
 				exit(EXIT_SUCCESS);
 		}
+		if (optarg != NULL && !arg.eof()) {
+			cerr << PROGRAM ": invalid option: `-"
+				<< (char)c << optarg << "'\n";
+			exit(EXIT_FAILURE);
+		}
 	}
 
 	if (opt::k <= 0) {
@@ -329,26 +414,45 @@ int main(int argc, char** argv)
 	Graph graph;
 	fin >> graph;
 	assert(fin.eof());
+	g_contigNames.lock();
 
+	// Open the output file.
 	ofstream out(opt::out.c_str());
-	assert(out.is_open());
+	assert_good(out, opt::out);
+
+	// Read the scaffold graph.
 	ifstream in(estPath.c_str());
 	assert_good(in, estPath);
 
 	// Find overlapping contigs.
 	OverlapGraph scaffoldGraph(graph.num_vertices() / 2);
-	for (EstimateRecord er; in >> er;) {
-		for (int rc = false; rc <= true; ++rc) {
-			const vector<Estimate>& ests = er.estimates[rc];
-			for (EstimateVector::const_iterator iter = ests.begin();
-					iter != ests.end(); ++iter)
-				findOverlap(graph, er.refID, rc, *iter,
-						scaffoldGraph);
+	if (in.peek() == 'd') {
+		// dot graph format
+		in >> scaffoldGraph;
+		assert(in.eof());
+		if (opt::verbose > 0)
+			printGraphStats(cout, scaffoldGraph);
+		remove_edge_if(
+				!bind(checkEdgeForOverlap,
+					cref(graph), ref(scaffoldGraph), _1),
+				static_cast<OverlapGraph::base_type&>(scaffoldGraph));
+	} else {
+		// dist graph format
+		for (EstimateRecord er; in >> er;) {
+			for (int sense = false; sense <= true; ++sense) {
+				typedef vector<
+					pair<ContigNode, DistanceEst> > Estimates;
+				const Estimates& ests = er.estimates[sense];
+				for (Estimates::const_iterator it = ests.begin();
+						it != ests.end(); ++it)
+					findOverlap(graph, er.refID, sense,
+							it->first, it->second,
+							scaffoldGraph);
+			}
 		}
+		assert(in.eof());
 	}
-	assert(in.eof());
 	in.close();
-	ContigID::unlock();
 
 	if (opt::verbose > 1)
 		cout << dot_writer(scaffoldGraph);
@@ -363,7 +467,7 @@ int main(int argc, char** argv)
 		out_edge_iterator;
 
 	/** The overlapping edges (d<0) of scaffoldGraph. */
-	OverlapGraph overlapGraph(graph.num_vertices() / 2);
+	OverlapGraph overlapGraph(num_vertices(graph) / 2);
 
 	/** The canonical edges of scaffoldGraph. */
 	unsigned numOverlaps = num_edges(scaffoldGraph) / 2;
@@ -404,7 +508,7 @@ int main(int argc, char** argv)
 		if (contiguous_out(overlapGraph, t)) {
 			stats.overlap++;
 			assert(*adjacent_vertices(t, overlapGraph).first == h);
-			graph.add_edge(t, h, overlap);
+			add_edge(t, h, overlap, graph);
 
 			// Clear the out-edges of t and the in-edges of h.
 			clear_out_edges(t, scaffoldGraph);
@@ -415,6 +519,7 @@ int main(int argc, char** argv)
 	overlapGraph.clear();
 
 	// Second, handle scaffolded edges.
+	g_contigNames.unlock();
 	for (Edges::const_iterator it = edges.begin();
 			it != edges.end(); ++it) {
 		const ContigNode& t = source(*it, scaffoldGraph),
@@ -430,18 +535,21 @@ int main(int argc, char** argv)
 			// This edge is not scaffolded.
 		} else if (contiguous_out(scaffoldGraph, t)) {
 			assert(*adjacent_vertices(t, scaffoldGraph).first == h);
-			FastaRecord contig = createGapContig(t, h, overlap);
+			FastaRecord contig = createGapContig(graph,
+					t, h, overlap);
 			out << contig;
 			assert(out.good());
 
 			// Add the new contig to the adjacency graph.
-			Graph::vertex_descriptor v = graph.add_vertex(
-					ContigProperties(contig.seq.length(), 0));
-			graph.add_edge(t, v);
-			graph.add_edge(v, h);
+			vertex_descriptor v = add_vertex(
+					ContigProperties(contig.seq.length(), 0), graph);
+			put(vertex_name, graph, v, contig.id);
+			add_edge(t, v, graph);
+			add_edge(v, h, graph);
 		} else
 			stats.ambiguous++;
 	}
+	g_contigNames.lock();
 	out.close();
 
 	if (!opt::graphPath.empty()) {
