@@ -1,4 +1,4 @@
-#include "Aligner.h"
+#include "Alignment.h"
 #include "Estimate.h"
 #include "Histogram.h"
 #include "IOUtil.h"
@@ -30,18 +30,18 @@ static const char VERSION_MESSAGE[] =
 PROGRAM " (" PACKAGE_NAME ") " VERSION "\n"
 "Written by Jared Simpson and Shaun Jackman.\n"
 "\n"
-"Copyright 2011 Canada's Michael Smith Genome Science Centre\n";
+"Copyright 2012 Canada's Michael Smith Genome Science Centre\n";
 
 static const char USAGE_MESSAGE[] =
 "Usage: " PROGRAM " [OPTION]... [FILE]...\n"
-"Write read pairs that align to the same contig to FRAGMENTS or HISTOGRAM.\n"
-"Write read pairs that align to different contigs to standard output.\n"
-"Alignments may be in FILE(s) or standard input.\n"
+"Write pairs that map to the same contig to the file SAME.\n"
+"Write pairs that map to different contigs to standard output.\n"
+"Alignments may be read from FILE(s) or standard input.\n"
 "\n"
-"  -k, --kmer=KMER_SIZE  k-mer size\n"
+"  -l, --min-align=N     minimum alignment length\n"
 "  -d, --dist=DISTANCE   write distance estimates to this file\n"
-"  -f, --frag=FRAGMENTS  write fragment sizes to this file\n"
-"  -h, --hist=HISTOGRAM  write the fragment size histogram to this file\n"
+"  -f, --frag=SAME       write fragment sizes to this file\n"
+"  -h, --hist=FILE       write the fragment size histogram to FILE\n"
 "      --sam             alignments are in SAM format\n"
 "      --kaligner        alignments are in KAligner format\n"
 "  -c, --cover=COVERAGE  coverage cut-off for distance estimates\n"
@@ -67,13 +67,13 @@ namespace opt {
  	int format = ADJ; // used by Estimate
 }
 
-static const char shortopts[] = "d:k:f:h:c:v";
+static const char shortopts[] = "d:l:f:h:c:v";
 
 enum { OPT_HELP = 1, OPT_VERSION };
 
 static const struct option longopts[] = {
 	{ "dist",    required_argument, NULL, 'd' },
-	{ "kmer",    required_argument, NULL, 'k' },
+	{ "min-align", required_argument, NULL, 'l' },
 	{ "frag",    required_argument, NULL, 'f' },
 	{ "hist",    required_argument, NULL, 'h' },
 	{ "kaligner",no_argument, &opt::inputFormat, opt::KALIGNER },
@@ -98,6 +98,8 @@ static struct {
 static ofstream fragFile;
 static Histogram histogram;
 
+typedef vector<Alignment> AlignmentVector;
+
 /** A map of read IDs to alignments. */
 typedef unordered_map<string, AlignmentVector> ReadAlignMap;
 
@@ -121,20 +123,23 @@ static int fragmentSize(const Alignment& a0, const Alignment& a1)
 	return r - f;
 }
 
-static void addEstimate(EstimateMap& map, const Alignment& a, Estimate& est, bool reverse)
+typedef pair<ContigNode, DistanceEst> Estimate;
+typedef vector<Estimate> Estimates;
+
+static void addEstimate(EstimateMap& map, const Alignment& a,
+		Estimate& est, bool reverse)
 {
 	//count up the number of estimates that agree
 	bool placed = false;
 	bool a_isRC = a.isRC != reverse;
 	EstimateMap::iterator estimatesIt = map.find(a.contig);
 	if (estimatesIt != map.end()) {
-		EstimateVector& estimates =
-			estimatesIt->second.estimates[a_isRC];
-		for (EstimateVector::iterator estIt = estimates.begin();
+		Estimates& estimates = estimatesIt->second.estimates[a_isRC];
+		for (Estimates::iterator estIt = estimates.begin();
 				estIt != estimates.end(); ++estIt) {
-			if (estIt->contig.id() == est.contig.id()) {
-				estIt->numPairs++;
-				estIt->distance += est.distance;
+			if (estIt->first.id() == est.first.id()) {
+				estIt->second.numPairs++;
+				estIt->second.distance += est.second.distance;
 				placed = true;
 				break;
 			}
@@ -180,11 +185,12 @@ static void doReadIntegrity(const ReadAlignMap::value_type& a)
 		unsigned largest_end =
 			largest.read_start_pos + largest.align_length - opt::k;
 		int distance = last.read_start_pos - largest_end;
-		est.contig = ContigNode(last.contig,
-				largest.isRC != last.isRC);
-		est.distance = distance - opt::k;
-		est.numPairs = 1;
-		est.stdDev = 0;
+		est.first = find_vertex(
+				last.contig, largest.isRC != last.isRC,
+				g_contigNames);
+		est.second.distance = distance - opt::k;
+		est.second.numPairs = 1;
+		est.second.stdDev = 0;
 		addEstimate(estMap, largest, est, false);
 	}
 
@@ -194,10 +200,12 @@ static void doReadIntegrity(const ReadAlignMap::value_type& a)
 		unsigned first_end =
 			first.read_start_pos + first.align_length - opt::k;
 		int distance = last.read_start_pos - first_end;
-		est.contig = ContigNode(last.contig, first.isRC != last.isRC);
-		est.distance = distance - opt::k;
-		est.numPairs = 1;
-		est.stdDev = 0;
+		est.first = find_vertex(
+				last.contig, first.isRC != last.isRC,
+				g_contigNames);
+		est.second.distance = distance - opt::k;
+		est.second.numPairs = 1;
+		est.second.stdDev = 0;
 		addEstimate(estMap, first, est, false);
 	}
 
@@ -208,11 +216,12 @@ static void doReadIntegrity(const ReadAlignMap::value_type& a)
 		unsigned largest_end =
 			largest.read_start_pos + largest.align_length - opt::k;
 		int distance = first.read_start_pos - largest_end;
-		est.contig = ContigNode(first.contig,
-				largest.isRC != first.isRC);
-		est.distance = distance - opt::k;
-		est.numPairs = 1;
-		est.stdDev = 0;
+		est.first = find_vertex(
+				first.contig, largest.isRC != first.isRC,
+				g_contigNames);
+		est.second.distance = distance - opt::k;
+		est.second.numPairs = 1;
+		est.second.stdDev = 0;
 		addEstimate(estMap, largest, est, false);
 	}
 
@@ -261,13 +270,18 @@ static void generateDistFile()
 			if (refIsRC)
 				distFile << " ;";
 
-			for (EstimateVector::iterator vecIt = mapIt->second.estimates[refIsRC].begin();
+			for (Estimates::iterator vecIt
+					= mapIt->second.estimates[refIsRC].begin();
 					vecIt != mapIt->second.estimates[refIsRC].end(); ++vecIt) {
-				vecIt->distance = (int)round((double)vecIt->distance /
-						(double)vecIt->numPairs);
-				if (vecIt->numPairs >= opt::c && vecIt->numPairs != 0
+				vecIt->second.distance
+					= (int)round((double)vecIt->second.distance /
+							(double)vecIt->second.numPairs);
+				if (vecIt->second.numPairs >= opt::c
+						&& vecIt->second.numPairs != 0
 						/*&& vecIt->distance > 1 - opt::k*/)
-					distFile << ' ' << *vecIt;
+					distFile
+						<< ' ' << get(g_contigNames, vecIt->first)
+						<< ',' << vecIt->second;
 			}
 		}
 		distFile << '\n';
@@ -477,7 +491,7 @@ int main(int argc, char* const* argv)
 		istringstream arg(optarg != NULL ? optarg : "");
 		switch (c) {
 			case '?': die = true; break;
-			case 'k': arg >> opt::k; break;
+			case 'l': arg >> opt::k; break;
 			case 'c': arg >> opt::c; break;
 			case 'd': arg >> opt::distPath; break;
 			case 'f': arg >> opt::fragPath; break;
@@ -490,6 +504,11 @@ int main(int argc, char* const* argv)
 				cout << VERSION_MESSAGE;
 				exit(EXIT_SUCCESS);
 		}
+		if (optarg != NULL && !arg.eof()) {
+			cerr << PROGRAM ": invalid option: `-"
+				<< (char)c << optarg << "'\n";
+			exit(EXIT_FAILURE);
+		}
 	}
 
 	if (opt::k <= 0 && opt::inputFormat == opt::KALIGNER) {
@@ -555,6 +574,8 @@ int main(int argc, char* const* argv)
 	if (numFR < numRF)
 		histogram = histogram.negate();
 	histogram.eraseNegative();
+	histogram.removeNoise();
+	histogram.removeOutliers();
 	Histogram h = histogram.trimFraction(0.0001);
 	if (opt::verbose > 0)
 		cerr << "Stats mean: " << setprecision(4) << h.mean() << " "
