@@ -1,10 +1,11 @@
 #include "Histogram.h"
+#include "IOUtil.h"
+#include "MemoryUtil.h"
 #include "SAM.h"
 #include "StringUtil.h"
 #include "Uncompress.h"
 #include "UnorderedMap.h"
 #include <algorithm>
-#include <cerrno>
 #include <climits>
 #include <cstdlib>
 #include <fstream>
@@ -24,18 +25,18 @@ static const char VERSION_MESSAGE[] =
 PROGRAM " (" PACKAGE_NAME ") " VERSION "\n"
 "Written by Shaun Jackman.\n"
 "\n"
-"Copyright 2011 Canada's Michael Smith Genome Science Centre\n";
+"Copyright 2012 Canada's Michael Smith Genome Science Centre\n";
 
 static const char USAGE_MESSAGE[] =
 "Usage: " PROGRAM " [OPTION]... [FILE]...\n"
-"Write read pairs that align to the same contig to SAME.\n"
-"Write read pairs that align to different contigs to standard output.\n"
+"Write read pairs that map to the same contig to the file SAME.\n"
+"Write read pairs that map to different contigs to stdout.\n"
 "Alignments may be in FILE(s) or standard input.\n"
 "\n"
 "      --no-qname        set the qname to * [default]\n"
 "      --qname           do not alter the qname\n"
 "  -s, --same=SAME       write properly-paired reads to this file\n"
-"  -h, --hist=HISTOGRAM  write the fragment size histogram to this file\n"
+"  -h, --hist=FILE       write the fragment size histogram to FILE\n"
 "  -v, --verbose         display verbose output\n"
 "      --help            display this help and exit\n"
 "      --version         output version information and exit\n"
@@ -79,7 +80,10 @@ static void handlePair(SAMRecord& a0, SAMRecord& a1)
 {
 	if ((a0.isRead1() && a1.isRead1())
 			|| (a0.isRead2() && a1.isRead2())) {
-		cerr << "error: duplicate read ID `" << a0.qname << "'\n";
+		cerr << "error: duplicate read ID `" << a0.qname
+			<< (a0.isRead1() ? "/1" : "")
+			<< (a0.isRead2() ? "/2" : "")
+			<< "'\n";
 		exit(EXIT_FAILURE);
 	}
 
@@ -122,9 +126,6 @@ typedef unordered_map<string, SAMRecord> Alignments;
 typedef unordered_map<string, SAMAlignment> Alignments;
 #endif
 
-/** Start of the data segment. */
-static intptr_t sbrk0 = reinterpret_cast<intptr_t>(sbrk(0));
-
 static void printProgress(const Alignments& map)
 {
 	if (opt::verbose == 0)
@@ -136,13 +137,12 @@ static void printProgress(const Alignments& map)
 
 	size_t buckets = map.bucket_count();
 	if (stats.alignments % 1000000 == 0 || buckets != prevBuckets) {
-		ptrdiff_t bytes = reinterpret_cast<intptr_t>(sbrk(0)) - sbrk0;
 		prevBuckets = buckets;
 		size_t size = map.size();
 		cerr << "Read " << stats.alignments << " alignments. "
 			<< "Hash load: " << size << " / " << buckets
 			<< " = " << (float)size / buckets
-			<< " using " << toSI(bytes) << "B." << endl;
+			<< " using " << toSI(getMemoryUsage()) << "B." << endl;
 	}
 }
 
@@ -188,20 +188,12 @@ static void readAlignments(istream& in, Alignments* pMap)
 	assert_eof(in);
 }
 
-static void assert_open(ifstream& f, const string& p)
-{
-	if (!f.is_open()) {
-		cerr << p << ": " << strerror(errno) << endl;
-		exit(EXIT_FAILURE);
-	}
-}
-
 static void readAlignmentsFile(string path, Alignments* pMap)
 {
 	if (opt::verbose > 0)
 		cerr << "Reading `" << path << "'..." << endl;
 	ifstream fin(path.c_str());
-	assert_open(fin, path);
+	assert_good(fin, path);
 	readAlignments(fin, pMap);
 	fin.close();
 }
@@ -216,6 +208,18 @@ static string percent(size_t x, size_t n)
 	return ss.str();
 }
 
+/** Print statistics of the specified histogram. */
+static void printHistogramStats(const Histogram& h)
+{
+	cerr << "Stats mean: " << setprecision(4) << h.mean() << " "
+		"median: " << setprecision(4) << h.median() << " "
+		"sd: " << setprecision(4) << h.sd() << " "
+		"n: " << h.size() << " "
+		"min: " << h.minimum() << " "
+		"max: " << h.maximum() << '\n'
+		<< h.barplot() << endl;
+}
+
 int main(int argc, char* const* argv)
 {
 	bool die = false;
@@ -234,6 +238,11 @@ int main(int argc, char* const* argv)
 				cout << VERSION_MESSAGE;
 				exit(EXIT_SUCCESS);
 		}
+		if (optarg != NULL && !arg.eof()) {
+			cerr << PROGRAM ": invalid option: `-"
+				<< (char)c << optarg << "'\n";
+			exit(EXIT_FAILURE);
+		}
 	}
 
 	if (die) {
@@ -286,17 +295,29 @@ int main(int argc, char* const* argv)
 		histFile.close();
 	}
 
-	if (numFR < numRF)
-		g_histogram = g_histogram.negate();
-	g_histogram.eraseNegative();
-	Histogram h = g_histogram.trimFraction(0.0001);
-	if (opt::verbose > 0)
-		cerr << "Stats mean: " << setprecision(4) << h.mean() << " "
-			"median: " << setprecision(4) << h.median() << " "
-			"sd: " << setprecision(4) << h.sd() << " "
-			"n: " << h.size() << " "
-			"min: " << h.minimum() << " max: " << h.maximum() << '\n'
-			<< h.barplot() << endl;
+	if (opt::verbose > 0) {
+		size_t numTotal = numFR + numRF;
+
+		// Print the statistics of the forward-reverse distribution.
+		if ((float)numFR / numTotal > 0.001) {
+			Histogram h = g_histogram;
+			h.eraseNegative();
+			h.removeNoise();
+			h.removeOutliers();
+			cerr << "FR ";
+			printHistogramStats(h.trimFraction(0.0001));
+		}
+
+		// Print the statistics of the reverse-forward distribution.
+		if ((float)numRF / numTotal > 0.001) {
+			Histogram h = g_histogram.negate();
+			h.eraseNegative();
+			h.removeNoise();
+			h.removeOutliers();
+			cerr << "RF ";
+			printHistogramStats(h.trimFraction(0.0001));
+		}
+	}
 
 	if (stats.numFF > numFR && stats.numFF > numRF) {
 		cerr << "error: The mate pairs of this library are oriented "
