@@ -32,7 +32,7 @@ static const char *VERSION_MESSAGE =
 PROGRAM " (ABySS) " VERSION "\n"
 "Written by Shaun Jackman and Tony Raymond.\n"
 "\n"
-"Copyright 2011 Canada's Michael Smith Genome Science Centre\n";
+"Copyright 2012 Canada's Michael Smith Genome Science Centre\n";
 
 static const char *USAGE_MESSAGE =
 "Usage: " PROGRAM " [OPTION]... ADJ PATH\n"
@@ -119,7 +119,7 @@ struct Vertex {
 		return id == v.id && sense == v.sense;
 	}
 
-	operator ContigNode() const
+	ContigNode descriptor() const
 	{
 		return ContigNode(s_offset + id, sense);
 	}
@@ -178,8 +178,12 @@ static ContigPath getPath(const Paths& paths, const ContigNode& u)
 static Paths readPaths(Graph& g,
 		const string& inPath, vector<string>& pathIDs)
 {
+	typedef graph_traits<Graph>::vertex_descriptor V;
+
 	assert(pathIDs.empty());
 	ifstream fin(inPath.c_str());
+	if (opt::verbose > 0)
+		cerr << "Reading `" << inPath << "'..." << endl;
 	if (inPath != "-")
 		assert_good(fin, inPath);
 	istream& in = inPath == "-" ? cin : fin;
@@ -191,7 +195,7 @@ static Paths readPaths(Graph& g,
 	while (in >> id >> path) {
 		if (path.empty()) {
 			// Remove this contig from the graph.
-			ContigNode u(id, false);
+			V u = find_vertex(id, false, g);
 			clear_vertex(u, g);
 			remove_vertex(u, g);
 		} else {
@@ -218,7 +222,7 @@ static SeedMap makeSeedMap(const Paths& paths)
 		seedMap.insert(make_pair(it->front(),
 					Vertex(it - paths.begin(), false)));
 		assert(!it->back().ambiguous());
-		seedMap.insert(make_pair(~it->back(),
+		seedMap.insert(make_pair(it->back() ^ 1,
 					Vertex(it - paths.begin(), true)));
 	}
 	return seedMap;
@@ -308,7 +312,7 @@ static void recordTrimmedContigs(
 {
 	for (ContigPath::const_iterator it = first; it != last; ++it)
 		if (!it->ambiguous())
-			s_trimmedContigs.push_back(ContigID(*it));
+			s_trimmedContigs.push_back(it->contigIndex());
 }
 
 /** Remove ambiguous contigs from the ends of the path. */
@@ -382,18 +386,21 @@ static void addPathOverlapEdges(Graph& g,
 		const Paths& paths, const vector<string>& pathIDs,
 		const Overlaps& overlaps)
 {
+	typedef graph_traits<Graph>::vertex_descriptor V;
 	const bool allowParallelEdge = opt::mode == opt::ASSEMBLE;
 
 	// Add the path vertices.
-	ContigID::unlock();
+	g_contigNames.unlock();
 	for (Paths::const_iterator it = paths.begin();
 			it != paths.end(); ++it) {
-		if (it->empty())
-			continue;
-		ContigID::insert(pathIDs[it - paths.begin()]);
-		merge(g, it->begin(), it->end());
+		const ContigPath& path = *it;
+		const string& id = pathIDs[it - paths.begin()];
+		if (!path.empty()) {
+			V u = merge(g, path.begin(), path.end());
+			put(vertex_name, g, u, id);
+		}
 	}
-	ContigID::lock();
+	g_contigNames.lock();
 
 	// Remove the single-end contigs that are in paths.
 	for (Paths::const_iterator it = paths.begin();
@@ -404,12 +411,13 @@ static void addPathOverlapEdges(Graph& g,
 	// Add the path edges.
 	for (Overlaps::const_iterator it = overlaps.begin();
 			it != overlaps.end(); ++it) {
-		Vertex u = it->source, v = it->target;
+		V u = it->source.descriptor();
+		V v = it->target.descriptor();
 		if (allowParallelEdge || !edge(u, v, g).second)
 			add_edge(u, v, it->distance, static_cast<DG&>(g));
 		else if (opt::verbose > 0)
-			cerr << "ambiguous overlap: "
-				<< ContigNode(u) << " -> " << ContigNode(v) << '\n';
+			cerr << "ambiguous overlap: " << get(vertex_name, g, u)
+				<< " -> " << get(vertex_name, g, v) << '\n';
 	}
 }
 
@@ -421,7 +429,8 @@ typedef map<edge_descriptor, unsigned> OverlapMap;
 
 /** Return the number of contigs by which the two paths overlap. */
 static unsigned getOverlap(const OverlapMap& pmap,
-		const ContigNode& u, const ContigNode& v)
+		graph_traits<Graph>::vertex_descriptor u,
+		graph_traits<Graph>::vertex_descriptor v)
 {
 	if (isPath(u) && isPath(v)) {
 		// Both vertices are paths.
@@ -485,7 +494,9 @@ static void assembleOverlappingPaths(Graph& g,
 	for (Overlaps::const_iterator it = overlaps.begin();
 			it != overlaps.end(); ++it)
 		overlapMap.insert(OverlapMap::value_type(
-				OverlapMap::key_type(it->source, it->target),
+				OverlapMap::key_type(
+					it->source.descriptor(),
+					it->target.descriptor()),
 				it->overlap));
 
 	// Assemble unambiguously overlapping paths.
@@ -494,14 +505,17 @@ static void assembleOverlappingPaths(Graph& g,
 			IsPathOverlap(g, overlapMap));
 
 	// Merge overlapping paths.
+	g_contigNames.unlock();
 	assert(!pathIDs.empty());
-	ContigID::setNextContigID(pathIDs.back());
+	setNextContigName(pathIDs.back());
 	for (Paths::const_iterator it = merges.begin();
 			it != merges.end(); ++it) {
-		ContigID id = ContigID::create();
+		string name = createContigName();
 		if (opt::verbose > 0)
-			cerr << id << '\t' << *it << '\n';
-		pathIDs.push_back((string)id.str());
+			cerr << name << '\t' << *it << '\n';
+		Vertex u(paths.size(), false);
+		put(vertex_name, g, u.descriptor(), name);
+		pathIDs.push_back(name);
 		paths.push_back(mergePaths(paths, overlapMap, *it));
 
 		// Remove the merged paths.
@@ -511,6 +525,7 @@ static void assembleOverlappingPaths(Graph& g,
 				paths[it2->id() - Vertex::s_offset].clear();
 		}
 	}
+	g_contigNames.lock();
 }
 
 int main(int argc, char** argv)
@@ -541,6 +556,11 @@ int main(int argc, char** argv)
 				cout << VERSION_MESSAGE;
 				exit(EXIT_SUCCESS);
 		}
+		if (optarg != NULL && !arg.eof()) {
+			cerr << PROGRAM ": invalid option: `-"
+				<< (char)c << optarg << "'\n";
+			exit(EXIT_FAILURE);
+		}
 	}
 
 	if (opt::k <= 0) {
@@ -563,6 +583,8 @@ int main(int argc, char** argv)
 	}
 
 	const char *adjPath = argv[optind++];
+	if (opt::verbose > 0)
+		cerr << "Reading `" << adjPath << "'..." << endl;
 	ifstream fin(adjPath);
 	assert_good(fin, adjPath);
 	Graph g;
@@ -633,7 +655,7 @@ int main(int argc, char** argv)
 		for (vector<ContigID>::const_iterator it
 				= s_trimmedContigs.begin();
 				it != s_trimmedContigs.end(); ++it)
-			out << ContigID(*it) << '\n';
+			out << get(g_contigNames, *it) << '\n';
 		assert_good(out, opt::repeatContigs);
 	}
 
