@@ -11,7 +11,6 @@
 #include "ContigProperties.h"
 #include "FastaReader.h"
 #include "IOUtil.h"
-#include "Iterator.h"
 #include "Sequence.h"
 #include "Uncompress.h"
 #include "alignGlobal.h"
@@ -54,7 +53,7 @@ static const char VERSION_MESSAGE[] =
 PROGRAM " (" PACKAGE_NAME ") " VERSION "\n"
 "Written by Shaun Jackman.\n"
 "\n"
-"Copyright 2011 Canada's Michael Smith Genome Science Centre\n";
+"Copyright 2012 Canada's Michael Smith Genome Science Centre\n";
 
 static const char USAGE_MESSAGE[] =
 "Usage: " PROGRAM " [OPTION]... FASTA ADJ\n"
@@ -180,16 +179,16 @@ static void popBubble(Graph& g,
 	if (opt::dot)
 #pragma omp critical(cout)
 	{
-		cout << '"' << v << "\" -> {";
-		copy(sorted.begin(), sorted.end(),
-				affix_ostream_iterator<ContigNode>(cout,
-					" \"", "\""));
-		cout << " } -> \"" << tail << "\"\n";
+		cout << '"' << get(vertex_name, g, v) << "\" -> {";
+		for (vector<vertex_descriptor>::const_iterator
+				it = sorted.begin(); it != sorted.end(); ++it)
+			cout << " \"" << get(vertex_name, g, *it) << '"';
+		cout << " } -> \"" << get(vertex_name, g, tail) << "\"\n";
 	}
 #pragma omp critical(g_popped)
 	transform(sorted.begin() + 1, sorted.end(),
 			back_inserter(g_popped),
-			mem_fun_ref(&ContigNode::operator ContigID));
+			mem_fun_ref(&ContigNode::contigIndex));
 }
 
 static struct {
@@ -207,12 +206,12 @@ typedef vector<const_string> Contigs;
 static Contigs g_contigs;
 
 /** Return the sequence of vertex u. */
-static string getSequence(vertex_descriptor u)
+static string getSequence(const Graph* g, vertex_descriptor u)
 {
-	assert(!u.ambiguous());
-	assert(u.id() < g_contigs.size());
-	string seq(g_contigs[u.id()]);
-	return u.sense() ? reverseComplement(seq) : seq;
+	size_t i = get(vertex_contig_index, *g, u);
+	assert(i < g_contigs.size());
+	string seq(g_contigs[i]);
+	return get(vertex_sense, *g, u) ? reverseComplement(seq) : seq;
 }
 
 /** Return the length of vertex v. */
@@ -302,7 +301,7 @@ static float getAlignmentIdentity(const Graph& g,
 		return max_identity;
 
 	vector<string> seqs(nbranches);
-	transform(first, last, seqs.begin(), getSequence);
+	transform(first, last, seqs.begin(), bind(getSequence, &g, _1));
 	for (unsigned i = 0; i < seqs.size(); i++) {
 		// Remove the overlapping sequence.
 		int n = seqs[i].size();
@@ -332,7 +331,7 @@ static bool popSimpleBubble(Graph* pg, vertex_descriptor v)
 		return false;
 	}
 	vertex_descriptor tail = *g.adjacent_vertices(v1).first;
-	if (v == ~tail // Palindrome
+	if (v == get(vertex_complement, g, tail) // Palindrome
 			|| g.in_degree(tail) != nbranches) {
 #pragma omp atomic
 		g_count.notSimple++;
@@ -359,10 +358,11 @@ static bool popSimpleBubble(Graph* pg, vertex_descriptor v)
 	if (opt::verbose > 2)
 #pragma omp critical(cerr)
 	{
-		cerr << "\n* " << v << " -> ";
-		copy(adj.first, adj.second,
-				ostream_iterator<ContigNode>(cerr, " "));
-		cerr << "-> " << tail << '\n';
+		cerr << "\n* " << get(vertex_name, g, v) << " ->";
+		for (adjacency_iterator it = adj.first;
+				it != adj.second; ++it)
+			cerr << ' ' << get(vertex_name, g, *it);
+		cerr << " -> " << get(vertex_name, g, tail) << '\n';
 	}
 
 	if (nbranches > opt::maxBranches) {
@@ -476,8 +476,12 @@ static void scaffoldBubble(Graph& g, const Bubble& bubble)
 	}
 	assert(isBubble(g, bubble.begin(), bubble.end()));
 
-	g_popped.insert(g_popped.end(),
-			bubble.begin() + 1, bubble.end() - 1);
+	assert(bubble.size() > 2);
+	size_t n = bubble.size() - 2;
+	g_popped.reserve(g_popped.size() + n);
+	for (Bubble::const_iterator it = bubble.begin() + 1;
+			it != bubble.end() - 1; ++it)
+		g_popped.push_back(it->contigIndex());
 
 	add_edge(u, w, max(longestPath(g, bubble), 1), g);
 }
@@ -526,7 +530,7 @@ static void filterGraph(Graph& g)
 			removedKmer += getKmerLength(vp);
 			clear_vertex(u, g);
 			remove_vertex(u, g);
-			g_popped.push_back(u);
+			g_popped.push_back(get(vertex_contig_index, g, u));
 		}
 	}
 	if (opt::verbose > 0) {
@@ -580,6 +584,11 @@ int main(int argc, char** argv)
 				cout << VERSION_MESSAGE;
 				exit(EXIT_SUCCESS);
 		}
+		if (optarg != NULL && !arg.eof()) {
+			cerr << PROGRAM ": invalid option: `-"
+				<< (char)c << optarg << "'\n";
+			exit(EXIT_FAILURE);
+		}
 	}
 
 	if (opt::k <= 0) {
@@ -621,7 +630,7 @@ int main(int argc, char** argv)
 	Graph g;
 	fin >> g;
 	assert(fin.eof());
-	ContigID::lock();
+	g_contigNames.lock();
 	if (opt::verbose > 0)
 		printGraphStats(cerr, g);
 
@@ -632,10 +641,9 @@ int main(int argc, char** argv)
 			cerr << "Reading `" << contigsPath << "'...\n";
 		FastaReader in(contigsPath, FastaReader::NO_FOLD_CASE);
 		for (FastaRecord rec; in >> rec;) {
-			if (ContigID::count(rec.id) == 0)
+			if (g_contigNames.count(rec.id) == 0)
 				continue;
-			ContigID id(rec.id);
-			assert(contigs.size() == id);
+			assert(contigs.size() == get(g_contigNames, rec.id));
 			contigs.push_back(rec.seq);
 		}
 		assert(in.eof());
@@ -660,22 +668,24 @@ int main(int argc, char** argv)
 	g_popped.erase(unique(g_popped.begin(), g_popped.end()),
 			g_popped.end());
 
-	if (opt::dot)
+	if (opt::dot) {
 		cout << "}\n";
-	else
-		copy(g_popped.begin(), g_popped.end(),
-				ostream_iterator<ContigID>(cout, "\n"));
+	} else {
+		for (vector<ContigID>::const_iterator it = g_popped.begin();
+				it != g_popped.end(); ++it)
+			cout << get(g_contigNames, *it) << '\n';
+	}
 	free_prob_dist(pdist);
 	free(para);
 
 	if (opt::verbose > 0)
-		cerr << "Bubbles: " << g_count.bubbles/2
-			<< " Popped: " << g_count.popped/2
-			<< " Scaffolds: " << g_count.scaffold/2
-			<< " Complex: " << g_count.notSimple/2
-			<< " Too long: " << g_count.tooLong/2
-			<< " Too many: " << g_count.tooMany/2
-			<< " Dissimilar: " << g_count.dissimilar/2
+		cerr << "Bubbles: " << (g_count.bubbles + 1) / 2
+			<< " Popped: " << (g_count.popped + 1) / 2
+			<< " Scaffolds: " << (g_count.scaffold + 1) / 2
+			<< " Complex: " << (g_count.notSimple + 1) / 2
+			<< " Too long: " << (g_count.tooLong + 1) / 2
+			<< " Too many: " << (g_count.tooMany + 1) / 2
+			<< " Dissimilar: " << (g_count.dissimilar + 1) / 2
 			<< '\n';
 
 	if (!opt::graphPath.empty()) {
@@ -684,21 +694,32 @@ int main(int argc, char** argv)
 				bind1st(ptr_fun(removeContig), &g));
 
 		// Assemble unambiguous paths.
+		g_contigNames.unlock();
 		typedef vector<ContigPath> ContigPaths;
 		ContigPaths paths;
+		size_t numContigs = num_vertices(g) / 2;
 		if (opt::scaffold) {
 			Graph gorig = g;
 			assemble(g, back_inserter(paths));
 			for (ContigPaths::const_iterator it = paths.begin();
-					it != paths.end(); ++it)
-				cout << ContigID::create() << '\t'
+					it != paths.end(); ++it) {
+				ContigNode u(numContigs + it - paths.begin(), false);
+				string name = createContigName();
+				put(vertex_name, g, u, name);
+				cout << name << '\t'
 					<< addDistance(gorig, *it) << '\n';
+			}
 		} else {
 			assemble(g, back_inserter(paths));
 			for (ContigPaths::const_iterator it = paths.begin();
-					it != paths.end(); ++it)
-				cout << ContigID::create() << '\t' << *it << '\n';
+					it != paths.end(); ++it) {
+				ContigNode u(numContigs + it - paths.begin(), false);
+				string name = createContigName();
+				put(vertex_name, g, u, name);
+				cout << name << '\t' << *it << '\n';
+			}
 		}
+		g_contigNames.lock();
 		paths.clear();
 
 		// Output the updated adjacency graph.
