@@ -4,6 +4,9 @@
 #include "ContigProperties.h"
 #include "Estimate.h"
 #include "IOUtil.h"
+#include "Iterator.h"
+#include "Uncompress.h"
+#include "Graph/Assemble.h"
 #include "Graph/ContigGraph.h"
 #include "Graph/ContigGraphAlgorithms.h"
 #include "Graph/DirectedGraph.h"
@@ -13,8 +16,10 @@
 #include "Graph/PopBubbles.h"
 #include <cassert>
 #include <climits>
+#include <cmath>
 #include <cstdlib>
 #include <fstream>
+#include <functional>
 #include <getopt.h>
 #include <iostream>
 #include <sstream>
@@ -32,7 +37,7 @@ static const char VERSION_MESSAGE[] =
 PROGRAM " (" PACKAGE_NAME ") " VERSION "\n"
 "Written by Shaun Jackman.\n"
 "\n"
-"Copyright 2011 Canada's Michael Smith Genome Science Centre\n";
+"Copyright 2012 Canada's Michael Smith Genome Science Centre\n";
 
 static const char USAGE_MESSAGE[] =
 "Usage: " PROGRAM " [OPTION]... FASTA|OVERLAP DIST...\n"
@@ -42,7 +47,9 @@ static const char USAGE_MESSAGE[] =
 "  DIST     estimates of the distance between contigs\n"
 "\n"
 "  -n, --npairs=N        minimum number of pairs [0]\n"
-"  -s, --seed-length=N   minimum contig length [0]\n"
+"  -s, --seed-length=N   minimum contig length [200]\n"
+"          or -s N0-N1   Find the value of s in [N0,N1]\n"
+"                        that maximizes the scaffold N50.\n"
 "  -k, --kmer=N          length of a k-mer\n"
 "      --min-gap=N       minimum scaffold gap length to output [50]\n"
 "  -o, --out=FILE        write the paths to FILE\n"
@@ -60,7 +67,8 @@ namespace opt {
 	static unsigned minNumPairs;
 
 	/** Minimum contig length. */
-	static unsigned minContigLength;
+	static unsigned minContigLength = 200;
+	static unsigned minContigLengthEnd;
 
 	/** Minimum scaffold gap length to output. */
 	static int minGap = 50;
@@ -74,8 +82,8 @@ namespace opt {
 	/** Verbose output. */
 	int verbose; // used by PopBubbles
 
- 	/** Output format */
- 	int format = DOT; // used by DistanceEst
+	/** Output format */
+	int format = DOT; // used by DistanceEst
 }
 
 static const char shortopts[] = "g:k:n:o:s:v";
@@ -95,80 +103,10 @@ static const struct option longopts[] = {
 	{ NULL, 0, NULL, 0 }
 };
 
-/** Contig length property. */
-struct Length {
-	unsigned length;
-
-	bool operator==(const Length& o) const
-	{
-		return length == o.length;
-	}
-
-	Length& operator+=(const Length& o)
-	{
-		length += o.length;
-		return *this;
-	}
-
-	template <typename T>
-	Length& operator+=(const T& o)
-	{
-		assert((int)length + (int)o.distance > 0);
-		length += o.distance;
-		return *this;
-	}
-
-	friend ostream& operator<<(ostream& out, const Length& o)
-	{
-		return out << "l=" << o.length;
-	}
-
-	friend istream& operator>>(istream& in, Length& o)
-	{
-		if (in >> std::ws && in.peek() == 'l')
-			return in >> expect("l =") >> o.length;
-		else
-			return in >> o.length;
-	}
-};
-
-static inline
-void put(vertex_length_t, Length& vp, unsigned length)
-{
-	vp.length = length;
-}
-
-static inline
-void put(vertex_coverage_t, Length&, unsigned)
-{
-}
-
 /** A distance estimate graph. */
 typedef DirectedGraph<Length, DistanceEst> DG;
 typedef ContigGraph<DG> Graph;
 
-/** Add missing complementary edges. */
-static void addComplementaryEdges(DG& g)
-{
-	typedef graph_traits<Graph> GTraits;
-	typedef GTraits::edge_descriptor E;
-	typedef GTraits::edge_iterator Eit;
-	typedef GTraits::vertex_descriptor V;
-
-	std::pair<Eit, Eit> erange = edges(g);
-	unsigned numAdded = 0;
-	for (Eit eit = erange.first; eit != erange.second; ++eit) {
-		E e = *eit;
-		V u = source(e, g), v = target(e, g);
-		if (!edge(~v, ~u, g).second) {
-			add_edge(~v, ~u, g[e], g);
-			numAdded++;
-		}
-	}
-	if (opt::verbose > 0)
-		cerr << "Added " << numAdded << " complementary edges.\n";
-}
-
 /** Return whether the specified edges has sufficient support. */
 struct PoorSupport {
 	PoorSupport(Graph& g) : m_g(g) { }
@@ -180,7 +118,7 @@ struct PoorSupport {
 };
 
 /** Remove short vertices and unsupported edges from the graph. */
-static void filterGraph(Graph& g)
+static void filterGraph(Graph& g, unsigned minContigLength)
 {
 	typedef graph_traits<Graph> GTraits;
 	typedef GTraits::edge_descriptor E;
@@ -193,7 +131,7 @@ static void filterGraph(Graph& g)
 	std::pair<Vit, Vit> urange = vertices(g);
 	for (Vit uit = urange.first; uit != urange.second; ++uit) {
 		V u = *uit;
-		if (g[u].length < opt::minContigLength)
+		if (g[u].length < minContigLength)
 			clear_vertex(u, g);
 		if (out_degree(u, g) == 0 && in_degree(u, g) == 0) {
 			remove_vertex(u, g);
@@ -211,6 +149,35 @@ static void filterGraph(Graph& g)
 		cerr << "Removed " << numRemovedE << " edges.\n";
 }
 
+/** Return true if the specified edge is a cycle. */
+static bool isCycle(Graph& g, graph_traits<Graph>::edge_descriptor e)
+{
+	return edge(target(e, g), source(e, g), g).second;
+}
+
+/** Remove simple cycles of length two from the graph. */
+static void removeCycles(Graph& g)
+{
+	typedef graph_traits<Graph>::edge_descriptor E;
+	typedef graph_traits<Graph>::edge_iterator Eit;
+
+	// Identify the cycles.
+	vector<E> cycles;
+	Eit eit, elast;
+	for (tie(eit, elast) = edges(g); eit != elast; ++eit) {
+		E e = *eit;
+		if (isCycle(g, e))
+			cycles.push_back(e);
+	}
+
+	/** Remove the cycles. */
+	remove_edges(g, cycles.begin(), cycles.end());
+	if (opt::verbose > 0) {
+		cerr << "Removed " << cycles.size() << " cyclic edges.\n";
+		printGraphStats(cerr, g);
+	}
+}
+
 /** Find edges in g0 that resolve forks in g.
  * For a pair of edges (u,v1) and (u,v2) in g, if exactly one of the
  * edges (v1,v2) or (v2,v1) exists in g0, add that edge to g.
@@ -243,14 +210,17 @@ static void resolveForks(Graph& g, const Graph& g0)
 				pair<E, bool> e21 = edge(v2, v1, g0);
 				if (e12.second && e21.second) {
 					if (opt::verbose > 1)
-						cerr << "cycle: " << v1 << ' ' << v2 << '\n';
+						cerr << "cycle: " << get(vertex_name, g, v1)
+							<< ' ' << get(vertex_name, g, v2) << '\n';
 				} else if (e12.second || e21.second) {
 					E e = e12.second ? e12.first : e21.first;
 					V v = source(e, g0), w = target(e, g0);
 					add_edge(v, w, g0[e], g);
 					numEdges++;
 					if (opt::verbose > 1)
-						cerr << u << " -> " << v << " -> " << w
+						cerr << get(vertex_name, g, u)
+							<< " -> " << get(vertex_name, g, v)
+							<< " -> " << get(vertex_name, g, w)
 							<< " [" << g0[e] << "]\n";
 				}
 			}
@@ -270,31 +240,15 @@ static void pruneTips(Graph& g)
 	typedef graph_traits<Graph>::vertex_descriptor V;
 
 	/** Identify the tips. */
-	vector<V> tips;
-	pruneTips(g, back_inserter(tips));
+	size_t n = 0;
+	pruneTips(g, CountingOutputIterator(n));
 
 	if (opt::verbose > 0) {
-		cerr << "Removed " << tips.size() << " tips.\n";
+		cerr << "Removed " << n << " tips.\n";
 		printGraphStats(cerr, g);
 	}
 }
 
-/** Return true if the specified vertex is a repeat. */
-static bool isRepeat(const Graph& g,
-		graph_traits<Graph>::vertex_descriptor t1,
-		graph_traits<Graph>::vertex_descriptor t2,
-		graph_traits<Graph>::vertex_descriptor u,
-		graph_traits<Graph>::vertex_descriptor v1,
-		graph_traits<Graph>::vertex_descriptor v2)
-{
-	assert(in_degree(u, g) == 2 && out_degree(u, g) == 2);
-	(void)u;
-	return out_degree(t1, g) == 2 && out_degree(t2, g) == 2
-		&& in_degree(v1, g) == 2 && in_degree(v2, g) == 2
-		&& edge(t1, v1, g).second && edge(t2, v2, g).second
-		&& !edge(t1, v2, g).second && !edge(t2, v1, g).second;
-}
-
 /** Remove repetitive vertices from this graph.
  * input: digraph g { t1->v1 t2->v2 t1->u t2->u u->v1 u->v2 }
  * operation: remove vertex u
@@ -303,58 +257,72 @@ static bool isRepeat(const Graph& g,
 static void removeRepeats(Graph& g)
 {
 	typedef graph_traits<Graph>::adjacency_iterator Ait;
-	typedef graph_traits<Graph>::vertex_iterator Vit;
+	typedef graph_traits<Graph>::edge_descriptor E;
 	typedef graph_traits<Graph>::vertex_descriptor V;
 
 	vector<V> repeats;
-	pair<Vit, Vit> urange = vertices(g);
-	for (Vit uit = urange.first; uit != urange.second; ++uit) {
-		V u = *uit;
-		if (in_degree(u, g) != 2 || out_degree(u, g) != 2)
-			continue;
-
+	vector<E> transitive;
+	find_transitive_edges(g, back_inserter(transitive));
+	for (vector<E>::const_iterator it = transitive.begin();
+			it != transitive.end(); ++it) {
+		// Iterate through the transitive edges, u->w1.
+		V u = source(*it, g), w1 = target(*it, g);
 		Ait vit, vlast;
-		tie(vit, vlast) = adjacent_vertices(u, g);
-		V v1 = *vit;
-		++vit;
-		V v2 = *vit;
-		++vit;
-		assert(vit == vlast);
-		assert(v1 != v2 && v1 != u && v2 != u);
-
-		Ait tit, tlast;
-		tie(tit, tlast) = adjacent_vertices(~u, g);
-		V t1 = ~*tit;
-		++tit;
-		V t2 = ~*tit;
-		++tit;
-		assert(tit == tlast);
-		assert(t1 != t2 && t1 != u && t2 != u);
-
-		if (isRepeat(g, t1, t2, u, v1, v2))
-			repeats.push_back(u);
-		else if (isRepeat(g, t1, t2, u, v2, v1))
-			repeats.push_back(u);
+		for (tie(vit, vlast) = adjacent_vertices(u, g);
+				vit != vlast; ++vit) {
+			V v = *vit;
+			assert(u != v); // no self loops
+			if (!edge(v, w1, g).second)
+				continue;
+			// u->w1 is a transitive edge spanning u->v->w1.
+			Ait wit, wlast;
+			for (tie(wit, wlast) = adjacent_vertices(v, g);
+					wit != wlast; ++wit) {
+				// For each edge v->w2, check that an edge
+				// w1->w2 or w2->w1 exists. If not, v is a repeat.
+				V w2 = *wit;
+				assert(v != w2); // no self loops
+				if (w1 != w2
+						&& !edge(w1, w2, g).second
+						&& !edge(w2, w1, g).second) {
+					repeats.push_back(v);
+					break;
+				}
+			}
+		}
 	}
 
-	/** Remove the repetitive vertices. */
+	sort(repeats.begin(), repeats.end());
+	repeats.erase(unique(repeats.begin(), repeats.end()),
+			repeats.end());
+	if (opt::verbose > 1) {
+		cerr << "Ambiguous:";
+		for (vector<V>::const_iterator it = repeats.begin();
+				it != repeats.end(); ++it)
+			cerr << ' ' << get(vertex_name, g, *it);
+		cerr << '\n';
+	}
+
+	// Remove the repetitive vertices.
+	unsigned numRemoved = 0;
 	for (vector<V>::const_iterator it = repeats.begin();
 			it != repeats.end(); ++it) {
 		V u = *it;
-		clear_vertex(u, g);
-		remove_vertex(u, g);
+		V uc = get(vertex_complement, g, u);
+		clear_out_edges(u, g);
+		if (it != repeats.begin() && it[-1] == uc) {
+			remove_vertex(u, g);
+			numRemoved++;
+		}
 	}
+
 	if (opt::verbose > 0) {
-		cerr << "Removed "
-			<< repeats.size() << " repetitive vertices.\n";
+		cerr << "Cleared "
+			<< repeats.size() << " ambiguous vertices.\n"
+			<< "Removed "
+			<< numRemoved << " ambiguous vertices.\n";
 		printGraphStats(cerr, g);
 	}
-	if (opt::verbose > 1) {
-		cerr << "Repeats: ";
-		copy(repeats.begin(), repeats.end(),
-				ostream_iterator<ContigNode>(cerr, " "));
-		cerr << '\n';
-	}
 }
 
 /** Remove weak edges from this graph.
@@ -423,8 +391,8 @@ static void removeWeakEdges(Graph& g)
 		for (vector<E>::const_iterator it = weak.begin();
 				it != weak.end(); ++it) {
 			E e = *it;
-			cerr << "\t\"" << source(e, g) << "\"->\"" << target(e, g)
-				<< "\" [" << g[e] << "]\n";
+			cerr << '\t' << get(edge_name, g, e)
+				<< " [" << g[e] << "]\n";
 		}
 	}
 
@@ -495,9 +463,167 @@ static void readGraph(const string& path, Graph& g)
 	assert(in.eof());
 	if (opt::verbose > 0)
 		printGraphStats(cerr, g);
-	ContigID::lock();
+	g_contigNames.lock();
 }
 
+/** Return the scaffold length of [first, last), not counting gaps. */
+template<typename It>
+unsigned addLength(const Graph& g, It first, It last)
+{
+	typedef typename graph_traits<Graph>::vertex_descriptor V;
+	assert(first != last);
+	unsigned length = g[*first].length;
+	for (It it = first + 1; it != last; ++it) {
+		V u = *(it - 1);
+		V v = *it;
+		length += min(0, get(edge_bundle, g, u, v).distance);
+		length += g[v].length;
+	}
+	return length;
+}
+
+/** A container of contig paths. */
+typedef vector<ContigPath> ContigPaths;
+
+/** Build the scaffold length histogram. */
+static Histogram buildScaffoldLengthHistogram(
+		Graph& g, const ContigPaths& paths)
+{
+	Histogram h;
+
+	// Clear the removed flag.
+	typedef graph_traits<Graph>::vertex_iterator Vit;
+	Vit uit, ulast;
+	for (tie(uit, ulast) = vertices(g); uit != ulast; ++uit)
+		put(vertex_removed, g, *uit, false);
+
+	// Remove the vertices that are used in paths
+	// and add the lengths of the scaffolds.
+	for (ContigPaths::const_iterator it = paths.begin();
+			it != paths.end(); ++it) {
+		h.insert(addLength(g, it->begin(), it->end()));
+		remove_vertex_if(g, it->begin(), it->end(),
+				not1(std::mem_fun_ref(&ContigNode::ambiguous)));
+	}
+
+	// Add the contigs that were not used in paths.
+	for (tie(uit, ulast) = vertices(g); uit != ulast; ++++uit) {
+		typedef graph_traits<Graph>::vertex_descriptor V;
+		V u = *uit;
+		if (!get(vertex_removed, g, u))
+			h.insert(g[u].length);
+	}
+
+	return h;
+}
+
+/** Build scaffold paths.
+ * @param output write the results
+ * @return the scaffold N50
+ */
+unsigned scaffold(const Graph& g0, unsigned minContigLength,
+		bool output)
+{
+	Graph g(g0);
+
+	// Filter the graph.
+	filterGraph(g, minContigLength);
+	if (opt::verbose > 0)
+		printGraphStats(cerr, g);
+
+	// Remove cycles.
+	removeCycles(g);
+
+	// Resolve forks.
+	resolveForks(g, g0);
+
+	// Prune tips.
+	pruneTips(g);
+
+	// Remove repeats.
+	removeRepeats(g);
+
+	// Remove transitive edges.
+	unsigned numTransitive = remove_transitive_edges(g);
+	if (opt::verbose > 0) {
+		cerr << "Removed " << numTransitive << " transitive edges.\n";
+		printGraphStats(cerr, g);
+	}
+
+	// Prune tips.
+	pruneTips(g);
+
+	// Pop bubbles.
+	typedef graph_traits<Graph>::vertex_descriptor V;
+	vector<V> popped = popBubbles(g);
+	if (opt::verbose > 0) {
+		cerr << "Removed " << popped.size()
+			<< " vertices in bubbles.\n";
+		printGraphStats(cerr, g);
+	}
+	if (opt::verbose > 1) {
+		cerr << "Popped:";
+		for (vector<V>::const_iterator it = popped.begin();
+				it != popped.end(); ++it)
+			cerr << ' ' << get(vertex_name, g, *it);
+		cerr << '\n';
+	}
+
+	// Remove weak edges.
+	removeWeakEdges(g);
+
+	// Assemble the paths.
+	ContigPaths paths;
+	assembleDFS(g, back_inserter(paths));
+	sort(paths.begin(), paths.end());
+	if (opt::verbose > 0) {
+		unsigned n = 0;
+		for (ContigPaths::const_iterator it = paths.begin();
+				it != paths.end(); ++it)
+			n += it->size();
+		cerr << "Assembled " << n << " contigs in "
+			<< paths.size() << " scaffolds.\n";
+		printGraphStats(cerr, g);
+	}
+
+	const unsigned STATS_MIN_LENGTH = opt::minContigLength;
+	if (!output) {
+		static bool printHeader = true;
+		Histogram h = buildScaffoldLengthHistogram(g, paths);
+		printContiguityStats(cerr, h, STATS_MIN_LENGTH,
+				printHeader)
+			<< "\ts=" << minContigLength << '\n';
+		if (opt::verbose == 0)
+			printHeader = false;
+		return h.trimLow(STATS_MIN_LENGTH).n50();
+	}
+
+	// Output the paths.
+	ofstream fout(opt::out.c_str());
+	ostream& out = opt::out.empty() || opt::out == "-" ? cout : fout;
+	assert_good(out, opt::out);
+	g_contigNames.unlock();
+	for (vector<ContigPath>::const_iterator it = paths.begin();
+			it != paths.end(); ++it)
+		out << createContigName() << '\t'
+			<< addDistEst(g0, g, *it) << '\n';
+	assert_good(out, opt::out);
+
+	// Output the graph.
+	if (!opt::graphPath.empty()) {
+		ofstream out(opt::graphPath.c_str());
+		assert_good(out, opt::graphPath);
+		write_dot(out, g);
+		assert_good(out, opt::graphPath);
+	}
+
+	// Print assembly contiguity statistics.
+	Histogram h = buildScaffoldLengthHistogram(g, paths);
+	printContiguityStats(cerr, h, STATS_MIN_LENGTH) << '\n';
+	return h.trimLow(STATS_MIN_LENGTH).n50();
+}
+
+/** Run abyss-scaffold. */
 int main(int argc, char** argv)
 {
 	bool die = false;
@@ -505,20 +631,47 @@ int main(int argc, char** argv)
 					shortopts, longopts, NULL)) != -1;) {
 		istringstream arg(optarg != NULL ? optarg : "");
 		switch (c) {
-			case '?': die = true; break;
-			case 'k': arg >> opt::k; break;
-			case 'g': arg >> opt::graphPath; break;
-			case 'n': arg >> opt::minNumPairs; break;
-			case 'o': arg >> opt::out; break;
-			case 's': arg >> opt::minContigLength; break;
-			case 'v': opt::verbose++; break;
-			case OPT_MIN_GAP: arg >> opt::minGap; break;
-			case OPT_HELP:
-				cout << USAGE_MESSAGE;
-				exit(EXIT_SUCCESS);
-			case OPT_VERSION:
-				cout << VERSION_MESSAGE;
-				exit(EXIT_SUCCESS);
+		  case '?':
+			die = true;
+			break;
+		  case 'k':
+			arg >> opt::k;
+			break;
+		  case 'g':
+			arg >> opt::graphPath;
+			break;
+		  case 'n':
+			arg >> opt::minNumPairs;
+			break;
+		  case 'o':
+			arg >> opt::out;
+			break;
+		  case 's':
+			arg >> opt::minContigLength;
+			if (arg.peek() == '-') {
+				opt::minContigLengthEnd = 100 * opt::minContigLength;
+				arg >> expect("-") >> opt::minContigLengthEnd;
+				assert(opt::minContigLength
+						<= opt::minContigLengthEnd);
+			}
+			break;
+		  case 'v':
+			opt::verbose++;
+			break;
+		  case OPT_MIN_GAP:
+			arg >> opt::minGap;
+			break;
+		  case OPT_HELP:
+			cout << USAGE_MESSAGE;
+			exit(EXIT_SUCCESS);
+		  case OPT_VERSION:
+			cout << VERSION_MESSAGE;
+			exit(EXIT_SUCCESS);
+		}
+		if (optarg != NULL && !arg.eof()) {
+			cerr << PROGRAM ": invalid option: `-"
+				<< (char)c << optarg << "'\n";
+			exit(EXIT_FAILURE);
 		}
 	}
 
@@ -546,85 +699,43 @@ int main(int argc, char** argv)
 		readGraph("-", g);
 
 	// Add any missing complementary edges.
-	addComplementaryEdges(g);
-	Graph g0(g);
-
-	// Filter the graph.
-	filterGraph(g);
-	if (opt::verbose > 0)
-		printGraphStats(cerr, g);
-
-	// Resolve forks.
-	resolveForks(g, g0);
-
-	// Prune tips.
-	pruneTips(g);
-
-	// Remove repeats.
-	removeRepeats(g);
-
-	// Remove transitive edges.
-	unsigned numTransitive = remove_transitive_edges(g);
+	size_t numAdded = addComplementaryEdges(g);
 	if (opt::verbose > 0) {
-		cerr << "Removed " << numTransitive << " transitive edges.\n";
+		cerr << "Added " << numAdded << " complementary edges.\n";
 		printGraphStats(cerr, g);
 	}
 
-	// Prune tips.
-	pruneTips(g);
-
-	// Pop bubbles.
-	vector<ContigNode> popped = popBubbles(g);
-	if (opt::verbose > 0) {
-		cerr << "Removed " << popped.size()
-			<< " vertices in bubbles.\n";
-		printGraphStats(cerr, g);
-	}
-	if (opt::verbose > 1) {
-		cerr << "Popped: ";
-		copy(popped.begin(), popped.end(),
-				ostream_iterator<ContigNode>(cerr, " "));
-		cerr << '\n';
+	if (opt::minContigLengthEnd == 0) {
+		scaffold(g, opt::minContigLength, true);
+		return 0;
 	}
 
-	// Remove weak edges.
-	removeWeakEdges(g);
-	Graph g1(g);
+	// Find the value of s that maximizes the scaffold N50.
+	unsigned bests = 0, bestN50 = 0;
+	const double STEP = cbrt(10); // Three steps per decade.
+	unsigned ilast = (unsigned)round(
+			log(opt::minContigLengthEnd) / log(STEP));
+	for (unsigned i = (unsigned)round(
+				log(opt::minContigLength) / log(STEP));
+			i <= ilast; ++i) {
+		unsigned s = (unsigned)pow(STEP, (int)i);
 
-	// Assemble the paths.
-	typedef vector<ContigPath> ContigPaths;
-	ContigPaths paths;
-	assemble(g, back_inserter(paths));
-	sort(paths.begin(), paths.end());
-	if (opt::verbose > 0) {
-		unsigned n = 0;
-		for (ContigPaths::const_iterator it = paths.begin();
-				it != paths.end(); ++it)
-			n += it->size();
-		cerr << "Assembled " << n << " contigs in "
-			<< paths.size() << " scaffolds.\n";
-		printGraphStats(cerr, g);
+		// Round to 1 figure.
+		double nearestDecade = pow(10, floor(log10(s)));
+		s = unsigned(round(s / nearestDecade) * nearestDecade);
+
+		unsigned n50 = scaffold(g, s, false);
+		if (opt::verbose > 0)
+			cerr << '\n';
+		if (n50 > bestN50) {
+			bestN50 = n50;
+			bests = s;
+		}
 	}
 
-	// Output the paths.
-	ofstream fout(opt::out.c_str());
-	ostream& out = opt::out.empty() || opt::out == "-" ? cout : fout;
-	assert_good(out, opt::out);
-	ContigID::unlock();
-	for (vector<ContigPath>::const_iterator it = paths.begin();
-			it != paths.end(); ++it) {
-		out << ContigID::create() << '\t'
-			<< addDistEst(g0, g1, *it) << '\n';
-	}
-	assert_good(out, opt::out);
-
-	// Output the graph.
-	if (!opt::graphPath.empty()) {
-		ofstream out(opt::graphPath.c_str());
-		assert_good(out, opt::graphPath);
-		write_dot(out, g);
-		assert_good(out, opt::graphPath);
-	}
+	bestN50 = scaffold(g, bests, true);
+	cerr << "Best scaffold N50 is " << bestN50
+		<< " at s=" << bests << ".\n";
 
 	return 0;
 }
