@@ -2,7 +2,6 @@
 #include "ContigPath.h"
 #include "Estimate.h"
 #include "IOUtil.h"
-#include "Iterator.h"
 #include "Uncompress.h"
 #include "Graph/ConstrainedSearch.h"
 #include "Graph/ContigGraph.h"
@@ -31,7 +30,7 @@ static const char VERSION_MESSAGE[] =
 PROGRAM " (" PACKAGE_NAME ") " VERSION "\n"
 "Written by Jared Simpson and Shaun Jackman.\n"
 "\n"
-"Copyright 2011 Canada's Michael Smith Genome Science Centre\n";
+"Copyright 2012 Canada's Michael Smith Genome Science Centre\n";
 
 static const char USAGE_MESSAGE[] =
 "Usage: " PROGRAM " [OPTION]... ADJ DIST\n"
@@ -45,6 +44,8 @@ static const char USAGE_MESSAGE[] =
 "      --max-cost=COST   maximum computational cost\n"
 "  -o, --out=FILE        write result to FILE\n"
 "  -j, --threads=THREADS use THREADS parallel threads [1]\n"
+"      --extend          extend unambiguous paths\n"
+"      --no-extend       do not extend unambiguous paths [default]\n"
 "      --scaffold        join contigs with Ns [default]\n"
 "      --no-scaffold     do not scaffold\n"
 "  -v, --verbose         display verbose output\n"
@@ -56,10 +57,14 @@ static const char USAGE_MESSAGE[] =
 namespace opt {
 	unsigned k; // used by ContigProperties
 	static unsigned threads = 1;
+	static int extend;
 	static int scaffold = 1;
 	static int verbose;
 	static string out;
 
+	/** The acceptable error of a distance estimate. */
+	unsigned distanceError = 6;
+
  	/** Output format */
  	int format = DIST; // used by Estimate
 }
@@ -73,6 +78,8 @@ static const struct option longopts[] = {
 	{ "dist-error",  required_argument, NULL, 'd' },
 	{ "max-cost",    required_argument, NULL, OPT_MAX_COST },
 	{ "out",         required_argument, NULL, 'o' },
+	{ "extend",      no_argument,       &opt::extend, 1 },
+	{ "no-extend",   no_argument,       &opt::extend, 0 },
 	{ "scaffold",    no_argument,       &opt::scaffold, 1 },
 	{ "no-scaffold", no_argument,       &opt::scaffold, 0 },
 	{ "threads",     required_argument,	NULL, 'j' },
@@ -106,6 +113,11 @@ int main(int argc, char** argv)
 				cout << VERSION_MESSAGE;
 				exit(EXIT_SUCCESS);
 		}
+		if (optarg != NULL && !arg.eof()) {
+			cerr << PROGRAM ": invalid option: `-"
+				<< (char)c << optarg << "'\n";
+			exit(EXIT_FAILURE);
+		}
 	}
 
 	if (opt::k <= 0) {
@@ -151,11 +163,14 @@ int main(int argc, char** argv)
 	generatePathsThroughEstimates(g, estFile);
 }
 
-static ostream& printConstraints(ostream& out, Constraints s)
+/** Print a set of constraints. */
+static ostream& printConstraints(ostream& out,
+		const Graph& g, const Constraints& s)
 {
 	for (Constraints::const_iterator it = s.begin();
 			it != s.end(); ++it)
-		out << ' ' << it->first << ',' << it->second;
+		out << ' ' << get(vertex_name, g, it->first)
+			<< ',' << it->second;
 	return out;
 }
 
@@ -172,7 +187,7 @@ static set<ContigID> findRepeats(ContigID seed,
 		count[seed]++;
 		for (ContigPath::const_iterator it = solIt->begin();
 				it != solIt->end(); ++it)
-			count[ContigID(*it)]++;
+			count[it->contigIndex()]++;
 		for (map<ContigID, unsigned>::const_iterator
 				it = count.begin(); it != count.end(); ++it)
 			if (it->second > 1)
@@ -371,10 +386,12 @@ static void printDistanceMap(ostream& out, const Graph& g,
 	DistanceMap distanceMap = makeDistanceMap(g, u, path);
 	for (DistanceMap::const_iterator it = distanceMap.begin();
 			it != distanceMap.end(); ++it)
-		out << '"' << u << "\" -> \"" << it->first
-			<< "\" [d=" << it->second << "]\n";
+		out << get(edge_name, g, make_pair(u, it->first))
+			<< " [d=" << it->second << "]\n";
 }
 
+typedef std::vector<std::pair<ContigNode, DistanceEst> > Estimates;
+
 /** Find a path for the specified distance estimates.
  * @param out [out] the solution path
  */
@@ -389,21 +406,23 @@ static void handleEstimate(const Graph& g,
 	ostringstream vout_ss;
 	ostream bitBucket(NULL);
 	ostream& vout = opt::verbose > 0 ? vout_ss : bitBucket;
-	vout << "\n* " << origin << '\n';
+	vout << "\n* " << get(vertex_name, g, origin) << '\n';
 
 	unsigned minNumPairs = UINT_MAX;
 	// generate the reachable set
 	Constraints constraints;
-	for (EstimateVector::const_iterator iter
+	for (Estimates::const_iterator iter
 				= er.estimates[dirIdx].begin();
 			iter != er.estimates[dirIdx].end(); ++iter) {
-		minNumPairs = min(minNumPairs, iter->numPairs);
-		constraints.push_back(Constraint(iter->contig,
-					iter->distance + allowedError(iter->stdDev)));
+		ContigNode v = iter->first;
+		const DistanceEst& ep = iter->second;
+		minNumPairs = min(minNumPairs, ep.numPairs);
+		constraints.push_back(Constraint(v,
+					ep.distance + allowedError(ep.stdDev)));
 	}
 
 	vout << "Constraints:";
-	printConstraints(vout, constraints) << '\n';
+	printConstraints(vout, g, constraints) << '\n';
 
 	ContigPaths solutions;
 	unsigned numVisited = 0;
@@ -414,8 +433,9 @@ static void handleEstimate(const Graph& g,
 	set<ContigID> repeats = findRepeats(er.refID, solutions);
 	if (!repeats.empty()) {
 		vout << "Repeats:";
-		copy(repeats.begin(), repeats.end(),
-				affix_ostream_iterator<ContigID>(vout, " "));
+		for (set<ContigID>::const_iterator it = repeats.begin();
+				it != repeats.end(); ++it)
+			vout << ' ' << get(g_contigNames, *it);
 		vout << '\n';
 	}
 
@@ -434,13 +454,15 @@ static void handleEstimate(const Graph& g,
 
 		// Remove solutions whose distance estimates are not correct.
 		unsigned validCount = 0, invalidCount = 0, ignoredCount = 0;
-		for (EstimateVector::const_iterator iter
+		for (Estimates::const_iterator iter
 					= er.estimates[dirIdx].begin();
 				iter != er.estimates[dirIdx].end(); ++iter) {
-			vout << *iter << '\t';
+			ContigNode v = iter->first;
+			const DistanceEst& ep = iter->second;
+			vout << get(vertex_name, g, v) << ',' << ep << '\t';
 
 			map<ContigNode, int>::iterator dmIter
-				= distanceMap.find(iter->contig);
+				= distanceMap.find(v);
 			if (dmIter == distanceMap.end()) {
 				// This contig is a repeat.
 				ignoredCount++;
@@ -451,10 +473,10 @@ static void handleEstimate(const Graph& g,
 			// translate distance by -overlap to match
 			// coordinate space used by the estimate
 			int actualDistance = dmIter->second;
-			int diff = actualDistance - iter->distance;
-			unsigned buffer = allowedError(iter->stdDev);
+			int diff = actualDistance - ep.distance;
+			unsigned buffer = allowedError(ep.stdDev);
 			bool invalid = (unsigned)abs(diff) > buffer;
-			bool repeat = repeats.count(ContigID(iter->contig)) > 0;
+			bool repeat = repeats.count(v.contigIndex()) > 0;
 			bool ignored = invalid && repeat;
 			if (ignored)
 				ignoredCount++;
@@ -465,7 +487,7 @@ static void handleEstimate(const Graph& g,
 			vout << "dist: " << actualDistance
 				<< " diff: " << diff
 				<< " buffer: " << buffer
-				<< " n: " << iter->numPairs
+				<< " n: " << ep.numPairs
 				<< (ignored ? " ignored" : invalid ? " invalid" : "")
 				<< '\n';
 		}
@@ -490,16 +512,18 @@ static void handleEstimate(const Graph& g,
 		map<ContigNode, int> distanceMap
 			= makeDistanceMap(g, origin, *solIter);
 		int sumDiff = 0;
-		for (EstimateVector::const_iterator iter
+		for (Estimates::const_iterator iter
 					= er.estimates[dirIdx].begin();
 				iter != er.estimates[dirIdx].end(); ++iter) {
-			if (repeats.count(ContigID(iter->contig)) > 0)
+			ContigNode v = iter->first;
+			const DistanceEst& ep = iter->second;
+			if (repeats.count(v.contigIndex()) > 0)
 				continue;
 			map<ContigNode, int>::iterator dmIter
-				= distanceMap.find(iter->contig);
+				= distanceMap.find(v);
 			assert(dmIter != distanceMap.end());
 			int actualDistance = dmIter->second;
-			int diff = actualDistance - iter->distance;
+			int diff = actualDistance - ep.distance;
 			sumDiff += abs(diff);
 		}
 
@@ -528,13 +552,14 @@ static void handleEstimate(const Graph& g,
 	} else if (solutions.empty()) {
 		stats.noValidPaths++;
 	} else if (repeats.count(er.refID) > 0) {
-		vout << "Repeat: " << ContigNode(er.refID, dirIdx) << '\n';
+		vout << "Repeat: " << get(vertex_name, g, origin) << '\n';
 		stats.repeat++;
 	} else if (solutions.size() > 1) {
 		ContigPath path
 			= constructAmbiguousPath(g, origin, solutions);
 		if (!path.empty()) {
-			extend(g, path.back(), back_inserter(path));
+			if (opt::extend)
+				extend(g, path.back(), back_inserter(path));
 			vout << path << '\n';
 			if (opt::scaffold) {
 				out.insert(out.end(), path.begin(), path.end());
@@ -549,8 +574,8 @@ static void handleEstimate(const Graph& g,
 		ContigPath& path = *bestSol;
 		if (opt::verbose > 1)
 			printDistanceMap(vout, g, origin, path);
-
-		extend(g, path.back(), back_inserter(path));
+		if (opt::extend)
+			extend(g, path.back(), back_inserter(path));
 		out.insert(out.end(), path.begin(), path.end());
 		stats.uniqueEnd++;
 		g_minNumPairsUsed = min(g_minNumPairsUsed, minNumPairs);
@@ -583,9 +608,9 @@ static void* worker(void* pArg)
 			break;
 
 		// Flip the anterior distance estimates.
-		for (EstimateVector::iterator it = er.estimates[1].begin();
+		for (Estimates::iterator it = er.estimates[1].begin();
 				it != er.estimates[1].end(); ++it)
-			it->contig.flip();
+			it->first ^= 1;
 
 		ContigPath path;
 		handleEstimate(*arg.graph, er, true, path);
@@ -597,7 +622,8 @@ static void* worker(void* pArg)
 			static pthread_mutex_t outMutex
 				= PTHREAD_MUTEX_INITIALIZER;
 			pthread_mutex_lock(&outMutex);
-			*arg.out << er.refID << '\t' << path << '\n';
+			*arg.out << get(g_contigNames, er.refID)
+				<< '\t' << path << '\n';
 			assert(arg.out->good());
 			pthread_mutex_unlock(&outMutex);
 		}
